@@ -27,6 +27,7 @@ import org.eclipse.cdt.core.errorparsers.AbstractErrorParser;
 import org.eclipse.cdt.core.errorparsers.ErrorPattern;
 import org.eclipse.cdt.core.testplugin.CTestPlugin;
 import org.eclipse.cdt.core.testplugin.ResourceHelper;
+import org.eclipse.cdt.internal.core.Cygwin;
 import org.eclipse.core.internal.registry.ExtensionRegistry;
 import org.eclipse.core.resources.IFile;
 import org.eclipse.core.resources.IFolder;
@@ -55,8 +56,10 @@ public class ErrorParserFileMatchingTest extends TestCase {
 
 	private final IMarkerGenerator markerGenerator = new IMarkerGenerator() {
 		// deprecated
+		@Override
 		public void addMarker(IResource file, int lineNumber, String errorDesc, int severity, String errorVar) {}
 
+		@Override
 		public void addMarker(ProblemMarkerInfo problemMarkerInfo) {
 			errorList.add(problemMarkerInfo);
 		}
@@ -901,7 +904,7 @@ public class ErrorParserFileMatchingTest extends TestCase {
 		String windowsFileName = fProject.getLocation().append(fileName).toOSString();
 		String cygwinFileName;
 		try {
-			cygwinFileName = ResourceHelper.windowsToCygwinPath(windowsFileName);
+			cygwinFileName = Cygwin.windowsToCygwinPath(windowsFileName);
 		} catch (UnsupportedOperationException e) {
 			// Skip the test if Cygwin is not available.
 			return;
@@ -929,7 +932,7 @@ public class ErrorParserFileMatchingTest extends TestCase {
 
 		String usrIncludeWindowsPath;
 		try {
-			usrIncludeWindowsPath = ResourceHelper.cygwinToWindowsPath(cygwinFolder);
+			usrIncludeWindowsPath = Cygwin.cygwinToWindowsPath(cygwinFolder);
 		} catch (UnsupportedOperationException e) {
 			// Skip the test if Cygwin is not available.
 			return;
@@ -959,7 +962,7 @@ public class ErrorParserFileMatchingTest extends TestCase {
 		String windowsFileName = anotherProject.getLocation().append(fileName).toOSString();
 		String cygwinFileName;
 		try {
-			cygwinFileName = ResourceHelper.windowsToCygwinPath(windowsFileName);
+			cygwinFileName = Cygwin.windowsToCygwinPath(windowsFileName);
 		} catch (UnsupportedOperationException e) {
 			// Skip the test if Cygwin is not available.
 			return;
@@ -1008,7 +1011,7 @@ public class ErrorParserFileMatchingTest extends TestCase {
 		String windowsFileName = fProject.getLocation().append(fileName).toOSString();
 		String cygwinFileName;
 		try {
-			cygwinFileName = ResourceHelper.windowsToCygwinPath(windowsFileName);
+			cygwinFileName = Cygwin.windowsToCygwinPath(windowsFileName);
 		} catch (UnsupportedOperationException e) {
 			// Skip the test if Cygwin is not available.
 			return;
@@ -1019,7 +1022,7 @@ public class ErrorParserFileMatchingTest extends TestCase {
 
 		String lines = "make[0]: Entering directory `dir'\n"
 			+ cygwinFileName+":1:error\n";
-		
+
 		String[] errorParsers = {CWD_LOCATOR_ID, mockErrorParserId };
 		parseOutput(fProject, fProject.getLocation(), errorParsers, lines);
 		assertEquals(1, errorList.size());
@@ -1202,6 +1205,36 @@ public class ErrorParserFileMatchingTest extends TestCase {
 		assertEquals("error",problemMarkerInfo.description);
 	}
 
+	/**
+	 * Checks if output of -w or --print-directory options of make can be recognized.
+	 *
+	 * @throws Exception...
+	 */
+	public void testPushPop_WithNoLevel() throws Exception {
+		String fileName = getName() + ".c";
+
+		ResourceHelper.createFolder(fProject, "Folder");
+		ResourceHelper.createFolder(fProject, "Folder/SubFolder");
+
+		ResourceHelper.createFile(fProject, fileName);
+		ResourceHelper.createFile(fProject, "Folder/"+fileName);
+		ResourceHelper.createFile(fProject, "Folder/SubFolder/"+fileName);
+
+		String lines = "make: Entering directory `Folder'\n"
+				+ "make: Entering directory `SubFolder'\n"
+				+ "make: Leaving directory `SubFolder'\n"
+				+ fileName+":1:error\n";
+
+		String[] errorParsers = {CWD_LOCATOR_ID, mockErrorParserId };
+		parseOutput(fProject, fProject.getLocation(), errorParsers, lines);
+		assertEquals(1, errorList.size());
+
+		ProblemMarkerInfo problemMarkerInfo = errorList.get(0);
+		assertEquals("L/FindMatchingFilesTest/Folder/"+fileName,problemMarkerInfo.file.toString());
+		assertEquals(1,problemMarkerInfo.lineNumber);
+		assertEquals("error",problemMarkerInfo.description);
+	}
+
 	/**
 	 * Checks if a file from error output can be found.
 	 *
@@ -1271,21 +1304,21 @@ public class ErrorParserFileMatchingTest extends TestCase {
 		ResourceHelper.createFolder(fProject, "Folder");
 		ResourceHelper.createFile(fProject, fileName);
 		ResourceHelper.createFile(fProject, "Folder/"+fileName);
-		
+
 		String lines = "make -j2\n"
 				+ "make[0]: Entering directory `Folder'\n"
 				+ fileName+":1:error\n";
-		
+
 		String[] errorParsers = {CWD_LOCATOR_ID, mockErrorParserId };
 		parseOutput(fProject, fProject.getLocation(), errorParsers, lines);
 		assertEquals(1, errorList.size());
-		
+
 		ProblemMarkerInfo problemMarkerInfo = errorList.get(0);
 		assertEquals("L/FindMatchingFilesTest/"+fileName,problemMarkerInfo.file.toString());
 		assertEquals(1,problemMarkerInfo.lineNumber);
 		assertEquals("error",problemMarkerInfo.description);
 	}
-	
+
 	/**
 	 * Checks if a file from error output can be found.
 	 *
@@ -1296,21 +1329,21 @@ public class ErrorParserFileMatchingTest extends TestCase {
 		ResourceHelper.createFolder(fProject, "Folder");
 		ResourceHelper.createFile(fProject, fileName);
 		ResourceHelper.createFile(fProject, "Folder/"+fileName);
-		
+
 		String lines = "make -j  2\n"
 				+ "make[0]: Entering directory `Folder'\n"
 				+ fileName+":1:error\n";
-		
+
 		String[] errorParsers = {CWD_LOCATOR_ID, mockErrorParserId };
 		parseOutput(fProject, fProject.getLocation(), errorParsers, lines);
 		assertEquals(1, errorList.size());
-		
+
 		ProblemMarkerInfo problemMarkerInfo = errorList.get(0);
 		assertEquals("L/FindMatchingFilesTest/"+fileName,problemMarkerInfo.file.toString());
 		assertEquals(1,problemMarkerInfo.lineNumber);
 		assertEquals("error",problemMarkerInfo.description);
 	}
-	
+
 	/**
 	 * Checks if a file from error output can be found.
 	 *
@@ -1321,21 +1354,21 @@ public class ErrorParserFileMatchingTest extends TestCase {
 		ResourceHelper.createFolder(fProject, "Folder");
 		ResourceHelper.createFile(fProject, fileName);
 		ResourceHelper.createFile(fProject, "Folder/"+fileName);
-		
+
 		String lines = "make -j1\n"
 				+ "make[0]: Entering directory `Folder'\n"
 				+ fileName+":1:error\n";
-		
+
 		String[] errorParsers = {CWD_LOCATOR_ID, mockErrorParserId };
 		parseOutput(fProject, fProject.getLocation(), errorParsers, lines);
 		assertEquals(1, errorList.size());
-		
+
 		ProblemMarkerInfo problemMarkerInfo = errorList.get(0);
 		assertEquals("L/FindMatchingFilesTest/Folder/"+fileName,problemMarkerInfo.file.toString());
 		assertEquals(1,problemMarkerInfo.lineNumber);
 		assertEquals("error",problemMarkerInfo.description);
 	}
-	
+
 	/**
 	 * Checks if a file from error output can be found.
 	 *
@@ -1346,21 +1379,21 @@ public class ErrorParserFileMatchingTest extends TestCase {
 		ResourceHelper.createFolder(fProject, "Folder");
 		ResourceHelper.createFile(fProject, fileName);
 		ResourceHelper.createFile(fProject, "Folder/"+fileName);
-		
+
 		String lines = "make -j  1\n"
 				+ "make[0]: Entering directory `Folder'\n"
 				+ fileName+":1:error\n";
-		
+
 		String[] errorParsers = {CWD_LOCATOR_ID, mockErrorParserId };
 		parseOutput(fProject, fProject.getLocation(), errorParsers, lines);
 		assertEquals(1, errorList.size());
-		
+
 		ProblemMarkerInfo problemMarkerInfo = errorList.get(0);
 		assertEquals("L/FindMatchingFilesTest/Folder/"+fileName,problemMarkerInfo.file.toString());
 		assertEquals(1,problemMarkerInfo.lineNumber);
 		assertEquals("error",problemMarkerInfo.description);
 	}
-	
+
 	/**
 	 * Checks if a file from error output can be found.
 	 *
@@ -1371,21 +1404,21 @@ public class ErrorParserFileMatchingTest extends TestCase {
 		ResourceHelper.createFolder(fProject, "Folder");
 		ResourceHelper.createFile(fProject, fileName);
 		ResourceHelper.createFile(fProject, "Folder/"+fileName);
-		
+
 		String lines = "make --jobs=2\n"
 				+ "make[0]: Entering directory `Folder'\n"
 				+ fileName+":1:error\n";
-		
+
 		String[] errorParsers = {CWD_LOCATOR_ID, mockErrorParserId };
 		parseOutput(fProject, fProject.getLocation(), errorParsers, lines);
 		assertEquals(1, errorList.size());
-		
+
 		ProblemMarkerInfo problemMarkerInfo = errorList.get(0);
 		assertEquals("L/FindMatchingFilesTest/"+fileName,problemMarkerInfo.file.toString());
 		assertEquals(1,problemMarkerInfo.lineNumber);
 		assertEquals("error",problemMarkerInfo.description);
 	}
-	
+
 	/**
 	 * Checks if a file from error output can be found.
 	 *
@@ -1396,21 +1429,21 @@ public class ErrorParserFileMatchingTest extends TestCase {
 		ResourceHelper.createFolder(fProject, "Folder");
 		ResourceHelper.createFile(fProject, fileName);
 		ResourceHelper.createFile(fProject, "Folder/"+fileName);
-		
+
 		String lines = "make --jobs=1\n"
 				+ "make[0]: Entering directory `Folder'\n"
 				+ fileName+":1:error\n";
-		
+
 		String[] errorParsers = {CWD_LOCATOR_ID, mockErrorParserId };
 		parseOutput(fProject, fProject.getLocation(), errorParsers, lines);
 		assertEquals(1, errorList.size());
-		
+
 		ProblemMarkerInfo problemMarkerInfo = errorList.get(0);
 		assertEquals("L/FindMatchingFilesTest/Folder/"+fileName,problemMarkerInfo.file.toString());
 		assertEquals(1,problemMarkerInfo.lineNumber);
 		assertEquals("error",problemMarkerInfo.description);
 	}
-	
+
 	/**
 	 * Checks if a file from error output can be found.
 	 *
@@ -1421,21 +1454,21 @@ public class ErrorParserFileMatchingTest extends TestCase {
 		ResourceHelper.createFolder(fProject, "Folder");
 		ResourceHelper.createFile(fProject, fileName);
 		ResourceHelper.createFile(fProject, "Folder/"+fileName);
-		
+
 		String lines = "gmake384 -k -j all\n"
 				+ "make[0]: Entering directory `Folder'\n"
 				+ fileName+":1:error\n";
-		
+
 		String[] errorParsers = {CWD_LOCATOR_ID, mockErrorParserId };
 		parseOutput(fProject, fProject.getLocation(), errorParsers, lines);
 		assertEquals(1, errorList.size());
-		
+
 		ProblemMarkerInfo problemMarkerInfo = errorList.get(0);
 		assertEquals("L/FindMatchingFilesTest/"+fileName,problemMarkerInfo.file.toString());
 		assertEquals(1,problemMarkerInfo.lineNumber);
 		assertEquals("error",problemMarkerInfo.description);
 	}
-	
+
 	/**
 	 * Checks if a file from error output can be found.
 	 *
@@ -1445,10 +1478,10 @@ public class ErrorParserFileMatchingTest extends TestCase {
 		ResourceHelper.createFolder(fProject, "Folder");
 		ResourceHelper.createFolder(fProject, "Folder/AbsoluteRemoteFolder");
 		IFile file = ResourceHelper.createFile(fProject, "Folder/AbsoluteRemoteFolder/testMappedRemoteAbsolutePath.h");
-		
+
 		parseOutput("/AbsoluteRemoteFolder/testMappedRemoteAbsolutePath.h:1:error");
 		assertEquals(1, errorList.size());
-		
+
 		ProblemMarkerInfo problemMarkerInfo = errorList.get(0);
 		assertEquals("L/FindMatchingFilesTest/Folder/AbsoluteRemoteFolder/testMappedRemoteAbsolutePath.h",problemMarkerInfo.file.toString());
 		assertEquals(1,problemMarkerInfo.lineNumber);
@@ -1467,16 +1500,16 @@ public class ErrorParserFileMatchingTest extends TestCase {
 		ResourceHelper.createFolder(anotherProject, "Folder");
 		ResourceHelper.createFolder(anotherProject, "Folder/AbsoluteRemoteFolder");
 		IFile file = ResourceHelper.createFile(anotherProject, "Folder/AbsoluteRemoteFolder/testMappedRemoteAbsolutePathAnotherProject.h");
-		
+
 		parseOutput("/AbsoluteRemoteFolder/testMappedRemoteAbsolutePathAnotherProject.h:1:error");
 		assertEquals(1, errorList.size());
-		
+
 		ProblemMarkerInfo problemMarkerInfo = errorList.get(0);
 		assertEquals("L/ProjectMappedRemoteAbsolutePathAnotherProject/Folder/AbsoluteRemoteFolder/testMappedRemoteAbsolutePathAnotherProject.h",problemMarkerInfo.file.toString());
 		assertEquals(1,problemMarkerInfo.lineNumber);
 		assertEquals("error",problemMarkerInfo.description);
 	}
-	
+
 	/**
 	 * Checks if a file from error output can be found.
 	 *
