@@ -61,6 +61,7 @@ public abstract class IndexCPPBindingResolutionTest extends IndexBindingResoluti
 		public SingleProject() {setStrategy(new SinglePDOMTestStrategy(true));}
 		public static TestSuite suite() {return suite(SingleProject.class);}
 	}
+
 	public static class ProjectWithDepProj extends IndexCPPBindingResolutionTest {
 		public ProjectWithDepProj() {setStrategy(new ReferencedProject(true));}
 		public static TestSuite suite() {return suite(ProjectWithDepProj.class);}
@@ -1584,6 +1585,35 @@ public abstract class IndexCPPBindingResolutionTest extends IndexBindingResoluti
 		getBindingFromASTName("g(1)", 1);
 	}
 
+	//	namespace ns {
+	//		void fun();
+	//	}
+
+	//	namespace alias = ns;
+	//	void alias::fun() {
+	//	}
+	public void testNamespaceAliasAsQualifier_356493() throws Exception {
+		IFunction ref= getBindingFromASTName("fun", 0);
+		assertEquals("ns", ref.getOwner().getName());
+	}
+
+	//	class A {};
+	//	void f(A a) {}
+	//	struct B {};
+	//	void g(B b) {}
+
+	//	struct A;
+	//	class B;
+	//
+	//	void test(A a, B b) {
+	//	  f(a);
+	//	  g(b);
+	//	}
+	public void testStructClassMismatch_358282() throws Exception {
+		getBindingFromASTName("f(a)", 1, ICPPFunction.class);
+		getBindingFromASTName("g(b)", 1, ICPPFunction.class);
+	}
+	
 	/* CPP assertion helpers */
 	/* ##################################################################### */
 
