@@ -6,7 +6,8 @@
  * http://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors:
- *    Markus Schorn - initial API and implementation
+ *     Markus Schorn - initial API and implementation
+ *     Sergey Prigogin (Google)
  *******************************************************************************/ 
 package org.eclipse.cdt.internal.index.tests;
 
@@ -48,8 +49,8 @@ public class IndexIncludeTest extends IndexTestBase {
 		return suite;
 	}
 
-	private ICProject fProject= null;
-	private IIndex fIndex= null;
+	private ICProject fProject;
+	private IIndex fIndex;
 	
 	public IndexIncludeTest(String name) {
 		super(name);
@@ -61,9 +62,10 @@ public class IndexIncludeTest extends IndexTestBase {
 		if (fProject == null) {
 			fProject= createProject(true, "resources/indexTests/includes");
 			IPathEntry[] entries= new IPathEntry[] {
-					CoreModel.newIncludeEntry(fProject.getPath(),
-							null, fProject.getResource().getLocation())};
+					CoreModel.newIncludeEntry(fProject.getPath(), null,
+							fProject.getResource().getLocation()) };
 			fProject.setRawPathEntries(entries, npm());
+			IndexerPreferences.set(fProject.getProject(), IndexerPreferences.KEY_INDEX_UNUSED_HEADERS_WITH_DEFAULT_LANG, "false");
 		}
 		fIndex= CCorePlugin.getIndexManager().getIndex(fProject);
 	}
@@ -113,18 +115,18 @@ public class IndexIncludeTest extends IndexTestBase {
 		waitForIndexer();
 		
 		ResourcesPlugin.getWorkspace().run(new IWorkspaceRunnable() {
+			@Override
 			public void run(IProgressMonitor monitor) throws CoreException {
-				file.setContents(new ByteArrayInputStream( "int included; int CONTEXT;\n".getBytes()), false, false, npm());
-				file.setLocalTimeStamp(timestamp+1000); 
+				file.setContents(new ByteArrayInputStream("int included; int CONTEXT;\n".getBytes()), false, false, npm());
+				file.setLocalTimeStamp(timestamp + 1000); 
 			}
 		}, npm());
 		assertTrue("Timestamp was not increased", file.getLocalTimeStamp() >= timestamp);
 		TestSourceReader.waitUntilFileIsIndexed(fIndex, file, 4000);
 		fIndex.acquireReadLock();
 		try {
-			IIndexFile ifile= fIndex.getFile(ILinkage.CPP_LINKAGE_ID, IndexLocationFactory.getWorkspaceIFL(file));
-			assertNotNull("Can't find " + file.getLocation(), ifile);
-			assertTrue("timestamp not ok", ifile.getTimestamp() >= timestamp);
+			IIndexFile ifile= getIndexFile(file);
+			assertTrue("Timestamp not ok", ifile.getTimestamp() >= timestamp);
 
 			IIndexBinding[] result= fIndex.findBindings(Pattern.compile("testInclude_cpp"), true, IndexFilter.ALL, npm());
 			assertEquals(1, result.length);
@@ -134,6 +136,13 @@ public class IndexIncludeTest extends IndexTestBase {
 		} finally {
 			fIndex.releaseReadLock();
 		}
+	}
+
+	private IIndexFile getIndexFile(IFile file) throws CoreException {
+		IIndexFile[] files = fIndex.getFiles(ILinkage.CPP_LINKAGE_ID, IndexLocationFactory.getWorkspaceIFL(file));
+		assertTrue("Can't find " + file.getLocation(), files.length > 0);
+		assertEquals("Found " + files.length + " files for " + file.getLocation() + " instead of one", 1, files.length);
+		return files[0];
 	}			
 
 	// {source20061107}
@@ -141,7 +150,7 @@ public class IndexIncludeTest extends IndexTestBase {
 	// #include <system20061107.h>
 	public void testIncludeProperties() throws Exception {
 		waitForIndexer();
-		TestScannerProvider.sIncludes= new String[]{fProject.getProject().getLocation().toOSString()};
+		TestScannerProvider.sIncludes= new String[] { fProject.getProject().getLocation().toOSString() };
 		String content= readTaggedComment("source20061107");
 		TestSourceReader.createFile(fProject.getProject(), "user20061107.h", "");
 		TestSourceReader.createFile(fProject.getProject(), "system20061107.h", "");
@@ -150,8 +159,7 @@ public class IndexIncludeTest extends IndexTestBase {
 
 		fIndex.acquireReadLock();
 		try {
-			IIndexFile ifile= fIndex.getFile(ILinkage.CPP_LINKAGE_ID, IndexLocationFactory.getWorkspaceIFL(file));
-			assertNotNull(ifile);
+			IIndexFile ifile= getIndexFile(file);
 			IIndexInclude[] includes= ifile.getIncludes();
 			assertEquals(2, includes.length);
 
@@ -163,7 +171,7 @@ public class IndexIncludeTest extends IndexTestBase {
 	}
 	
 	public void testIncludeProperties_2() throws Exception {
-		TestScannerProvider.sIncludes= new String[]{fProject.getProject().getLocation().toOSString()};
+		TestScannerProvider.sIncludes= new String[] { fProject.getProject().getLocation().toOSString() };
 		TestSourceReader.createFile(fProject.getProject(), "header20061107.h", "");
 		String content = "// comment \n#include \"header20061107.h\"\n";
 		IFile file= TestSourceReader.createFile(fProject.getProject(), "intermed20061107.h", content);
@@ -174,8 +182,7 @@ public class IndexIncludeTest extends IndexTestBase {
 
 		fIndex.acquireReadLock();
 		try {
-			IIndexFile ifile= fIndex.getFile(ILinkage.CPP_LINKAGE_ID, IndexLocationFactory.getWorkspaceIFL(file));
-			assertNotNull(ifile);
+			IIndexFile ifile= getIndexFile(file);
 			IIndexInclude[] includes= ifile.getIncludes();
 			assertEquals(1, includes.length);
 
@@ -186,7 +193,7 @@ public class IndexIncludeTest extends IndexTestBase {
 	}
 
 	public void testInactiveInclude() throws Exception {
-		TestScannerProvider.sIncludes= new String[]{fProject.getProject().getLocation().toOSString()};
+		TestScannerProvider.sIncludes= new String[] { fProject.getProject().getLocation().toOSString() };
 		String content = "#if 0\n#include \"inactive20070213.h\"\n#endif\n";
 		IFile file= TestSourceReader.createFile(fProject.getProject(), "source20070213.cpp", content);
 		CCorePlugin.getIndexManager().reindex(fProject);
@@ -194,8 +201,7 @@ public class IndexIncludeTest extends IndexTestBase {
 
 		fIndex.acquireReadLock();
 		try {
-			IIndexFile ifile= fIndex.getFile(ILinkage.CPP_LINKAGE_ID, IndexLocationFactory.getWorkspaceIFL(file));
-			assertNotNull(ifile);
+			IIndexFile ifile= getIndexFile(file);
 			IIndexInclude[] includes= ifile.getIncludes();
 			assertEquals(1, includes.length);
 
@@ -207,7 +213,7 @@ public class IndexIncludeTest extends IndexTestBase {
 	}
 
 	public void testUnresolvedInclude() throws Exception {
-		TestScannerProvider.sIncludes= new String[]{fProject.getProject().getLocation().toOSString()};
+		TestScannerProvider.sIncludes= new String[] { fProject.getProject().getLocation().toOSString() };
 		String content = "#include \"unresolved20070213.h\"\n";
 		IFile file= TestSourceReader.createFile(fProject.getProject(), "source20070214.cpp", content);
 		CCorePlugin.getIndexManager().reindex(fProject);
@@ -215,8 +221,7 @@ public class IndexIncludeTest extends IndexTestBase {
 
 		fIndex.acquireReadLock();
 		try {
-			IIndexFile ifile= fIndex.getFile(ILinkage.CPP_LINKAGE_ID, IndexLocationFactory.getWorkspaceIFL(file));
-			assertNotNull(ifile);
+			IIndexFile ifile= getIndexFile(file);
 			IIndexInclude[] includes= ifile.getIncludes();
 			assertEquals(1, includes.length);
 
@@ -238,7 +243,10 @@ public class IndexIncludeTest extends IndexTestBase {
 	public void testUpdateOfIncluded() throws Exception {
 		String content1 = "int CONTEXT_20070404(x);\n";
 		String content2 = "int CONTEXT_20070404(y);\n";
-		String content3 = "#define CONTEXT_20070404(x) ctx_20070404##x\n #include \"included_20070404.h\"\n int source_20070404;\n";
+		String content3 =
+				"#define CONTEXT_20070404(x) ctx_20070404##x\n" +
+				"#include \"included_20070404.h\"\n" +
+				"int source_20070404;\n";
 		TestSourceReader.createFile(fProject.getProject(), "included_20070404.h", content1);
 		TestSourceReader.createFile(fProject.getProject(), "notIncluded_20070404.h", "int notIncluded_20070404\n;");
 		TestSourceReader.createFile(fProject.getProject(), "includer_20070404.cpp", content3);
@@ -285,7 +293,7 @@ public class IndexIncludeTest extends IndexTestBase {
 	// #include "header1.h"
 	// #include "header2.h"
 	public void testParsingInContext_bug220358() throws Exception {
-		StringBuffer[] sources= getContentsForTest(4);
+		CharSequence[] sources= getContentsForTest(4);
 		IFile h1= TestSourceReader.createFile(fProject.getProject(), "header1.h", sources[0].toString());
 		IFile h2= TestSourceReader.createFile(fProject.getProject(), "header2.h", sources[1].toString());
 		IFile s1= TestSourceReader.createFile(fProject.getProject(), "s1.cpp", sources[3].toString());
@@ -318,7 +326,7 @@ public class IndexIncludeTest extends IndexTestBase {
 	
 	// #include "resolved20070426.h"
 	public void testFixedContext() throws Exception {
-		TestScannerProvider.sIncludes= new String[]{fProject.getProject().getLocation().toOSString()};
+		TestScannerProvider.sIncludes= new String[] { fProject.getProject().getLocation().toOSString() };
 		String source= getContentsForTest(1)[0].toString();
 		IFile header= TestSourceReader.createFile(fProject.getProject(), "resolved20070426.h", "");
 		IFile s1= TestSourceReader.createFile(fProject.getProject(), "s1.cpp", source);
@@ -332,8 +340,7 @@ public class IndexIncludeTest extends IndexTestBase {
 
 		fIndex.acquireReadLock();
 		try {
-			IIndexFile ifile= fIndex.getFile(ILinkage.CPP_LINKAGE_ID, IndexLocationFactory.getWorkspaceIFL(header));
-			assertNotNull(ifile);
+			IIndexFile ifile= getIndexFile(header);
 			IIndexInclude[] includes= fIndex.findIncludedBy(ifile);
 			assertEquals(2, includes.length);
 			
@@ -352,8 +359,7 @@ public class IndexIncludeTest extends IndexTestBase {
 		fIndex.acquireReadLock();
 		try {
 			assertEquals(1, fIndex.findBindings("a20070426".toCharArray(), IndexFilter.ALL_DECLARED, npm()).length);
-			IIndexFile ifile= fIndex.getFile(ILinkage.CPP_LINKAGE_ID, IndexLocationFactory.getWorkspaceIFL(header));
-			assertNotNull(ifile);
+			IIndexFile ifile= getIndexFile(header);
 			IIndexInclude[] includes= fIndex.findIncludedBy(ifile);
 			assertEquals(2, includes.length);
 			assertEquals(s1.getFullPath().toString(), includes[0].getIncludedByLocation().getFullPath());
@@ -370,8 +376,7 @@ public class IndexIncludeTest extends IndexTestBase {
 		fIndex.acquireReadLock();
 		try {
 			assertEquals(1, fIndex.findBindings("b20070426".toCharArray(), IndexFilter.ALL_DECLARED, npm()).length);
-			IIndexFile ifile= fIndex.getFile(ILinkage.CPP_LINKAGE_ID, IndexLocationFactory.getWorkspaceIFL(header));
-			assertNotNull(ifile);
+			IIndexFile ifile= getIndexFile(header);
 			IIndexInclude[] includes= fIndex.findIncludedBy(ifile);
 			assertEquals(2, includes.length);
 			assertEquals(s1.getFullPath().toString(), includes[0].getIncludedByLocation().getFullPath());
@@ -404,8 +409,8 @@ public class IndexIncludeTest extends IndexTestBase {
 	// #include "unesolved20070427.h"
 	public void testUpdateIncludes() throws Exception {
 		waitForIndexer();
-		TestScannerProvider.sIncludes= new String[]{fProject.getProject().getLocation().toOSString()};
-		StringBuffer[] source= getContentsForTest(4);
+		TestScannerProvider.sIncludes= new String[] { fProject.getProject().getLocation().toOSString() };
+		CharSequence[] source= getContentsForTest(4);
 		IFile header= TestSourceReader.createFile(fProject.getProject(), "resolved20070427.h", "");
 		IFile s1= TestSourceReader.createFile(fProject.getProject(), "s20070427.cpp", 
 				source[0].toString() + "\nint a20070427;");
@@ -446,7 +451,184 @@ public class IndexIncludeTest extends IndexTestBase {
 				source[0].toString() + "\nint h20070427;");
 		TestSourceReader.waitUntilFileIsIndexed(fIndex, s1, INDEXER_WAIT_TIME);
 		standardCheckUpdateIncludes(header, s1, "h20070427");
+	}
 
+	// #ifdef A
+	// static const int a = 0;
+	// #endif
+	// #ifdef B
+	// static const int b = 0;
+	// #endif
+	// #ifdef C
+	// static const int c = 0;
+	// #endif
+
+	// #define A
+	// #include "h1.h"
+	// #undef A
+	// #define B
+	// #include "h1.h"
+	// #undef B
+
+	// #define C
+	// #include "h1.h"
+
+	// #include "h2.h"
+	public void testMultiVariantHeaderUpdate() throws Exception {
+		waitForIndexer();
+		TestScannerProvider.sIncludes= new String[] { fProject.getProject().getLocation().toOSString() };
+		StringBuilder[] contents= getContentsForTest(4);
+		final StringBuilder h1Contents = contents[0];
+		final IFile h1= TestSourceReader.createFile(fProject.getProject(), "h1.h", h1Contents.toString());
+		IFile h2= TestSourceReader.createFile(fProject.getProject(), "h2.h", contents[1].toString());
+		IFile s1= TestSourceReader.createFile(fProject.getProject(), "s1.cpp", contents[2].toString());
+		IFile s2= TestSourceReader.createFile(fProject.getProject(), "s2.cpp", contents[3].toString());
+		TestSourceReader.waitUntilFileIsIndexed(fIndex, s1, INDEXER_WAIT_TIME);
+		TestSourceReader.waitUntilFileIsIndexed(fIndex, s2, INDEXER_WAIT_TIME);
+
+		fIndex.acquireReadLock();
+		try {
+			IIndexFile[] indexFiles = fIndex.getFiles(ILinkage.CPP_LINKAGE_ID, IndexLocationFactory.getWorkspaceIFL(h1));
+			assertEquals(3, indexFiles.length);
+		} finally {
+			fIndex.releaseReadLock();
+		}
+
+		final long timestamp= System.currentTimeMillis();
+		while (true) {
+			int pos = h1Contents.indexOf("int");
+			if (pos < 0)
+				break;
+			h1Contents.replace(pos, pos + "int".length(), "float");
+		}
+		ResourcesPlugin.getWorkspace().run(new IWorkspaceRunnable() {
+			@Override
+			public void run(IProgressMonitor monitor) throws CoreException {
+				h1.setContents(new ByteArrayInputStream(h1Contents.toString().getBytes()), false, false, npm());
+				h1.setLocalTimeStamp(timestamp + 1000); 
+			}
+		}, npm());
+		waitForIndexer();
+
+		fIndex.acquireReadLock();
+		try {
+			IIndexFile[] indexFiles = fIndex.getFiles(ILinkage.CPP_LINKAGE_ID, IndexLocationFactory.getWorkspaceIFL(h1));
+			assertEquals(3, indexFiles.length);
+			for (IIndexFile indexFile : indexFiles) {
+				assertTrue("Timestamp not ok", indexFile.getTimestamp() >= timestamp);
+			}
+		} finally {
+			fIndex.releaseReadLock();
+		}
+	}
+
+	// #ifdef A
+	// static const int a = 0;
+	// #endif
+	// #ifdef B
+	// static const int b = 0;
+	// #endif
+	// #ifdef C
+	// static const int c = 0;
+	// #endif
+
+	// #define A
+	// #include "h1.h"
+	// #undef A
+	// #define B
+	// #include "h1.h"
+	// #undef B
+
+	// #define C
+	// #include "h1.h"
+
+	// #include "h2.h"
+
+	// #ifndef H1_H_
+	// #define H1_H_
+	// #ifdef A
+	// static const int a = 0;
+	// #endif
+	// #ifdef B
+	// static const int b = 0;
+	// #endif
+	// #ifdef C
+	// static const int c = 0;
+	// #endif
+	// #endif // H1_H_
+	public void testPragmaOnceChange() throws Exception {
+		waitForIndexer();
+		TestScannerProvider.sIncludes= new String[] { fProject.getProject().getLocation().toOSString() };
+		CharSequence[] contents= getContentsForTest(5);
+		final CharSequence h1Contents = contents[0];
+		final IFile h1= TestSourceReader.createFile(fProject.getProject(), "h1.h", h1Contents.toString());
+		IFile h2= TestSourceReader.createFile(fProject.getProject(), "h2.h", contents[1].toString());
+		IFile s1= TestSourceReader.createFile(fProject.getProject(), "s1.cpp", contents[2].toString());
+		IFile s2= TestSourceReader.createFile(fProject.getProject(), "s2.cpp", contents[3].toString());
+		TestSourceReader.waitUntilFileIsIndexed(fIndex, s1, INDEXER_WAIT_TIME);
+		TestSourceReader.waitUntilFileIsIndexed(fIndex, s2, INDEXER_WAIT_TIME);
+
+		fIndex.acquireReadLock();
+		try {
+			IIndexFile[] indexFiles = fIndex.getFiles(ILinkage.CPP_LINKAGE_ID, IndexLocationFactory.getWorkspaceIFL(h1));
+			assertEquals(3, indexFiles.length);
+			for (IIndexFile indexFile : indexFiles) {
+				assertFalse(indexFile.hasPragmaOnceSemantics());
+				assertEquals(1, fIndex.findIncludedBy(indexFile).length);
+			}
+		} finally {
+			fIndex.releaseReadLock();
+		}
+
+		// Change h1.h so that it has the pragma-once semantics. 
+		final long t1= System.currentTimeMillis();
+		final String changedContents = contents[4].toString();
+		ResourcesPlugin.getWorkspace().run(new IWorkspaceRunnable() {
+			@Override
+			public void run(IProgressMonitor monitor) throws CoreException {
+				h1.setContents(new ByteArrayInputStream(changedContents.getBytes()), false, false, npm());
+				h1.setLocalTimeStamp(t1 + 1000); 
+			}
+		}, npm());
+		waitForIndexer();
+
+		fIndex.acquireReadLock();
+		try {
+			IIndexFile[] indexFiles = fIndex.getFiles(ILinkage.CPP_LINKAGE_ID, IndexLocationFactory.getWorkspaceIFL(h1));
+			assertEquals(1, indexFiles.length);
+			for (IIndexFile indexFile : indexFiles) {
+				assertTrue("Timestamp not ok", indexFile.getTimestamp() >= t1);
+				assertTrue(indexFile.hasPragmaOnceSemantics());
+				// Included twice by h2.h and once by s1.cpp
+				assertEquals(2, fIndex.findIncludedBy(indexFile).length);
+			}
+		} finally {
+			fIndex.releaseReadLock();
+		}
+
+		// Change h1.h back to the original state without the pragma-once semantics. 
+		final long t2= System.currentTimeMillis();
+		ResourcesPlugin.getWorkspace().run(new IWorkspaceRunnable() {
+			@Override
+			public void run(IProgressMonitor monitor) throws CoreException {
+				h1.setContents(new ByteArrayInputStream(h1Contents.toString().getBytes()), false, false, npm());
+				h1.setLocalTimeStamp(t2 + 2000); 
+			}
+		}, npm());
+		waitForIndexer();
+
+		fIndex.acquireReadLock();
+		try {
+			IIndexFile[] indexFiles = fIndex.getFiles(ILinkage.CPP_LINKAGE_ID, IndexLocationFactory.getWorkspaceIFL(h1));
+			assertEquals(3, indexFiles.length);
+			for (IIndexFile indexFile : indexFiles) {
+				assertTrue("Timestamp not ok", indexFile.getTimestamp() >= t2);
+				assertFalse(indexFile.hasPragmaOnceSemantics());
+				assertEquals(1, fIndex.findIncludedBy(indexFile).length);
+			}
+		} finally {
+			fIndex.releaseReadLock();
+		}
 	}
 
 	private void standardCheckUpdateIncludes(IFile header, IFile s1, String tag) throws Exception {
@@ -454,9 +636,8 @@ public class IndexIncludeTest extends IndexTestBase {
 		try {
 			assertEquals(1, fIndex.findBindings(tag.toCharArray(), IndexFilter.ALL_DECLARED, npm()).length);
 
-			IIndexFile ifile= fIndex.getFile(ILinkage.CPP_LINKAGE_ID, IndexLocationFactory.getWorkspaceIFL(header));
-			IIndexFile sfile= fIndex.getFile(ILinkage.CPP_LINKAGE_ID, IndexLocationFactory.getWorkspaceIFL(s1));
-			assertNotNull(ifile);
+			IIndexFile ifile= getIndexFile(header);
+			IIndexFile sfile= getIndexFile(s1);
 			IIndexInclude[] includes= fIndex.findIncludedBy(ifile);
 			assertEquals(1, includes.length);
 			assertEquals(s1.getFullPath().toString(), includes[0].getIncludedByLocation().getFullPath());
@@ -465,7 +646,6 @@ public class IndexIncludeTest extends IndexTestBase {
 			assertTrue(includes[0].isResolved());
 			assertFalse(includes[0].isSystemInclude());
 
-			assertNotNull(sfile);
 			includes= fIndex.findIncludes(sfile);
 			assertEquals(3, includes.length);
 			assertEquals(header.getFullPath().toString(), includes[0].getIncludesLocation().getFullPath());
@@ -495,13 +675,11 @@ public class IndexIncludeTest extends IndexTestBase {
 		try {
 			assertEquals(1, fIndex.findBindings(tag.toCharArray(), IndexFilter.ALL_DECLARED, npm()).length);
 
-			IIndexFile ifile= fIndex.getFile(ILinkage.CPP_LINKAGE_ID, IndexLocationFactory.getWorkspaceIFL(header));
-			IIndexFile sfile= fIndex.getFile(ILinkage.CPP_LINKAGE_ID, IndexLocationFactory.getWorkspaceIFL(s1));
-			assertNotNull(ifile);
+			IIndexFile ifile= getIndexFile(header);
+			IIndexFile sfile= getIndexFile(s1);
 			IIndexInclude[] includes= fIndex.findIncludedBy(ifile);
 			assertEquals(0, includes.length);
 
-			assertNotNull(sfile);
 			includes= fIndex.findIncludes(sfile);
 			assertEquals(2, includes.length);
 
@@ -526,9 +704,8 @@ public class IndexIncludeTest extends IndexTestBase {
 		try {
 			assertEquals(1, fIndex.findBindings(tag.toCharArray(), IndexFilter.ALL_DECLARED, npm()).length);
 
-			IIndexFile ifile= fIndex.getFile(ILinkage.CPP_LINKAGE_ID, IndexLocationFactory.getWorkspaceIFL(header));
-			IIndexFile sfile= fIndex.getFile(ILinkage.CPP_LINKAGE_ID, IndexLocationFactory.getWorkspaceIFL(s1));
-			assertNotNull(ifile);
+			IIndexFile ifile= getIndexFile(header);
+			IIndexFile sfile= getIndexFile(s1);
 			IIndexInclude[] includes= fIndex.findIncludedBy(ifile);
 			assertEquals(1, includes.length);
 			assertEquals(s1.getFullPath().toString(), includes[0].getIncludedByLocation().getFullPath());
@@ -537,7 +714,6 @@ public class IndexIncludeTest extends IndexTestBase {
 			assertTrue(includes[0].isResolved());
 			assertTrue(includes[0].isSystemInclude());
 
-			assertNotNull(sfile);
 			includes= fIndex.findIncludes(sfile);
 			assertEquals(2, includes.length);
 			assertEquals(header.getFullPath().toString(), includes[0].getIncludesLocation().getFullPath());
@@ -561,9 +737,8 @@ public class IndexIncludeTest extends IndexTestBase {
 		try {
 			assertEquals(1, fIndex.findBindings(tag.toCharArray(), IndexFilter.ALL_DECLARED, npm()).length);
 
-			IIndexFile ifile= fIndex.getFile(ILinkage.CPP_LINKAGE_ID, IndexLocationFactory.getWorkspaceIFL(header));
-			IIndexFile sfile= fIndex.getFile(ILinkage.CPP_LINKAGE_ID, IndexLocationFactory.getWorkspaceIFL(s1));
-			assertNotNull(ifile);
+			IIndexFile ifile= getIndexFile(header);
+			IIndexFile sfile= getIndexFile(s1);
 			IIndexInclude[] includes= fIndex.findIncludedBy(ifile);
 			assertEquals(1, includes.length);
 			assertEquals(s1.getFullPath().toString(), includes[0].getIncludedByLocation().getFullPath());
@@ -572,7 +747,6 @@ public class IndexIncludeTest extends IndexTestBase {
 			assertTrue(includes[0].isResolved());
 			assertFalse(includes[0].isSystemInclude());
 
-			assertNotNull(sfile);
 			includes= fIndex.findIncludes(sfile);
 			assertEquals(2, includes.length);
 			assertEquals(header.getFullPath().toString(), includes[0].getIncludesLocation().getFullPath());
