@@ -26,6 +26,7 @@ import org.eclipse.cdt.core.CCorePlugin;
 import org.eclipse.cdt.core.cdtvariables.CdtVariableException;
 import org.eclipse.cdt.core.cdtvariables.ICdtVariable;
 import org.eclipse.cdt.core.cdtvariables.ICdtVariableManager;
+import org.eclipse.cdt.core.language.settings.providers.ScannerDiscoveryLegacySupport;
 import org.eclipse.cdt.core.model.CModelException;
 import org.eclipse.cdt.core.model.CoreModel;
 import org.eclipse.cdt.core.model.CoreModelUtil;
@@ -40,11 +41,6 @@ import org.eclipse.cdt.core.model.IPathEntry;
 import org.eclipse.cdt.core.model.ISourceEntry;
 import org.eclipse.cdt.core.resources.IPathEntryVariableManager;
 import org.eclipse.cdt.core.settings.model.CExternalSetting;
-import org.eclipse.cdt.core.settings.model.CIncludeFileEntry;
-import org.eclipse.cdt.core.settings.model.CIncludePathEntry;
-import org.eclipse.cdt.core.settings.model.CLibraryFileEntry;
-import org.eclipse.cdt.core.settings.model.CMacroEntry;
-import org.eclipse.cdt.core.settings.model.CMacroFileEntry;
 import org.eclipse.cdt.core.settings.model.COutputEntry;
 import org.eclipse.cdt.core.settings.model.CSourceEntry;
 import org.eclipse.cdt.core.settings.model.ICConfigurationDescription;
@@ -68,6 +64,7 @@ import org.eclipse.cdt.internal.core.CharOperation;
 import org.eclipse.cdt.internal.core.cdtvariables.CoreVariableSubstitutor;
 import org.eclipse.cdt.internal.core.cdtvariables.DefaultVariableContextInfo;
 import org.eclipse.cdt.internal.core.cdtvariables.ICoreVariableContextInfo;
+import org.eclipse.cdt.internal.core.language.settings.providers.LanguageSettingsProvidersSerializer;
 import org.eclipse.cdt.internal.core.model.APathEntry;
 import org.eclipse.cdt.internal.core.model.CModelStatus;
 import org.eclipse.cdt.internal.core.model.PathEntry;
@@ -128,9 +125,9 @@ public class PathEntryTranslator {
 		ICConfigurationDescription fCfg;
 		ICdtVariableManager fMngr = CCorePlugin.getDefault().getCdtVariableManager();
 
-		public VarSubstitutor(ICConfigurationDescription cfg) {
-			super(new DefaultVariableContextInfo(ICoreVariableContextInfo.CONTEXT_CONFIGURATION, cfg), "", " ");  //$NON-NLS-1$  //$NON-NLS-2$
-			fCfg = cfg;
+		public VarSubstitutor(ICConfigurationDescription cfgDescription) {
+			super(new DefaultVariableContextInfo(ICoreVariableContextInfo.CONTEXT_CONFIGURATION, cfgDescription), "", " ");  //$NON-NLS-1$  //$NON-NLS-2$
+			fCfg = cfgDescription;
 		}
 
 		@Override
@@ -146,9 +143,9 @@ public class PathEntryTranslator {
 		private IPath[] fRefProjPaths;
 		private ICExternalSetting[] fExtSettings;
 
-		public ReferenceSettingsInfo(ICConfigurationDescription des) {
-			fExtSettings = des.getExternalSettings();
-			Map<String, String> map = des.getReferenceInfo();
+		public ReferenceSettingsInfo(ICConfigurationDescription cfgDescription) {
+			fExtSettings = cfgDescription.getExternalSettings();
+			Map<String, String> map = cfgDescription.getReferenceInfo();
 			fRefProjPaths = new IPath[map.size()];
 			int num = 0;
 			for (String proj : map.keySet()) {
@@ -301,25 +298,28 @@ public class PathEntryTranslator {
 	}
 
 	private static ICLanguageSettingEntry createLangEntry(ResolvedEntry entry) {
-		PathEntryValueInfo value = entry.getResolvedValue();
+		PathEntryValueInfo pathEntryValue = entry.getResolvedValue();
+		String name = pathEntryValue.getName();
+
 		int flags = ICSettingEntry.RESOLVED;
 		if (entry.isReadOnly())
 			flags |= ICSettingEntry.READONLY;
 		if (entry.isBuiltIn())
 			flags |= ICSettingEntry.BUILTIN;
 
-		switch (entry.fEntry.getEntryKind()) {
-			case IPathEntry.CDT_LIBRARY:{
-				ILibraryEntry libEntry = (ILibraryEntry)entry.fEntry;
-				IPath path = value.getFullPath();
-				if (path != null) {
-					flags |= ICSettingEntry.VALUE_WORKSPACE_PATH;
-				} else {
-					path = value.getLocation();
-				}
+		IPath path = pathEntryValue.getFullPath();
+		if (path != null) {
+			flags |= ICSettingEntry.VALUE_WORKSPACE_PATH;
+		} else {
+			path = pathEntryValue.getLocation();
+		}
 
+		int kind = entry.fEntry.getEntryKind();
+		switch (kind) {
+			case IPathEntry.CDT_LIBRARY:{
 				if (path != null) {
-					return new CLibraryFileEntry(value.getName(), flags,
+					ILibraryEntry libEntry = (ILibraryEntry)entry.fEntry;
+					return (ICLanguageSettingEntry) CDataUtil.createEntry(ICSettingEntry.LIBRARY_FILE, name, null, null, flags,
 							libEntry.getSourceAttachmentPath(),
 							libEntry.getSourceAttachmentRootPath(),
 							libEntry.getSourceAttachmentPrefixMapping());
@@ -331,52 +331,30 @@ public class PathEntryTranslator {
 //			case IPathEntry.CDT_SOURCE:
 //				return INDEX_CDT_SOURCE;
 			case IPathEntry.CDT_INCLUDE:{
-				IPath path = value.getFullPath();
 				if (path != null) {
-					flags |= ICSettingEntry.VALUE_WORKSPACE_PATH;
-				} else {
-					path = value.getLocation();
-				}
-
-				if (path != null) {
-					return new CIncludePathEntry(value.getName(), flags);
+					return CDataUtil.createCIncludePathEntry(name, flags);
 				}
 				break;
 			}
 //			case IPathEntry.CDT_CONTAINER:
 //				return INDEX_CDT_CONTAINER;
 			case IPathEntry.CDT_MACRO:
-				String name = value.getName();
 				if (name.length() != 0) {
-					String mValue = value.getValue();
-					return new CMacroEntry(name, mValue, flags);
+					String value = pathEntryValue.getValue();
+					return CDataUtil.createCMacroEntry(name, value, flags);
 				}
 				break;
 //			case IPathEntry.CDT_OUTPUT:
 //				return INDEX_CDT_OUTPUT;
 			case IPathEntry.CDT_INCLUDE_FILE:{
-				IPath path = value.getFullPath();
 				if (path != null) {
-					flags |= ICSettingEntry.VALUE_WORKSPACE_PATH;
-				} else {
-					path = value.getLocation();
-				}
-
-				if (path != null) {
-					return new CIncludeFileEntry(value.getName(), flags);
+					return CDataUtil.createCIncludeFileEntry(name, flags);
 				}
 				break;
 			}
 			case IPathEntry.CDT_MACRO_FILE:{
-				IPath path = value.getFullPath();
 				if (path != null) {
-					flags |= ICSettingEntry.VALUE_WORKSPACE_PATH;
-				} else {
-					path = value.getLocation();
-				}
-
-				if (path != null) {
-					return new CMacroFileEntry(value.getName(), flags);
+					return CDataUtil.createCMacroFileEntry(name, flags);
 				}
 				break;
 			}
@@ -626,7 +604,7 @@ public class PathEntryTranslator {
 		private boolean fIsExported;
 		private IProject fProject;
 
-		PathEntryComposer(IPath path, IProject project/*, ICConfigurationDescription cfg*/) {
+		PathEntryComposer(IPath path, IProject project/*, ICConfigurationDescription cfgDescription*/) {
 			fPath = toProjectPath(path);
 			fProject = project;
 		}
@@ -638,7 +616,7 @@ public class PathEntryTranslator {
 			return path.makeAbsolute();
 		}
 
-		PathEntryComposer(ICExclusionPatternPathEntry entry, IProject project/*, ICConfigurationDescription cfg*/) {
+		PathEntryComposer(ICExclusionPatternPathEntry entry, IProject project/*, ICConfigurationDescription cfgDescription*/) {
 			fPath = new Path(entry.getValue());
 			fLangEntry = entry;
 			fProject = project;
@@ -649,7 +627,7 @@ public class PathEntryTranslator {
 			}
 		}
 
-		PathEntryComposer(IPath path, ICLanguageSettingEntry entry, boolean exported, IProject project/*, ICConfigurationDescription cfg*/) {
+		PathEntryComposer(IPath path, ICLanguageSettingEntry entry, boolean exported, IProject project/*, ICConfigurationDescription cfgDescription*/) {
 			fPath = path;
 			fLangEntry = entry;
 			fIsExported = exported;
@@ -677,15 +655,15 @@ public class PathEntryTranslator {
 			return new IPath[0];
 		}
 
-		private IPath[][] getEntryPath(ICSettingEntry entry, ICConfigurationDescription cfg) {
-			return valueToEntryPath(entry.getName(), (entry.getFlags() & ICSettingEntry.VALUE_WORKSPACE_PATH) != 0, cfg);
+		private IPath[][] getEntryPath(ICSettingEntry entry, ICConfigurationDescription cfgDescription) {
+			return valueToEntryPath(entry.getName(), (entry.getFlags() & ICSettingEntry.VALUE_WORKSPACE_PATH) != 0, cfgDescription);
 		}
 
-		private IPath[][] valueToEntryPath(String value, boolean isWsp, ICConfigurationDescription cfg) {
-			String[] pathVarValues = resolveKeepingPathEntryFars(value, cfg);
+		private IPath[][] valueToEntryPath(String value, boolean isWsp, ICConfigurationDescription cfgDescription) {
+			String[] pathVarValues = resolveKeepingPathEntryFars(value, cfgDescription);
 			IPath result[][] = new IPath[2][pathVarValues.length];
 			for (int i = 0; i < pathVarValues.length; i++) {
-				String resolvedValue = resolveAll(value, cfg);
+				String resolvedValue = resolveAll(value, cfgDescription);
 				IPath resolvedPath = new Path(resolvedValue);
 				IPath pathVarPath = new Path(pathVarValues[i]);
 				if (isWsp) {
@@ -725,20 +703,20 @@ public class PathEntryTranslator {
 			return result;
 		}
 
-		public IPathEntry[] toPathEntry(ICConfigurationDescription cfg, boolean keepPathInfo) {
+		public IPathEntry[] toPathEntry(ICConfigurationDescription cfgDescription, boolean keepPathInfo) {
 			IPath path = keepPathInfo ? fPath : fProject.getFullPath();
 			IPathEntry[] result = new IPathEntry[0];
 			if (fLangEntry != null) {
 				switch (fLangEntry.getKind()) {
 				case ICSettingEntry.INCLUDE_FILE:{
-						IPath paths[][] = getEntryPath(fLangEntry, cfg);
+						IPath paths[][] = getEntryPath(fLangEntry, cfgDescription);
 						result = new IPathEntry[paths[0].length];
 						for (int i = 0; i < result.length; i++)
 							result[i] = CoreModel.newIncludeFileEntry(path, null, paths[0][i], paths[1][i], getExclusionPatterns(), fIsExported);
 						return result;
 					}
 				case ICSettingEntry.INCLUDE_PATH:{
-						IPath paths[][] = getEntryPath(fLangEntry, cfg);
+						IPath paths[][] = getEntryPath(fLangEntry, cfgDescription);
 						ICIncludePathEntry ipe = (ICIncludePathEntry)fLangEntry;
 
 						result = new IPathEntry[paths[0].length];
@@ -751,7 +729,7 @@ public class PathEntryTranslator {
 					result[0] = CoreModel.newMacroEntry(path, fLangEntry.getName(), fLangEntry.getValue(), getExclusionPatterns(), fIsExported);
 					return result;
 				case ICSettingEntry.MACRO_FILE:{
-						IPath paths[][] = getEntryPath(fLangEntry, cfg);
+						IPath paths[][] = getEntryPath(fLangEntry, cfgDescription);
 						result = new IPathEntry[paths[0].length];
 						for (int i = 0; i < result.length; i++)
 							result[i] = CoreModel.newMacroFileEntry(path, paths[0][i], null, paths[1][i], getExclusionPatterns(), fIsExported);
@@ -763,7 +741,7 @@ public class PathEntryTranslator {
 					// aren't currently resolved
 					return null;
 //				case ICSettingEntry.LIBRARY_FILE:{
-//						IPath paths[][] = getEntryPath(fLangEntry, cfg);
+//						IPath paths[][] = getEntryPath(fLangEntry, cfgDescription);
 //						result = new IPathEntry[paths[0].length];
 //						for (int i = 0; i < result.length; i++)
 //							result[i] = CoreModel.newLibraryEntry(path, paths[0][i], paths[1][i], null, null, null, fIsExported);
@@ -789,19 +767,19 @@ public class PathEntryTranslator {
 		}
 	}
 
-	private static String resolveAll(String value, ICConfigurationDescription cfg) {
+	private static String resolveAll(String value, ICConfigurationDescription cfgDescription) {
 		try {
-			return CCorePlugin.getDefault().getCdtVariableManager().resolveValue(value, "", " ", cfg); //$NON-NLS-1$ //$NON-NLS-2$
+			return CCorePlugin.getDefault().getCdtVariableManager().resolveValue(value, "", " ", cfgDescription); //$NON-NLS-1$ //$NON-NLS-2$
 		} catch (CdtVariableException e) {
 			CCorePlugin.log(e);
 		}
 		return value;
 	}
 
-	private static String[] resolveKeepingPathEntryFars(String value, ICConfigurationDescription cfg) {
+	private static String[] resolveKeepingPathEntryFars(String value, ICConfigurationDescription cfgDescription) {
 		String[] result = new String[] { value }; // default value;
 		try {
-			VarSubstitutor substitutor = new VarSubstitutor(cfg);
+			VarSubstitutor substitutor = new VarSubstitutor(cfgDescription);
 
 			result = CdtVariableResolver.resolveToStringList(value, substitutor);
 			if (result == null || result.length == 0)
@@ -819,7 +797,7 @@ public class PathEntryTranslator {
 		private LinkedHashMap<IPath, PathEntryComposer> fRefProjMap;
 		private IProject fProject;
 
-		private PathEntryCollector(IProject project/*, ICConfigurationDescription cfg*/) {
+		private PathEntryCollector(IProject project/*, ICConfigurationDescription cfgDescription*/) {
 			fStorage = PathSettingsContainer.createRootContainer();
 			fStorage.setValue(this);
 			fStore = new KindBasedStore<LinkedHashMap<ICSettingEntry, PathEntryComposer>>(false);
@@ -827,7 +805,7 @@ public class PathEntryTranslator {
 			fProject = project;
 		}
 
-		private PathEntryCollector(PathSettingsContainer container, KindBasedStore<LinkedHashMap<ICSettingEntry, PathEntryComposer>> store, IProject project/*, ICConfigurationDescription cfg*/) {
+		private PathEntryCollector(PathSettingsContainer container, KindBasedStore<LinkedHashMap<ICSettingEntry, PathEntryComposer>> store, IProject project/*, ICConfigurationDescription cfgDescription*/) {
 			fStorage = container;
 			fStore = store;
 			fNameKeyMapStore = new KindBasedStore<LinkedHashMap<EntryNameKey, PathEntryComposer>>(false);
@@ -990,6 +968,7 @@ public class PathEntryTranslator {
 			final Set<PathEntryComposer> set = new HashSet<PathEntryComposer>();
 			fStorage.accept(new IPathSettingsContainerVisitor() {
 
+				@Override
 				public boolean visit(PathSettingsContainer container) {
 					PathEntryCollector clr = (PathEntryCollector)container.getValue();
 					clr.getLocalCollectedEntries(kind, list, set);
@@ -1013,7 +992,7 @@ public class PathEntryTranslator {
 			}
 		}
 
-		public List<IPathEntry> getEntries(int peKind, List<IPathEntry> list, int flags, ICConfigurationDescription cfg) {
+		public List<IPathEntry> getEntries(int peKind, List<IPathEntry> list, int flags, ICConfigurationDescription cfgDescription) {
 			if (list == null) {
 				list = new ArrayList<IPathEntry>();
 			}
@@ -1041,7 +1020,7 @@ public class PathEntryTranslator {
 								store.put(peKind, map);
 							}
 							if (!map.containsKey(name)) {
-								pe = cs.toPathEntry(cfg, false);
+								pe = cs.toPathEntry(cfgDescription, false);
 								if (pe != null) {
 									if (pe.length > 1) {
 										System.out.println();
@@ -1050,7 +1029,7 @@ public class PathEntryTranslator {
 								}
 							}
 						} else {
-							pe = cs.toPathEntry(cfg, true);
+							pe = cs.toPathEntry(cfgDescription, true);
 						}
 						if (pe != null)
 							list.addAll(Arrays.asList(pe));
@@ -1077,19 +1056,19 @@ public class PathEntryTranslator {
 			return entry != null &&	(entry.isBuiltIn() || entry.isReadOnly());
 		}
 
-		public List<IPathEntry> getEntries(List<IPathEntry> list, int flags, ICConfigurationDescription cfg) {
+		public List<IPathEntry> getEntries(List<IPathEntry> list, int flags, ICConfigurationDescription cfgDescription) {
 			if (list == null)
 				list = new ArrayList<IPathEntry>();
 			int peKinds[] = PathEntryKyndStore.getSupportedKinds();
 			for (int peKind : peKinds) {
-				getEntries(peKind, list, flags, cfg);
+				getEntries(peKind, list, flags, cfgDescription);
 			}
 
 			return list;
 		}
 
-		public IPathEntry[] getEntries(int flags, ICConfigurationDescription cfg) {
-			List<IPathEntry> list = getEntries(null, flags,cfg);
+		public IPathEntry[] getEntries(int flags, ICConfigurationDescription cfgDescription) {
+			List<IPathEntry> list = getEntries(null, flags,cfgDescription);
 			IPathEntry[] entries = list.toArray(new IPathEntry[list.size()]);
 			return entries;
 		}
@@ -1143,16 +1122,16 @@ public class PathEntryTranslator {
 		return new IPath[0];
 	}
 
-	public PathEntryTranslator(IProject project, CConfigurationData data) {
+	public PathEntryTranslator(IProject project, CConfigurationData cfgData) {
 		fProject = project;
-		fCfgData = data;
-		fRcDataHolder = createRcDataHolder(data);
+		fCfgData = cfgData;
+		fRcDataHolder = createRcDataHolder(cfgData);
 		fTranslatedFilters = PathSettingsContainer.createRootContainer();
 		fTranslatedFilters.setValue(new ResourceInfo[]{new ResourceInfo(fRoot, true)});
 	}
 
-	private static PathSettingsContainer createRcDataHolder(CConfigurationData data) {
-		return CDataUtil.createRcDataHolder(data);
+	private static PathSettingsContainer createRcDataHolder(CConfigurationData cfgData) {
+		return CDataUtil.createRcDataHolder(cfgData);
 	}
 
 	public ReferenceSettingsInfo applyPathEntries(PathEntryResolveInfo info, int op) {
@@ -1457,10 +1436,10 @@ public class PathEntryTranslator {
 		return null;
 	}
 
-	private void applyEntries(CResourceData data, RcDesInfo info, int op) {
-		CLanguageData lDatas[] = data.getType() == ICSettingBase.SETTING_FILE ?
-				new CLanguageData[] { ((CFileData)data).getLanguageData() } :
-				((CFolderData) data).getLanguageDatas();
+	private void applyEntries(CResourceData rcData, RcDesInfo info, int op) {
+		CLanguageData lDatas[] = rcData.getType() == ICSettingBase.SETTING_FILE ?
+				new CLanguageData[] { ((CFileData)rcData).getLanguageData() } :
+				((CFolderData) rcData).getLanguageDatas();
 
 		for (CLanguageData lData : lDatas) {
 			if (lData == null)
@@ -1987,13 +1966,13 @@ public class PathEntryTranslator {
 		}
 	}
 
-	private static CConfigurationData getCfgData(ICConfigurationDescription cfgDes) {
-		return cfgDes instanceof CConfigurationDescriptionCache ?
-				(CConfigurationData)cfgDes : ((IInternalCCfgInfo)cfgDes).getConfigurationData(false);
+	private static CConfigurationData getCfgData(ICConfigurationDescription cfgDescription) {
+		return cfgDescription instanceof CConfigurationDescriptionCache ?
+				(CConfigurationData)cfgDescription : ((IInternalCCfgInfo)cfgDescription).getConfigurationData(false);
 	}
 
-	private static void addOutputEntries(PathEntryCollector cr, CConfigurationData data) {
-		CBuildData bData = data.getBuildData();
+	private static void addOutputEntries(PathEntryCollector cr, CConfigurationData cfgData) {
+		CBuildData bData = cfgData.getBuildData();
 		if (bData != null) {
 			ICOutputEntry oEntries[] = bData.getOutputDirectories();
 			if (oEntries != null && oEntries.length != 0) {
@@ -2002,30 +1981,30 @@ public class PathEntryTranslator {
 		}
 	}
 
-	public static PathEntryCollector collectEntries(IProject project, ICConfigurationDescription des) {
-		CConfigurationData data = getCfgData(des);
+	public static PathEntryCollector collectEntries(IProject project, final ICConfigurationDescription cfgDescription) {
+		CConfigurationData cfgData = getCfgData(cfgDescription);
 
-		ReferenceSettingsInfo refInfo = new ReferenceSettingsInfo(des);
-		ICConfigurationDescription[] allCfgs = des.isPreferenceConfiguration() ?
-				new ICConfigurationDescription[] { des } :
-				des.getProjectDescription().getConfigurations();
+		ReferenceSettingsInfo refInfo = new ReferenceSettingsInfo(cfgDescription);
+		ICConfigurationDescription[] allCfgDescriptions = cfgDescription.isPreferenceConfiguration() ?
+				new ICConfigurationDescription[] { cfgDescription } :
+				cfgDescription.getProjectDescription().getConfigurations();
 
-		CConfigurationData[] allDatas = new CConfigurationData[allCfgs.length];
-		for (int i = 0; i < allCfgs.length; i++) {
-			allDatas[i] = getCfgData(allCfgs[i]);
+		CConfigurationData[] allDatas = new CConfigurationData[allCfgDescriptions.length];
+		for (int i = 0; i < allCfgDescriptions.length; i++) {
+			allDatas[i] = getCfgData(allCfgDescriptions[i]);
 		}
 
-		final PathEntryCollector cr = new PathEntryCollector(project/*, des*/);
-		PathSettingsContainer rcDatas = createRcDataHolder(data);
-		ICSourceEntry sEntries[] = data.getSourceEntries();
+		final PathEntryCollector collector = new PathEntryCollector(project/*, cfgDescription*/);
+		PathSettingsContainer rcDatas = createRcDataHolder(cfgData);
+		ICSourceEntry sEntries[] = cfgData.getSourceEntries();
 		if (sEntries != null && sEntries.length != 0) {
-			cr.setSourceOutputEntries(ICSettingEntry.SOURCE_PATH, sEntries);
+			collector.setSourceOutputEntries(ICSettingEntry.SOURCE_PATH, sEntries);
 		}
 		for (CConfigurationData allData : allDatas) {
-			addOutputEntries(cr, allData);
+			addOutputEntries(collector, allData);
 		}
 		final HashSet<ICSettingEntry> exportedSettings = new HashSet<ICSettingEntry>();
-		cr.setRefProjects(refInfo.getReferencedProjectsPaths());
+		collector.setRefProjects(refInfo.getReferencedProjectsPaths());
 		ICExternalSetting[] settings = refInfo.getExternalSettings();
 		for (ICExternalSetting setting : settings) {
 			exportedSettings.addAll(Arrays.asList(setting.getEntries()));
@@ -2033,13 +2012,14 @@ public class PathEntryTranslator {
 
 		final int kinds[] = KindBasedStore.getLanguageEntryKinds();
 		rcDatas.accept(new IPathSettingsContainerVisitor() {
+			@Override
 			public boolean visit(PathSettingsContainer container) {
-				CResourceData data = (CResourceData)container.getValue();
-				if (data != null) {
-					PathEntryCollector child = cr.createChild(container.getPath());
+				CResourceData rcData = (CResourceData)container.getValue();
+				if (rcData != null) {
+					PathEntryCollector child = collector.createChild(container.getPath());
 					for (int kind : kinds) {
 						List<ICLanguageSettingEntry> list = new ArrayList<ICLanguageSettingEntry>();
-						if (collectResourceDataEntries(kind, data, list)) {
+						if (collectResourceDataEntries(cfgDescription, kind, rcData, list)) {
 							ICLanguageSettingEntry[] entries = list.toArray(new ICLanguageSettingEntry[list.size()]);
 							child.setEntries(kind, entries, exportedSettings);
 						}
@@ -2048,19 +2028,19 @@ public class PathEntryTranslator {
 				return true;
 			}
 		});
-		return cr;
+		return collector;
 	}
 
-	private static boolean collectResourceDataEntries(int kind, CResourceData data, List<ICLanguageSettingEntry> list) {
+	private static boolean collectResourceDataEntries(ICConfigurationDescription cfgDescription, int kind, CResourceData rcData, List<ICLanguageSettingEntry> list) {
 		CLanguageData[] lDatas = null;
-		if (data instanceof CFolderData) {
-			lDatas = ((CFolderData)data).getLanguageDatas();
-		} else if (data instanceof CFileData) {
-			CLanguageData lData = ((CFileData)data).getLanguageData();
+		if (rcData instanceof CFolderData) {
+			lDatas = ((CFolderData)rcData).getLanguageDatas();
+		} else if (rcData instanceof CFileData) {
+			CLanguageData lData = ((CFileData)rcData).getLanguageData();
 			if (lData != null)
 				lDatas = new CLanguageData[] {lData};
 		} else {
-			Exception e = new Exception(UtilMessages.getString("PathEntryTranslator.1") + data.getClass().getName()); //$NON-NLS-1$
+			Exception e = new Exception(UtilMessages.getString("PathEntryTranslator.1") + rcData.getClass().getName()); //$NON-NLS-1$
 			IStatus status = new Status(IStatus.ERROR, CCorePlugin.PLUGIN_ID, e.getMessage(), e);
 			CCorePlugin.log(status);
 		}
@@ -2068,6 +2048,16 @@ public class PathEntryTranslator {
 			return false;
 		}
 
+		IProject project = cfgDescription.getProjectDescription().getProject();
+		if (ScannerDiscoveryLegacySupport.isLanguageSettingsProvidersFunctionalityEnabled(project)) {
+			IResource rc = findResourceInWorkspace(project, rcData.getPath());
+			for (CLanguageData lData : lDatas) {
+				list.addAll(LanguageSettingsProvidersSerializer.getSettingEntriesByKind(cfgDescription, rc, lData.getLanguageId(), kind));
+			}
+			return list.size()>0;
+
+		}
+		// Legacy logic
 		boolean supported = false;
 		for (CLanguageData lData : lDatas) {
 			if (collectLanguageDataEntries(kind, lData, list))
@@ -2088,8 +2078,18 @@ public class PathEntryTranslator {
 		return false;
 	}
 
-	public static IPathEntry[] getPathEntries(IProject project, ICConfigurationDescription cfg, int flags) {
-		PathEntryCollector cr = collectEntries(project, cfg);
-		return cr.getEntries(flags, cfg);
+	public static IPathEntry[] getPathEntries(IProject project, ICConfigurationDescription cfgDescription, int flags) {
+		PathEntryCollector cr = collectEntries(project, cfgDescription);
+		return cr.getEntries(flags, cfgDescription);
+	}
+
+	private static IResource findResourceInWorkspace(IProject project, IPath workspacePath) {
+		IResource rc;
+		if (project != null) {
+			rc = project.findMember(workspacePath);
+		} else {
+			rc = ResourcesPlugin.getWorkspace().getRoot().findMember(workspacePath);
+		}
+		return rc;
 	}
 }
