@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2011 QNX Software Systems and others.
+ * Copyright (c) 2000, 2012 QNX Software Systems and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -12,6 +12,7 @@
  *     Anton Leherbauer (Wind River Systems)
  *     Warren Paul (Nokia) - Bug 218266
  *     James Blackburn (Broadcom Corp.)
+ *     Sergey Prigogin (Google)
  *******************************************************************************/
 package org.eclipse.cdt.internal.core.model;
 
@@ -116,10 +117,12 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		setContentTypeID(idType);
 	}
 
+	@Override
 	public ITranslationUnit getTranslationUnit() {
 		return this;
 	}
 
+	@Override
 	public IInclude createInclude(String includeName, boolean isStd, ICElement sibling,
 			IProgressMonitor monitor) throws CModelException {
 		CreateIncludeOperation op = new CreateIncludeOperation(includeName, isStd, this);
@@ -130,6 +133,7 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		return getInclude(includeName);
 	}
 
+	@Override
 	public IUsing createUsing(String usingName, boolean isDirective, ICElement sibling,
 			IProgressMonitor monitor) throws CModelException {
 		CreateIncludeOperation op = new CreateIncludeOperation(usingName, isDirective, this);
@@ -140,6 +144,7 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		return getUsing(usingName);
 	}
 
+	@Override
 	public INamespace createNamespace(String namespace, ICElement sibling,
 			IProgressMonitor monitor) throws CModelException {
 		CreateNamespaceOperation op = new CreateNamespaceOperation(namespace, this);
@@ -150,6 +155,7 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		return getNamespace(namespace);
 	}
 
+	@Override
 	public ICElement getElementAtLine(int line) throws CModelException {
 		ICElement[] celements = getChildren();
 		for (ICElement celement : celements) {
@@ -163,6 +169,7 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		return null;
 	}
 
+	@Override
 	public ICElement getElementAtOffset(int pos) throws CModelException {
 		ICElement e = getSourceElementAtOffset(pos);
 		if (e == this) {
@@ -171,6 +178,7 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		return e;
 	}
 
+	@Override
 	public ICElement[] getElementsAtOffset(int pos) throws CModelException {
 		ICElement[] e = getSourceElementsAtOffset(pos);
 		if (e.length == 1 && e[0] == this) {
@@ -179,14 +187,15 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		return e;
 	}
 
-	public ICElement getElement(String name) {
-		if (name == null || name.length() == 0) {
+	@Override
+	public ICElement getElement(String qname) {
+		if (qname == null || qname.length() == 0) {
 			return null;
 		}
 		try {
 			ICElement[] celements = getChildren();
 			for (ICElement celement : celements) {
-				if (name.equals(celement.getElementName())) {
+				if (qname.equals(celement.getElementName())) {
 					return celement;
 				}
 			}
@@ -194,15 +203,15 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 			//
 		}
 
-		String[] names = name.split("::"); //$NON-NLS-1$
+		String[] names = qname.split("::"); //$NON-NLS-1$
 		ICElement current = this;
-		for (int j = 0; j < names.length; ++j) {
+		for (String name : names) {
 			if (current instanceof IParent) {
 				try {
 					ICElement[] celements = ((IParent) current).getChildren();
 					current = null;
 					for (ICElement celement : celements) {
-						if (names[j].equals(celement.getElementName())) {
+						if (name.equals(celement.getElementName())) {
 							current = celement;
 							break;
 						}
@@ -217,6 +226,7 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		return current;
 	}
 
+	@Override
 	public IInclude getInclude(String name) {
 		try {
 			ICElement[] celements = getChildren();
@@ -232,6 +242,7 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		return null;
 	}
 
+	@Override
 	public IInclude[] getIncludes() throws CModelException {
 		ICElement[] celements = getChildren();
 		ArrayList<ICElement> aList = new ArrayList<ICElement>();
@@ -243,6 +254,7 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		return aList.toArray(new IInclude[0]);
 	}
 
+	@Override
 	public IUsing getUsing(String name) {
 		try {
 			ICElement[] celements = getChildren();
@@ -258,6 +270,7 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		return null;
 	}
 
+	@Override
 	public IUsing[] getUsings() throws CModelException {
 		ICElement[] celements = getChildren();
 		ArrayList<ICElement> aList = new ArrayList<ICElement>();
@@ -269,6 +282,7 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		return aList.toArray(new IUsing[0]);
 	}
 
+	@Override
 	public INamespace getNamespace(String name) {
 		try {
 			String[] names = name.split("::"); //$NON-NLS-1$
@@ -297,6 +311,7 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		return null;
 	}
 
+	@Override
 	public INamespace[] getNamespaces() throws CModelException {
 		ICElement[] celements = getChildren();
 		ArrayList<ICElement> elementList = new ArrayList<ICElement>();
@@ -312,6 +327,7 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		location = loc;
 	}
 
+	@Override
 	public IPath getLocation() {
 		if (location == null) {
 			IFile file = getFile();
@@ -323,7 +339,7 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		}
 		return UNCPathConverter.toPath(location);
 	}
-	
+
 	@Override
 	public URI getLocationURI() {
 		if (location == null) {
@@ -345,28 +361,34 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		return null;
 	}
 
+	@Override
 	public void copy(ICElement container, ICElement sibling, String rename, boolean force,
 			IProgressMonitor monitor) throws CModelException {
 		getSourceManipulationInfo().copy(container, sibling, rename, force, monitor);
 	}
 
+	@Override
 	public void delete(boolean force, IProgressMonitor monitor) throws CModelException {
 		getSourceManipulationInfo().delete(force, monitor);
 	}
 
+	@Override
 	public void move(ICElement container, ICElement sibling, String rename, boolean force,
 			IProgressMonitor monitor) throws CModelException {
 		getSourceManipulationInfo().move(container, sibling, rename, force, monitor);
 	}
 
+	@Override
 	public void rename(String name, boolean force, IProgressMonitor monitor) throws CModelException {
 		getSourceManipulationInfo().rename(name, force, monitor);
 	}
 
+	@Override
 	public String getSource() throws CModelException {
 		return getSourceManipulationInfo().getSource();
 	}
 
+	@Override
 	public ISourceRange getSourceRange() throws CModelException {
 		return getSourceManipulationInfo().getSourceRange();
 	}
@@ -390,9 +412,9 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 	/**
 	 * Returns true if this handle represents the same Java element
 	 * as the given handle.
-	 * 
+	 *
 	 * <p>Compilation units must also check working copy state;
-	 * 
+	 *
 	 * @see Object#equals(java.lang.Object)
 	 */
 	@Override
@@ -401,6 +423,7 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		return super.equals(o) && !((ITranslationUnit) o).isWorkingCopy();
 	}
 
+	@Override
 	public IWorkingCopy findSharedWorkingCopy() {
 		return CModelManager.getDefault().findSharedWorkingCopy(null, this);
 	}
@@ -424,14 +447,14 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 			Map<ICElement, CElementInfo> newElements, IResource underlyingResource) throws CModelException {
 		TranslationUnitInfo unitInfo = (TranslationUnitInfo) info;
 
-		// generate structure
+		// Generate structure
 		this.parse(newElements, pm);
 
 		// /////////////////////////////////////////////////////////////
 
 		if (isWorkingCopy()) {
 			ITranslationUnit original =  ((IWorkingCopy)this).getOriginalElement();
-			// might be IResource.NULL_STAMP if original does not exist
+			// Might be IResource.NULL_STAMP if original does not exist
 			IResource r = original.getResource();
 			if (r != null && r instanceof IFile) {
 				unitInfo.fTimestamp = ((IFile) r).getModificationStamp();
@@ -441,6 +464,7 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		return unitInfo.isStructureKnown();
 	}
 
+	@Override
 	public char[] getContents() {
 		try {
 			IBuffer buffer = this.getBuffer();
@@ -450,19 +474,23 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		}
 	}
 
+	@Override
 	public IWorkingCopy getSharedWorkingCopy(IProgressMonitor monitor, IProblemRequestor requestor)
 			throws CModelException {
 		return CModelManager.getDefault().getSharedWorkingCopy(null, this, requestor, monitor);
 	}
 
+	@Override
 	public IWorkingCopy getWorkingCopy() throws CModelException {
 		return this.getWorkingCopy(null, null);
 	}
 
+	@Override
 	public IWorkingCopy getWorkingCopy(IProgressMonitor monitor) throws CModelException {
 		return getWorkingCopy(monitor, null);
 	}
 
+	@Override
 	public IWorkingCopy getWorkingCopy(IProgressMonitor monitor, IBufferFactory factory) throws CModelException {
 		WorkingCopy workingCopy;
 		IFile file= getFile();
@@ -471,7 +499,7 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		} else {
 			workingCopy= new WorkingCopy(getParent(), getLocationURI(), getContentTypeId(), factory);
 		}
-		// open the working copy now to ensure contents are that of the current state of this element
+		// Open the working copy now to ensure contents are that of the current state of this element
 		workingCopy.open(monitor);
 		return workingCopy;
 	}
@@ -512,8 +540,8 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		if (!computeAST && isConsistent()) {
 			return null;
 		}
-		
-		// create a new info and make it the current info
+
+		// Create a new info and make it the current info
 		// (this will remove the info and its children just before storing the new infos)
 		CModelManager manager = CModelManager.getDefault();
 		boolean hadTemporaryCache = manager.hasTemporaryCache();
@@ -527,7 +555,7 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 			Map<ICElement, CElementInfo> newElements = manager.getTemporaryCache();
 			openWhenClosed(info, monitor);
 			if (newElements.get(this) == null) {
-				// close any buffer that was opened for the new elements
+				// Close any buffer that was opened for the new elements
 				Iterator<ICElement> iterator = newElements.keySet().iterator();
 				while (iterator.hasNext()) {
 					ICElement element = iterator.next();
@@ -558,19 +586,20 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		return true;
 	}
 
+	@Override
 	public boolean isWorkingCopy() {
 		return false;
 	}
 
 	@Override
 	protected IBuffer openBuffer(IProgressMonitor pm) throws CModelException {
-		// create buffer - translation units only use default buffer factory
+		// Create buffer - translation units only use default buffer factory
 		BufferManager bufManager = getBufferManager();
 		IBuffer buffer = getBufferFactory().createBuffer(this);
 		if (buffer == null)
 			return null;
 
-		// set the buffer source
+		// Set the buffer source
 		if (buffer.getCharacters() == null) {
 			IResource resource = this.getResource();
 			if (resource != null && resource.getType() == IResource.FILE) {
@@ -579,11 +608,19 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 				IPath path = this.getLocation();
 				java.io.File file = path.toFile();
 				if (file != null && file.isFile()) {
+					InputStream stream= null;
 					try {
-						InputStream stream = new FileInputStream(file);
+						stream = new FileInputStream(file);
 						buffer.setContents(Util.getInputStreamAsCharArray(stream, (int)file.length(), null));
 					} catch (IOException e) {
 						buffer.setContents(new char[0]);
+					} finally {
+						if (stream != null) {
+							try {
+								stream.close();
+							} catch (IOException e) {
+							}
+						}
 					}
 				} else {
 					buffer.setContents(new char[0]);
@@ -591,15 +628,16 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 			}
 		}
 
-		// add buffer to buffer cache
+		// Add buffer to buffer cache
 		bufManager.addBuffer(buffer);
 
-		// listen to buffer changes
+		// Listen to buffer changes
 		buffer.addBufferChangedListener(this);
 
 		return buffer;
 	}
 
+	@Override
 	public Map<?, ?> parse() {
 		throw new UnsupportedOperationException("Deprecated method"); //$NON-NLS-1$
 	}
@@ -654,31 +692,44 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		return problemRequestor;
 	}
 
+	@Override
 	public boolean isHeaderUnit() {
-		return CCorePlugin.CONTENT_TYPE_CHEADER.equals(contentTypeId)
-				|| CCorePlugin.CONTENT_TYPE_CXXHEADER.equals(contentTypeId);
+		return isHeaderContentType(contentTypeId);
 	}
 
+	@Override
 	public boolean isSourceUnit() {
-		if (isHeaderUnit())
+		return isSourceContentType(contentTypeId);
+	}
+
+	private static boolean isHeaderContentType(String contentType) {
+		return CCorePlugin.CONTENT_TYPE_CHEADER.equals(contentType)
+				|| CCorePlugin.CONTENT_TYPE_CXXHEADER.equals(contentType);
+	}
+
+	private static boolean isSourceContentType(String contentType) {
+		if (isHeaderContentType(contentType))
 			return false;
 
-		return CCorePlugin.CONTENT_TYPE_CSOURCE.equals(contentTypeId)
-				|| CCorePlugin.CONTENT_TYPE_CXXSOURCE.equals(contentTypeId)
-				|| CCorePlugin.CONTENT_TYPE_ASMSOURCE.equals(contentTypeId)
-				|| LanguageManager.getInstance().isContributedContentType(contentTypeId);
+		return CCorePlugin.CONTENT_TYPE_CSOURCE.equals(contentType)
+				|| CCorePlugin.CONTENT_TYPE_CXXSOURCE.equals(contentType)
+				|| CCorePlugin.CONTENT_TYPE_ASMSOURCE.equals(contentType)
+				|| LanguageManager.getInstance().isContributedContentType(contentType);
 	}
 
+	@Override
 	public boolean isCLanguage() {
 		return CCorePlugin.CONTENT_TYPE_CSOURCE.equals(contentTypeId)
 				|| CCorePlugin.CONTENT_TYPE_CHEADER.equals(contentTypeId);
 	}
 
+	@Override
 	public boolean isCXXLanguage() {
 		return CCorePlugin.CONTENT_TYPE_CXXSOURCE.equals(contentTypeId)
 				|| CCorePlugin.CONTENT_TYPE_CXXHEADER.equals(contentTypeId);
 	}
 
+	@Override
 	public boolean isASMLanguage() {
 		return CCorePlugin.CONTENT_TYPE_ASMSOURCE.equals(contentTypeId);
 	}
@@ -696,20 +747,21 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 			} catch (CoreException e) {
 				CCorePlugin.log(e);
 			}
-			
+
 		}
 		return false;
 	}
 
+	@Override
 	public ILanguage getLanguage() throws CoreException {
 		ILanguage language = null;
-		
+
 		ICProject cProject = getCProject();
 		IProject project= cProject.getProject();
-		
+
 		ICProjectDescription description = CoreModel.getDefault().getProjectDescription(project, false);
 		ICConfigurationDescription configuration;
-		
+
 		if (description == null) {
 			// TODO: Sometimes, CoreModel returns a null ICProjectDescription
 			// so for now, fall back to configuration-less language determination.
@@ -717,7 +769,7 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		} else {
 			configuration = description.getActiveConfiguration();
 		}
-		
+
 		IFile file= getFile();
 		if (file != null) {
 			language = LanguageManager.getInstance().getLanguageForFile(file, configuration, contentTypeId);
@@ -729,6 +781,7 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		return language;
 	}
 
+	@Override
 	public String getContentTypeId() {
 		return contentTypeId;
 	}
@@ -750,9 +803,10 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 	 * Contributed languages' model builders need to be able to indicate whether or
 	 * not the parse of a translation unit was successful without having access to
 	 * the <code>CElementInfo</code> object associated with the translation unit
-	 * 
+	 *
 	 * @param wasSuccessful
 	 */
+	@Override
 	public void setIsStructureKnown(boolean wasSuccessful) {
 		try {
 			this.getElementInfo().setIsStructureKnown(wasSuccessful);
@@ -760,22 +814,28 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		}
 	}
 
+	@Override
 	public IASTTranslationUnit getAST() throws CoreException {
 		return getAST(null, 0, null);
 	}
 
+	@Override
 	public IASTTranslationUnit getAST(IIndex index, int style) throws CoreException {
 		return getAST(index, style, null);
 	}
 
 	public IASTTranslationUnit getAST(IIndex index, int style, IProgressMonitor monitor) throws CoreException {
-		ITranslationUnit configureWith = getSourceContextTU(index, style);
-		
+		boolean incompleteIndex = index != null && !index.isFullyInitialized();
+		IIndexFile[] contextToHeader = getContextToHeader(index, style);
+		ITranslationUnit configureWith = getConfigureWith(contextToHeader);
+		if (configureWith == this)
+			contextToHeader= null;
+
 		IScannerInfo scanInfo= configureWith.getScannerInfo((style & AST_SKIP_IF_NO_BUILD_INFO) == 0);
 		if (scanInfo == null) {
 			return null;
 		}
-		
+
 		FileContent fileContent= FileContent.create(this);
 		if (fileContent == null) {
 			return null;
@@ -786,7 +846,7 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 			return null;
 		}
 
-		IncludeFileContentProvider crf= getIncludeFileContentProvider(style, index, language.getLinkageID());
+		IncludeFileContentProvider crf= getIncludeFileContentProvider(style, index, language.getLinkageID(), contextToHeader);
 		int options= 0;
 		if ((style & AST_SKIP_FUNCTION_BODIES) != 0) {
 			options |= ILanguage.OPTION_SKIP_FUNCTION_BODIES;
@@ -809,10 +869,11 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		ASTTranslationUnit ast = (ASTTranslationUnit) ((AbstractLanguage) language).getASTTranslationUnit(
 				fileContent, scanInfo, crf, index, options, log);
 		ast.setOriginatingTranslationUnit(this);
+		ast.setBasedOnIncompleteIndex(incompleteIndex);
 		return ast;
 	}
 
-	private IncludeFileContentProvider getIncludeFileContentProvider(int style, IIndex index, int linkageID) {
+	private IncludeFileContentProvider getIncludeFileContentProvider(int style, IIndex index, int linkageID, IIndexFile[] contextToHeader) {
 		final ICProject cprj= getCProject();
 		final ProjectIndexerInputAdapter pathResolver = new ProjectIndexerInputAdapter(cprj);
 		IncludeFileContentProvider fileContentsProvider;
@@ -821,82 +882,113 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		} else {
 			fileContentsProvider= IncludeFileContentProvider.getSavedFilesProvider();
 		}
-		
+
 		if (index != null && (style & AST_SKIP_INDEXED_HEADERS) != 0) {
 			IndexBasedFileContentProvider ibcf= new IndexBasedFileContentProvider(index, pathResolver, linkageID,
 					fileContentsProvider);
-			if ((style & AST_CONFIGURE_USING_SOURCE_CONTEXT) != 0) {
-				ibcf.setSupportFillGapFromContextToHeader(true);
-			}
+			ibcf.setContextToHeaderGap(contextToHeader);
 			fileContentsProvider= ibcf;
 		}
-		
+
 		if (fileContentsProvider instanceof InternalFileContentProvider) {
 			final ProjectIndexerIncludeResolutionHeuristics heuristics = new ProjectIndexerIncludeResolutionHeuristics(cprj.getProject(), pathResolver);
 			((InternalFileContentProvider) fileContentsProvider).setIncludeResolutionHeuristics(heuristics);
 		}
-		
+
 		return fileContentsProvider;
 	}
 
-	private static int[] CTX_LINKAGES= {ILinkage.CPP_LINKAGE_ID, ILinkage.C_LINKAGE_ID};
-	public ITranslationUnit getSourceContextTU(IIndex index, int style) {
+	private static final int[] CTX_LINKAGES= { ILinkage.CPP_LINKAGE_ID, ILinkage.C_LINKAGE_ID };
+
+	public IIndexFile[] getContextToHeader(IIndex index, int style) {
 		if (index != null && (style & AST_CONFIGURE_USING_SOURCE_CONTEXT) != 0) {
 			try {
 				fLanguageOfContext= null;
-				for (int element : CTX_LINKAGES) {
-					IIndexFile context= null;
-					final IIndexFileLocation ifl = IndexLocationFactory.getIFL(this);
-					if (ifl != null) {
-						IIndexFile indexFile= index.getFile(element, ifl);
-						if (indexFile != null) {
-							// bug 199412, when a source-file includes itself the context may recurse.
-							HashSet<IIndexFile> visited= new HashSet<IIndexFile>();
-							visited.add(indexFile);
-							indexFile = getParsedInContext(indexFile);
-							while (indexFile != null && visited.add(indexFile)) {
-								context= indexFile;
-								indexFile= getParsedInContext(indexFile);
-							}
-						}
-						if (context != null) {
-							ITranslationUnit tu= CoreModelUtil.findTranslationUnitForLocation(context.getLocation(), getCProject());
-							if (tu != null && tu.isSourceUnit()) {
-								return tu;
+				final IIndexFileLocation ifl = IndexLocationFactory.getIFL(this);
+				if (ifl != null) {
+					IIndexFile best = null;
+					IIndexFile contextOfBest = null;
+					int bestScore= -1;
+					// Find file variant that has the most content and preferably was parsed in
+					// context of a source file.
+					for (int linkageID : CTX_LINKAGES) {
+						for (IIndexFile indexFile : index.getFiles(linkageID, ifl)) {
+							int score= indexFile.getMacros().length * 2;
+							IIndexFile context= getParsedInContext(indexFile);
+							if (isSourceFile(context))
+								score++;
+							if (score > bestScore) {
+								bestScore= score;
+								best= indexFile;
+								contextOfBest = context;
 							}
 						}
 					}
+
+					if (best != null && contextOfBest != best) {
+						return new IIndexFile[] { contextOfBest, best };
+					}
 				}
 			} catch (CoreException e) {
 				CCorePlugin.log(e);
 			}
 		}
-		return this;
-	}
-
-	private IIndexFile getParsedInContext(IIndexFile indexFile)
-			throws CoreException {
-		IIndexInclude include= indexFile.getParsedInContext();
-		if (include != null) {
-			return include.getIncludedBy();
-		}
 		return null;
 	}
 
+	private IIndexFile getParsedInContext(IIndexFile indexFile) throws CoreException {
+		HashSet<IIndexFile> visited= new HashSet<IIndexFile>();
+		// Bug 199412, may recurse.
+		while (visited.add(indexFile)) {
+			IIndexInclude include= indexFile.getParsedInContext();
+			if (include == null)
+				break;
+			indexFile = include.getIncludedBy();
+		}
+		return indexFile;
+	}
+
+	/**
+	 * Returns <code>true</code> if the given file was parsed in a context of a source file.
+	 * @throws CoreException
+	 */
+	private boolean isSourceFile(IIndexFile indexFile) throws CoreException {
+		String path = indexFile.getLocation().getURI().getPath();
+		IContentType cType = CCorePlugin.getContentType(getCProject().getProject(), path);
+		if (cType == null)
+			return false;
+
+		return isSourceContentType(cType.getId());
+	}
+
+	private ITranslationUnit getConfigureWith(IIndexFile[] contextToHeader) throws CoreException {
+		if (contextToHeader != null) {
+			ITranslationUnit configureWith = CoreModelUtil.findTranslationUnitForLocation(
+					contextToHeader[0].getLocation(), getCProject());
+			if (configureWith != null)
+				return configureWith;
+		}
+		return this;
+	}
+
+	@Override
 	public IASTCompletionNode getCompletionNode(IIndex index, int style, int offset) throws CoreException {
-		ITranslationUnit configureWith= getSourceContextTU(index, style);
-		
+		IIndexFile[] contextToHeader = getContextToHeader(index, style);
+		ITranslationUnit configureWith = getConfigureWith(contextToHeader);
+		if (configureWith == this)
+			contextToHeader= null;
+
 		IScannerInfo scanInfo = configureWith.getScannerInfo((style & ITranslationUnit.AST_SKIP_IF_NO_BUILD_INFO) == 0);
 		if (scanInfo == null) {
 			return null;
 		}
-		
+
 		FileContent fileContent= FileContent.create(this);
-		
+
 		ILanguage language= configureWith.getLanguage();
 		fLanguageOfContext= language;
 		if (language != null) {
-			IncludeFileContentProvider crf= getIncludeFileContentProvider(style, index, language.getLinkageID());
+			IncludeFileContentProvider crf= getIncludeFileContentProvider(style, index, language.getLinkageID(), contextToHeader);
 			IASTCompletionNode result = language.getCompletionNode(fileContent, scanInfo, crf, index,
 					ParserUtil.getParserLogService(), offset);
 			if (result != null) {
@@ -911,6 +1003,7 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		return null;
 	}
 
+	@Override
 	@Deprecated
 	public org.eclipse.cdt.core.parser.CodeReader getCodeReader() {
 		IPath location= getLocation();
@@ -919,12 +1012,12 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		if (isWorkingCopy()) {
 			return new org.eclipse.cdt.core.parser.CodeReader(location.toOSString(), getContents());
 		}
-		
+
 		IResource res= getResource();
 		try {
 			if (res instanceof IFile)
 				return InternalParserUtil.createWorkspaceFileReader(location.toOSString(), (IFile) res, null);
-			else 
+			else
 				return InternalParserUtil.createExternalFileReader(location.toOSString(), null);
 		} catch (CoreException e) {
 			CCorePlugin.log(e);
@@ -934,6 +1027,7 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		return null;
 	}
 
+	@Override
 	public IScannerInfo getScannerInfo(boolean force) {
 		IResource resource = getResource();
 		ICProject project = getCProject();
@@ -943,7 +1037,7 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		if (!force && CoreModel.isScannerInformationEmpty(infoResource)) {
 			return null;
 		}
-		
+
 		IScannerInfoProvider provider = CCorePlugin.getDefault().getScannerInfoProvider(rproject);
 		if (provider != null) {
 			IScannerInfo scanInfo = provider.getScannerInformation(infoResource);
@@ -955,7 +1049,7 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 		}
 		return null;
 	}
-	
+
 	/**
 	 * Return the language of the context this file was parsed in. Works only after using
 	 * {@link #getAST(IIndex, int, IProgressMonitor)} with the flag {@link ITranslationUnit#AST_CONFIGURE_USING_SOURCE_CONTEXT}.
@@ -1118,30 +1212,35 @@ public class TranslationUnit extends Openable implements ITranslationUnit {
 			}
 		}
 	}
-	
+
 	@Override
 	protected char getHandleMementoDelimiter() {
 		return CElement.CEM_TRANSLATIONUNIT;
 	}
 
+	@Override
 	public boolean isActive() {
 		return true;
 	}
 
+	@Override
 	public int getIndex() {
 		return 0;
 	}
 
+	@Override
 	@Deprecated
 	public IWorkingCopy findSharedWorkingCopy(IBufferFactory bufferFactory) {
 		return CModelManager.getDefault().findSharedWorkingCopy(bufferFactory, this);
 	}
 
+	@Override
 	@Deprecated
 	public IWorkingCopy getSharedWorkingCopy(IProgressMonitor monitor, IBufferFactory factory, IProblemRequestor requestor) throws CModelException {
 		return CModelManager.getDefault().getSharedWorkingCopy(factory, this, requestor, monitor);
 	}
 
+	@Override
 	@Deprecated
 	public IWorkingCopy getSharedWorkingCopy(IProgressMonitor monitor, IBufferFactory factory) throws CModelException {
 		return CModelManager.getDefault().getSharedWorkingCopy(factory, this, null, monitor);
