@@ -16,7 +16,10 @@ import java.util.Map;
 
 import org.eclipse.cdt.core.cdtvariables.ICdtVariable;
 import org.eclipse.cdt.core.cdtvariables.ICdtVariablesContributor;
+import org.eclipse.cdt.core.language.settings.providers.ILanguageSettingsProvider;
+import org.eclipse.cdt.core.language.settings.providers.ILanguageSettingsProvidersKeeper;
 import org.eclipse.cdt.core.settings.model.CConfigurationStatus;
+import org.eclipse.cdt.core.settings.model.CProjectDescriptionEvent;
 import org.eclipse.cdt.core.settings.model.ICBuildSetting;
 import org.eclipse.cdt.core.settings.model.ICConfigExtensionReference;
 import org.eclipse.cdt.core.settings.model.ICConfigurationDescription;
@@ -44,7 +47,6 @@ import org.eclipse.cdt.core.settings.model.extension.CResourceData;
 import org.eclipse.cdt.core.settings.model.extension.CTargetPlatformData;
 import org.eclipse.cdt.core.settings.model.extension.impl.CDefaultConfigurationData;
 import org.eclipse.cdt.core.settings.model.util.CDataUtil;
-import org.eclipse.cdt.core.settings.model.util.CSettingEntryFactory;
 import org.eclipse.cdt.core.settings.model.util.PathSettingsContainer;
 import org.eclipse.cdt.internal.core.cdtvariables.CdtVariableManager;
 import org.eclipse.cdt.internal.core.cdtvariables.StorableCdtVariables;
@@ -53,8 +55,42 @@ import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.QualifiedName;
 
+/**
+ * CConfigurationDescriptionCache is a proxy class for serialization of configuration description data.
+ *
+ * An inspection of the scenario where user changes project properties and saves it yields
+ * following sequence of events:
+ * <ol>
+ * <li> Initialization:
+ *   <ul>
+ *   <li> After eclipse started a project is being opened. A new CConfigurationDescriptionCache is created
+ *        with CConfigurationDescriptionCache(ICStorageElement storage, CProjectDescription parent) constructor.
+ *   <li> Any clients needed ICConfigurationDescription get CConfigurationDescription using constructor
+ *        CConfigurationDescription(CConfigurationData data, String buildSystemId, ICDataProxyContainer cr)
+ *        where the CConfigurationDescriptionCache is passed as data. The reference to cache is kept in field fCfgCache.
+ *   <li> fCfgCache is used to getSpecSettings() CConfigurationSpecSettings, after that fCfgCache is set to null.
+ *   </ul>
+ * <li> User enters project properties/settings:
+ *   <ul>
+ *   <li> another CConfigurationDescription (settings configuration) created using the same constructor setting fCfgCache
+ *        to the CConfigurationDescriptionCache.
+ *   </ul>
+ * <li> User changes settings (in the settings configuration CConfigurationDescription) and saves it:
+ *   <ul>
+ *   <li> new CConfigurationDescriptionCache is created from the CConfigurationDescription via constructor
+ *        CConfigurationDescriptionCache(ICConfigurationDescription baseDescription, ...) where
+ *        baseDescription is saved as fBaseDescription.
+ *   <li> CConfigurationDescriptionCache.applyData(...) is used to persist the data. at that point
+ *        reference fBaseDescription gets set to null.
+ *   </ul>
+ * </ol>
+ *
+ * @see ICConfigurationDescription
+ * @see CConfigurationDescription
+ * @see CProjectDescriptionEvent
+ */
 public class CConfigurationDescriptionCache extends CDefaultConfigurationData
-		implements ICConfigurationDescription, IInternalCCfgInfo, ICachedData {
+		implements ICConfigurationDescription, IInternalCCfgInfo, ILanguageSettingsProvidersKeeper, ICachedData {
 	private CProjectDescription fParent;
 	private PathSettingsContainer fPathSettingContainer = PathSettingsContainer.createRootContainer();
 	private ResourceDescriptionHolder fRcHolder = new ResourceDescriptionHolder(fPathSettingContainer, true);
@@ -67,7 +103,6 @@ public class CConfigurationDescriptionCache extends CDefaultConfigurationData
 	private boolean fDataLoadded;
 	private boolean fInitializing;
 	private ICConfigurationDescription fBaseDescription;
-	private CSettingEntryFactory fSettingsFactory;
 	private ICSourceEntry[] fResolvedSourceEntries;
 
 	CConfigurationDescriptionCache(ICStorageElement storage, CProjectDescription parent) throws CoreException{
@@ -75,31 +110,27 @@ public class CConfigurationDescriptionCache extends CDefaultConfigurationData
 		fInitializing = true;
 		fParent = parent;
 		fSpecSettings = new CConfigurationSpecSettings(this, storage);
-		
+
 		fId = fSpecSettings.getId();
 		fName = fSpecSettings.getName();
-		
+
 //		loadData();
 	}
-	
+
 	public boolean isInitializing(){
 		return fInitializing;
 	}
-	
-	void loadData(CSettingEntryFactory factory) throws CoreException{
+
+	void loadData() throws CoreException{
 		if(fDataLoadded)
 			return;
-		
+
 		fDataLoadded = true;
-			
+
 		fData = CProjectDescriptionManager.getInstance().loadData(this, null);
-		
-		fSettingsFactory = factory;
-		
+
 		copySettingsFrom(fData, true);
-		
-		fSettingsFactory = null;
-		
+
 		fSpecSettings.reconcileExtensionSettings(true);
 		((CBuildSettingCache)fBuildData).initEnvironmentCache();
 		ICdtVariable vars[] = CdtVariableManager.getDefault().getVariables(this);
@@ -121,25 +152,24 @@ public class CConfigurationDescriptionCache extends CDefaultConfigurationData
 			fData = base;
 //			base = CProjectDescriptionManager.getInstance().applyData(this, baseDescription, base);
 //			fData = base;
-		} 
-		
+		}
+
 		fBaseCache = baseCache;
 	}
-	
+
 	CConfigurationDescriptionCache getBaseCache(){
 		return fBaseCache;
 	}
-	
-	boolean applyData(CSettingEntryFactory factory, SettingsContext context) throws CoreException{
-		boolean modified = true; 
+
+	boolean applyData(SettingsContext context) throws CoreException{
+		boolean modified = true;
 		if(fBaseDescription != null){
-		
+
 			fData = CProjectDescriptionManager.getInstance().applyData(this, fBaseDescription, fData, context, null);
 			fDataLoadded = true;
 			fName = fData.getName();
 			fId = fData.getId();
-			fSettingsFactory = factory;
-			
+
 			if((context.getAllConfigurationSettingsFlags() & IModificationContext.CFG_DATA_SETTINGS_UNMODIFIED) == 0  || fBaseCache == null){
 				copySettingsFrom(fData, true);
 			} else {
@@ -148,26 +178,20 @@ public class CConfigurationDescriptionCache extends CDefaultConfigurationData
 				if(!modified)
 					modified = (context.getAllConfigurationSettingsFlags() & IModificationContext.CFG_DATA_STORAGE_UNMODIFIED) == 0;
 			}
-			
-			fSettingsFactory = null;
-			
+
 			ICdtVariable vars[] = CdtVariableManager.getDefault().getVariables(this);
 			fMacros = new StorableCdtVariables(vars, true);
 			fSpecSettings.serialize();
 			fSpecSettings.setModified(false);
-		
+
 		}
-		
+
 		fBaseDescription = null;
 		fBaseCache = null;
-		
+
 		return modified;
 	}
-	
-	CSettingEntryFactory getSettingsFactory(){
-		return fSettingsFactory;
-	}
-	
+
 	public StorableCdtVariables getCachedVariables(){
 		return fMacros;
 	}
@@ -175,7 +199,7 @@ public class CConfigurationDescriptionCache extends CDefaultConfigurationData
 	protected void setId(String id) throws CoreException {
 		throw ExceptionFactory.createIsReadOnlyException();
 	}
-	
+
 	@Override
 	public CFileData copyFileData(IPath path, CFileData base, boolean clone) {
 		return new CFileDescriptionCache(base, this);
@@ -202,20 +226,22 @@ public class CConfigurationDescriptionCache extends CDefaultConfigurationData
 		fRcHolder.addResourceDescription(des.getPath(), des);
 		fChildList.add(des);
 	}
-	
+
 	void addTargetPlatformSetting(ICTargetPlatformSetting tpS){
 		fChildList.add(tpS);
 	}
-	
+
 	void addBuildSetting(ICBuildSetting bs){
 		fChildList.add(bs);
 		fBuildData = (CBuildData)bs;
 	}
 
+	@Override
 	public ICProjectDescription getProjectDescription() {
 		return fParent;
 	}
 
+	@Override
 	public ICResourceDescription getResourceDescription(IPath path, boolean exactPath) {
 		return fRcHolder.getResourceDescription(path, exactPath);
 	}
@@ -224,16 +250,19 @@ public class CConfigurationDescriptionCache extends CDefaultConfigurationData
 		return fRcHolder.getResourceDescriptions(kind);
 	}
 
+	@Override
 	public ICFolderDescription getRootFolderDescription() {
 		return (ICFolderDescription)fRootFolderData;
 	}
 
+	@Override
 	public boolean isActive() {
 		if(isPreferenceConfiguration())
 			return false;
 		return fParent.getActiveConfiguration() == this;
 	}
 
+	@Override
 	public void removeResourceDescription(ICResourceDescription des)
 			throws CoreException {
 		throw new CoreException(new DescriptionStatus(SettingsModelMessages.getString("CConfigurationDescriptionCache.0"))); //$NON-NLS-1$
@@ -269,46 +298,57 @@ public class CConfigurationDescriptionCache extends CDefaultConfigurationData
 		throw ExceptionFactory.createIsReadOnlyException();
 	}
 
+	@Override
 	public ICSettingObject[] getChildSettings() {
 		return fChildList.toArray(new ICSettingObject[fChildList.size()]);
 	}
 
+	@Override
 	public ICConfigurationDescription getConfiguration() {
 		return this;
 	}
 
+	@Override
 	public ICSettingContainer getParent() {
 		return fParent;
 	}
 
+	@Override
 	public ICResourceDescription[] getResourceDescriptions() {
 		return fRcHolder.getResourceDescriptions();
 	}
-	
+
+	@Override
 	public ICStorageElement getStorage(String id, boolean create) throws CoreException {
 		return getSpecSettings().getStorage(id, create);
 	}
-	
+
+	@Override
 	public void removeStorage(String id) throws CoreException {
 		getSpecSettings().removeStorage(id);
 	}
-	
+
+	@Override
 	public ICStorageElement importStorage(String id, ICStorageElement el) throws UnsupportedOperationException, CoreException {
 		return getSpecSettings().importStorage(id, el);
 	}
 
+	@Override
 	public CConfigurationSpecSettings getSpecSettings() /*throws CoreException*/{
 		return fSpecSettings;
 	}
 
+	@Override
 	public String getBuildSystemId() {
 		return fSpecSettings.getBuildSystemId();
 	}
 
+	@Override
 	public CConfigurationData getConfigurationData() {
 		return fData;
 	}
 
+	@Override
 	public void setConfigurationData(String bsId, CConfigurationData data) throws WriteAccessException {
 		throw ExceptionFactory.createIsReadOnlyException();
 	}
@@ -318,42 +358,49 @@ public class CConfigurationDescriptionCache extends CDefaultConfigurationData
 		return false;
 	}
 
+	@Override
 	public CConfigurationData getConfigurationData(boolean write) throws WriteAccessException {
 		if(write)
 			throw ExceptionFactory.createIsReadOnlyException();
-		
+
 		return this;
 	}
 
+	@Override
 	public void setActive() throws WriteAccessException {
 		throw ExceptionFactory.createIsReadOnlyException();
 	}
-	
+
 /*	public CConfigurationData getBaseData(){
 		return fData;
 	}
 */
+	@Override
 	public ICFileDescription createFileDescription(IPath path, ICResourceDescription base) throws CoreException, WriteAccessException {
 		throw ExceptionFactory.createIsReadOnlyException();
 	}
 
+	@Override
 	public ICFolderDescription createFolderDescription(IPath path, ICFolderDescription base) throws CoreException, WriteAccessException {
 		throw ExceptionFactory.createIsReadOnlyException();
 	}
-	
+
 	ResourceDescriptionHolder createHolderForRc(IPath path){
 		return new ResourceDescriptionHolder(fPathSettingContainer.getChildContainer(path, true, true), false);
 	}
 
+	@Override
 	public boolean isReadOnly() {
 		return !fInitializing;
 	}
 
+	@Override
 	public void setReadOnly(boolean readOnly, boolean keepModify) {
 		if (readOnly)
 			throw ExceptionFactory.createIsReadOnlyException();
 	}
 
+	@Override
 	public ICTargetPlatformSetting getTargetPlatformSetting() {
 		return (ICTargetPlatformSetting)getTargetPlatformData();
 	}
@@ -363,10 +410,12 @@ public class CConfigurationDescriptionCache extends CDefaultConfigurationData
 		return new CTargetPlatformSettingCache(base, this);
 	}
 
+	@Override
 	public ICFileDescription[] getFileDescriptions() {
 		return (ICFileDescription[])fRcHolder.getResourceDescriptions(ICSettingBase.SETTING_FILE);
 	}
 
+	@Override
 	public ICFolderDescription[] getFolderDescriptions() {
 		return (ICFolderDescription[])fRcHolder.getResourceDescriptions(ICSettingBase.SETTING_FOLDER);
 	}
@@ -380,14 +429,14 @@ public class CConfigurationDescriptionCache extends CDefaultConfigurationData
 		initSourceEntries();
 		return fProjSourceEntries.clone();
 	}
-	
+
 	private void initSourceEntries(){
 		if(fProjSourceEntries == null){
-			IProject project = getProject(); 
+			IProject project = getProject();
 			fProjSourceEntries = CDataUtil.adjustEntries(fSourceEntries, true, project);
 		}
 	}
-	
+
 	private IProject getProject(){
 		return isPreferenceConfiguration() ? null : getProjectDescription().getProject();
 	}
@@ -397,50 +446,59 @@ public class CConfigurationDescriptionCache extends CDefaultConfigurationData
 		throw ExceptionFactory.createIsReadOnlyException();
 	}
 
+	@Override
 	public Map<String, String> getReferenceInfo() {
 		return getSpecSettings().getReferenceInfo();
 	}
 
+	@Override
 	public void setReferenceInfo(Map<String, String> refs) {
 		throw ExceptionFactory.createIsReadOnlyException();
 	}
 
+	@Override
 	public ICExternalSetting createExternalSetting(String[] languageIDs,
 			String[] contentTypeIds, String[] extensions,
 			ICSettingEntry[] entries) {
 		if(!fInitializing)
 			throw ExceptionFactory.createIsReadOnlyException();
-		
+
 		return fSpecSettings.createExternalSetting(languageIDs, contentTypeIds, extensions, entries);
 	}
 
+	@Override
 	public ICExternalSetting[] getExternalSettings() {
 		return fSpecSettings.getExternalSettings();
 	}
 
+	@Override
 	public void removeExternalSetting(ICExternalSetting setting) {
 		if(!fInitializing)
 			throw ExceptionFactory.createIsReadOnlyException();
-		
+
 		fSpecSettings.removeExternalSetting(setting);
 	}
 
+	@Override
 	public void removeExternalSettings() {
 		if(!fInitializing)
 			throw ExceptionFactory.createIsReadOnlyException();
-		
+
 		fSpecSettings.removeExternalSettings();
 	}
-	
+
+	@Override
 	public ICBuildSetting getBuildSetting() {
 		return (ICBuildSetting)getBuildData();
 	}
 
+	@Override
 	public void setSessionProperty(QualifiedName name, Object value) {
 		fSpecSettings.setSettionProperty(name, value);
 		//throw ExceptionFactory.createIsReadOnlyException();
 	}
 
+	@Override
 	public Object getSessionProperty(QualifiedName name) {
 		return fSpecSettings.getSettionProperty(name);
 	}
@@ -450,6 +508,7 @@ public class CConfigurationDescriptionCache extends CDefaultConfigurationData
 		return fData.getBuildVariablesContributor();
 	}
 
+	@Override
 	public ICConfigExtensionReference create(String extensionPoint,
 			String extension) throws CoreException {
 		if(!fInitializing)
@@ -457,32 +516,37 @@ public class CConfigurationDescriptionCache extends CDefaultConfigurationData
 		return fSpecSettings.create(extensionPoint, extension);
 	}
 
+	@Override
 	public ICConfigExtensionReference[] get(String extensionPointID) {
 		return fSpecSettings.get(extensionPointID);
 	}
 
+	@Override
 	public void remove(ICConfigExtensionReference ext) throws CoreException {
 		if(!fInitializing)
 			throw ExceptionFactory.createIsReadOnlyException();
 		fSpecSettings.remove(ext);
 	}
 
+	@Override
 	public void remove(String extensionPoint) throws CoreException {
 		if(!fInitializing)
 			throw ExceptionFactory.createIsReadOnlyException();
 		fSpecSettings.remove(extensionPoint);
 	}
-	
+
+	@Override
 	public boolean isPreferenceConfiguration() {
 		return getProjectDescription() == null;
 	}
-	
+
 	void doneInitialization(){
 		CProjectDescriptionManager.getInstance().notifyCached(this, fData, null);
 		fInitializing = false;
 		fSpecSettings.doneInitialization();
 	}
-	
+
+	@Override
 	public ICLanguageSetting getLanguageSettingForFile(IPath path, boolean ignoreExcludeStatus) {
 		return CProjectDescriptionManager.getLanguageSettingForFile(this, path, ignoreExcludeStatus);
 	}
@@ -502,26 +566,30 @@ public class CConfigurationDescriptionCache extends CDefaultConfigurationData
 		IProject project = getProject();
 		if(project != null)
 			path = project.getFullPath().append(path);
-		
+
 		return CDataUtil.isExcluded(path, fProjSourceEntries);
 	}
 
+	@Override
 	public String[] getExternalSettingsProviderIds() {
 		return fSpecSettings.getExternalSettingsProviderIds();
 	}
 
+	@Override
 	public void setExternalSettingsProviderIds(String[] ids) {
 		if(!fInitializing)
 			throw ExceptionFactory.createIsReadOnlyException();
 		fSpecSettings.setExternalSettingsProviderIds(ids);
 	}
 
+	@Override
 	public void updateExternalSettingsProviders(String[] ids) {
 		if(!fInitializing)
 			throw ExceptionFactory.createIsReadOnlyException();
 		fSpecSettings.updateExternalSettingsProviders(ids);
 	}
 
+	@Override
 	public ICSourceEntry[] getResolvedSourceEntries() {
 		if(fResolvedSourceEntries == null){
 			ICSourceEntry[] entries = getSourceEntries();
@@ -530,9 +598,33 @@ public class CConfigurationDescriptionCache extends CDefaultConfigurationData
 		return fResolvedSourceEntries;
 	}
 
+	@Override
 	public CConfigurationStatus getConfigurationStatus() {
 		CConfigurationStatus status = getStatus();
 		return status != null ? status : CConfigurationStatus.CFG_STATUS_OK;
 	}
 
+	@Override
+	public void setLanguageSettingProviders(List<ILanguageSettingsProvider> providers) {
+		if(!fInitializing)
+			throw ExceptionFactory.createIsReadOnlyException();
+		fSpecSettings.setLanguageSettingProviders(providers);
+	}
+
+	@Override
+	public List<ILanguageSettingsProvider> getLanguageSettingProviders() {
+		return fSpecSettings.getLanguageSettingProviders();
+	}
+
+	@Override
+	public void setDefaultLanguageSettingsProvidersIds(String[] ids) {
+		if(!fInitializing)
+			throw ExceptionFactory.createIsReadOnlyException();
+		fSpecSettings.setDefaultLanguageSettingsProvidersIds(ids);
+	}
+
+	@Override
+	public String[] getDefaultLanguageSettingsProvidersIds() {
+		return fSpecSettings.getDefaultLanguageSettingsProvidersIds();
+	}
 }
