@@ -16,8 +16,7 @@ package org.eclipse.cdt.core.dom.ast;
  * @noimplement This interface is not intended to be implemented by clients.
  */
 public interface IField extends IVariable {
-
-    public static final IField[] EMPTY_FIELD_ARRAY = new IField[0];
+    public static final IField[] EMPTY_FIELD_ARRAY = {};
 
     /**
      * Returns the composite type that owns the field.
