@@ -112,7 +112,7 @@ public class CScope implements ICScope, IASTInternalScope {
     private IASTNode physicalNode = null;
     private boolean isCached = false;
     
-    private CharArrayObjectMap[] mapsToNameOrBinding = { CharArrayObjectMap.EMPTY_MAP, CharArrayObjectMap.EMPTY_MAP };
+    private final CharArrayObjectMap<?> mapsToNameOrBinding[] = { CharArrayObjectMap.EMPTY_MAP, CharArrayObjectMap.EMPTY_MAP };
 	private final EScopeKind kind;
     
 	public CScope(IASTNode physical, EScopeKind eKind) {
@@ -120,6 +120,7 @@ public class CScope implements ICScope, IASTInternalScope {
         kind= eKind;
     }
     
+	@Override
 	public EScopeKind getKind() {
 		return kind;
 	}
@@ -127,12 +128,13 @@ public class CScope implements ICScope, IASTInternalScope {
     /* (non-Javadoc)
      * @see org.eclipse.cdt.core.dom.ast.IScope#getParent()
      */
-    public IScope getParent() {
+    @Override
+	public IScope getParent() {
         return CVisitor.getContainingScope(physicalNode);
     }
 
     protected static class CollectNamesAction extends ASTVisitor {
-        private char[] name;
+        private final char[] name;
         private IASTName[] result = null;
         
         CollectNamesAction(char[] n) {
@@ -146,7 +148,7 @@ public class CScope implements ICScope, IASTInternalScope {
                 prop == IASTCompositeTypeSpecifier.TYPE_NAME ||
                 prop == IASTDeclarator.DECLARATOR_NAME) {
                 if (CharArrayUtils.equals(n.toCharArray(), name))
-                    result = (IASTName[]) ArrayUtil.append(IASTName.class, result, n);    
+                    result = ArrayUtil.append(IASTName.class, result, n);    
             }
             
             return PROCESS_CONTINUE; 
@@ -160,13 +162,14 @@ public class CScope implements ICScope, IASTInternalScope {
         }
 
         public IASTName[] getNames() {
-            return (IASTName[]) ArrayUtil.trim(IASTName.class, result);
+            return ArrayUtil.trim(IASTName.class, result);
         }
     }
     
     /* (non-Javadoc)
      * @see org.eclipse.cdt.core.dom.ast.IScope#find(java.lang.String)
      */
+	@Override
 	public IBinding[] find(String name) {
 		return CVisitor.findBindings(this, name);
 	}
@@ -188,19 +191,22 @@ public class CScope implements ICScope, IASTInternalScope {
     /* (non-Javadoc)
      * @see org.eclipse.cdt.core.dom.ast.IScope#getPhysicalNode()
      */
-    public IASTNode getPhysicalNode() {
+    @Override
+	public IASTNode getPhysicalNode() {
         return physicalNode;
     }
 
+	@Override
 	public void addName(IASTName name) {
 		final char[] nchars = name.toCharArray();
 		if (nchars.length == 0)
 			return;
 
 		int type = getNamespaceType(name);
-		CharArrayObjectMap map = mapsToNameOrBinding[type];
+		@SuppressWarnings("unchecked")
+		CharArrayObjectMap<Object> map = (CharArrayObjectMap<Object>) mapsToNameOrBinding[type];
 		if (map == CharArrayObjectMap.EMPTY_MAP)
-			map = mapsToNameOrBinding[type] = new CharArrayObjectMap(1);
+			mapsToNameOrBinding[type] = map = new CharArrayObjectMap<Object>(1);
 
 		Object o= map.get(nchars);
 		if (o instanceof IASTName) {
@@ -215,7 +221,7 @@ public class CScope implements ICScope, IASTInternalScope {
 				if (n == name)
 					return;
 			}
-			final IASTName[] newNames= (IASTName[]) ArrayUtil.append(IASTName.class, names, name);
+			final IASTName[] newNames= ArrayUtil.append(IASTName.class, names, name);
 			if (newNames != names)
 				map.put(nchars, newNames);
 		} else {
@@ -234,14 +240,17 @@ public class CScope implements ICScope, IASTInternalScope {
         
         return NAMESPACE_TYPE_OTHER;
     }
-    public final IBinding getBinding(IASTName name, boolean resolve) {
+    @Override
+	public final IBinding getBinding(IASTName name, boolean resolve) {
     	return getBinding(name, resolve, IIndexFileSet.EMPTY);
     }
     
+	@Override
 	public final IBinding[] getBindings(IASTName name, boolean resolve, boolean prefix) {
 		return getBindings(name, resolve, prefix, IIndexFileSet.EMPTY);
 	}
 
+	@Override
 	public final IBinding getBinding(IASTName name, boolean resolve, IIndexFileSet fileSet) {
 		char[] c = name.toCharArray();
 	    if (c.length == 0) {
@@ -323,12 +332,13 @@ public class CScope implements ICScope, IASTInternalScope {
     /* (non-Javadoc)
      * @see org.eclipse.cdt.core.dom.ast.c.ICScope#getBinding(org.eclipse.cdt.core.dom.ast.IASTName, boolean)
      */
+	@Override
 	public final IBinding[] getBindings(IASTName name, boolean resolve, boolean prefixLookup, IIndexFileSet fileSet) {
         char[] c = name.toCharArray();
         Object[] obj = null;
 
         populateCache();
-        for (CharArrayObjectMap map : mapsToNameOrBinding) {
+        for (CharArrayObjectMap<?> map : mapsToNameOrBinding) {
         	if (prefixLookup) {
         		IContentAssistMatcher matcher = ContentAssistMatcherFactory.getInstance().createMatcher(c);
         		Object[] keys = map.keyArray();
@@ -366,7 +376,7 @@ public class CScope implements ICScope, IASTInternalScope {
         
 		for (Object element : obj) {
 			if (element instanceof IBinding) {
-				result = (IBinding[]) ArrayUtil.append(IBinding.class, result, element);
+				result = ArrayUtil.append(IBinding.class, result, (IBinding) element);
 			} else {
 				IASTName n= null;
 				if (element instanceof IASTName) {
@@ -382,13 +392,13 @@ public class CScope implements ICScope, IASTInternalScope {
 						}
 					}
 					if (b != null) {
-						result = (IBinding[]) ArrayUtil.append(IBinding.class, result, b);
+						result = ArrayUtil.append(IBinding.class, result, b);
 					}
 				}
 			}
 		}
 
-        return (IBinding[]) ArrayUtil.trim(IBinding.class, result);
+        return ArrayUtil.trim(IBinding.class, result);
     }
     
     /**
@@ -405,7 +415,8 @@ public class CScope implements ICScope, IASTInternalScope {
     	return bindings[0];
     }
         
-    public void populateCache() {
+    @Override
+	public void populateCache() {
     	if (isCached)
     		return;
     	
@@ -413,6 +424,45 @@ public class CScope implements ICScope, IASTInternalScope {
     	isCached= true;
     }
     
+	@Override
+	public void removeNestedFromCache(IASTNode container) {
+		if (mapsToNameOrBinding != null) {
+			removeFromMap(mapsToNameOrBinding[0], container);
+			removeFromMap(mapsToNameOrBinding[1], container);
+		}
+	}
+	
+	private void removeFromMap(CharArrayObjectMap<?> map, IASTNode container) {
+		for (int i = 0; i < map.size(); i++) {
+			Object o= map.getAt(i);
+			if (o instanceof IASTName) {
+				if (container.contains((IASTNode) o)) {
+					final char[] key = map.keyAt(i);
+					map.remove(key, 0, key.length);
+					i--;
+				}
+			} else if (o instanceof IASTName[]) {
+				final IASTName[] set = (IASTName[]) o;
+				removeFromSet(set, container);
+			}
+		}
+	}
+
+	private void removeFromSet(IASTName[] set, IASTNode container) {
+		int j= 0;
+		for (int i = 0; i < set.length; i++) {
+			IASTName n= set[i];
+			if (n == null)
+				break;
+			if (container.contains(n)) {
+				set[i]= null;
+			} else if (i != j) {
+				set[j++]= n;
+				set[i]= null;
+			}
+		}
+	}
+
 	protected void doPopulateCache() {
 		final IASTNode scopeNode = physicalNode;
 		IASTNode[] nodes = null;
@@ -569,6 +619,7 @@ public class CScope implements ICScope, IASTInternalScope {
 	/* (non-Javadoc)
 	 * @see org.eclipse.cdt.core.dom.ast.IScope#getScopeName()
 	 */
+	@Override
 	public IName getScopeName() {
 		if (physicalNode instanceof IASTCompositeTypeSpecifier) {
 			return ((IASTCompositeTypeSpecifier) physicalNode).getName();
@@ -576,15 +627,17 @@ public class CScope implements ICScope, IASTInternalScope {
 		return null;
 	}
 
+	@Override
 	public void addBinding(IBinding binding) {
 		int type = NAMESPACE_TYPE_OTHER;
 		if (binding instanceof ICompositeType || binding instanceof IEnumeration) {
 			type = NAMESPACE_TYPE_TAG;
 		}
 
-		CharArrayObjectMap map = mapsToNameOrBinding[type];
+		@SuppressWarnings("unchecked")
+		CharArrayObjectMap<Object> map = (CharArrayObjectMap<Object>) mapsToNameOrBinding[type];
 		if (map == CharArrayObjectMap.EMPTY_MAP)
-			map = mapsToNameOrBinding[type] = new CharArrayObjectMap(2);
+			mapsToNameOrBinding[type] = map= new CharArrayObjectMap<Object>(2);
 
 		map.put(binding.getNameCharArray(), binding);
 	}
