@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2005, 2011 IBM Corporation and others.
+ * Copyright (c) 2005, 2012 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -11,6 +11,7 @@
  *     Bryan Wilkinson (QNX)
  *     Andrew Ferguson (Symbian)
  *     Jens Elmenthaler - http://bugs.eclipse.org/173458 (camel case completion)
+ *     Sergey Prigogin (Google)
  *******************************************************************************/
 package org.eclipse.cdt.internal.core.dom.parser.c;
 
@@ -22,6 +23,7 @@ import org.eclipse.cdt.core.dom.ast.ASTVisitor;
 import org.eclipse.cdt.core.dom.ast.DOMException;
 import org.eclipse.cdt.core.dom.ast.IASTArrayDeclarator;
 import org.eclipse.cdt.core.dom.ast.IASTArrayModifier;
+import org.eclipse.cdt.core.dom.ast.IASTAttribute;
 import org.eclipse.cdt.core.dom.ast.IASTBinaryExpression;
 import org.eclipse.cdt.core.dom.ast.IASTCompositeTypeSpecifier;
 import org.eclipse.cdt.core.dom.ast.IASTCompoundStatement;
@@ -88,6 +90,7 @@ import org.eclipse.cdt.core.dom.ast.gnu.c.ICASTKnRFunctionDeclarator;
 import org.eclipse.cdt.core.index.IIndexBinding;
 import org.eclipse.cdt.core.index.IIndexFileSet;
 import org.eclipse.cdt.core.parser.util.ArrayUtil;
+import org.eclipse.cdt.core.parser.util.AttributeUtil;
 import org.eclipse.cdt.core.parser.util.CharArraySet;
 import org.eclipse.cdt.core.parser.util.CharArrayUtils;
 import org.eclipse.cdt.core.parser.util.IContentAssistMatcher;
@@ -98,6 +101,7 @@ import org.eclipse.cdt.internal.core.dom.parser.IASTInternalScope;
 import org.eclipse.cdt.internal.core.dom.parser.ITypeContainer;
 import org.eclipse.cdt.internal.core.dom.parser.ProblemBinding;
 import org.eclipse.cdt.internal.core.dom.parser.ProblemType;
+import org.eclipse.cdt.internal.core.dom.parser.SizeofCalculator;
 import org.eclipse.cdt.internal.core.parser.util.ContentAssistMatcherFactory;
 
 /**
@@ -440,9 +444,7 @@ public class CVisitor extends ASTQueries {
 	private static final String SIZE_T = "size_t"; //$NON-NLS-1$
 	private static final String PTRDIFF_T = "ptrdiff_t"; //$NON-NLS-1$
 	public static final String EMPTY_STRING = ""; //$NON-NLS-1$
-	public static final char[] EMPTY_CHAR_ARRAY = "".toCharArray(); //$NON-NLS-1$
-	
-	// definition lookup start location
+	// Definition lookup start location
 	protected static final int AT_BEGINNING = 1;
 	protected static final int AT_NEXT = 2; 
 
@@ -609,25 +611,30 @@ public class CVisitor extends ASTQueries {
 	    }
 		
 		if (type != null && type instanceof ICompositeType) {
-			if (type instanceof IIndexBinding) {
-				type= ((CASTTranslationUnit) fieldReference.getTranslationUnit()).mapToASTType((ICompositeType) type);
+			ICompositeType ct = (ICompositeType) type;
+			if (ct instanceof IIndexBinding) {
+				ct= ((CASTTranslationUnit) fieldReference.getTranslationUnit()).mapToASTType(ct);
 			}
 		    if (prefix) {
-		        IBinding[] result = null;
 		        char[] p = fieldReference.getFieldName().toCharArray();
-		        IContentAssistMatcher matcher = ContentAssistMatcherFactory.getInstance().createMatcher(p);
-				IField[] fields = ((ICompositeType) type).getFields();
-				for (IField field : fields) {
-					if (matcher.match(field.getNameCharArray())) {
-				        result = (IBinding[]) ArrayUtil.append(IBinding.class, result, field);
-				    }
-				}
-				return ArrayUtil.trim(IBinding.class, result);
+		        return findFieldsByPrefix(ct, p);
 		    } 
-			return ((ICompositeType) type).findField(fieldReference.getFieldName().toString());
+			return ct.findField(fieldReference.getFieldName().toString());
 		}
 		return null;
 	}
+
+	public static IBinding[] findFieldsByPrefix(final ICompositeType ct, char[] p) {
+		IBinding[] result = null;
+		IContentAssistMatcher matcher = ContentAssistMatcherFactory.getInstance().createMatcher(p);
+		IField[] fields = ct.getFields();
+		for (IField field : fields) {
+			if (matcher.match(field.getNameCharArray())) {
+		        result = ArrayUtil.append(IBinding.class, result, field);
+		    }
+		}
+		return ArrayUtil.trim(IBinding.class, result);
+	}
 	
 	static IType getPtrDiffType(IASTBinaryExpression expr) {
 		IScope scope = getContainingScope(expr);
@@ -1079,7 +1086,7 @@ public class CVisitor extends ASTQueries {
 					final char[] n= b.getNameCharArray();
 					// consider binding only if no binding with the same name was found in another scope.
 					if (!handled.containsKey(n)) {
-						result= (IBinding[]) ArrayUtil.append(IBinding.class, result, b);
+						result= ArrayUtil.append(IBinding.class, result, b);
 					}
 				}
 				// store names of bindings
@@ -1091,7 +1098,7 @@ public class CVisitor extends ASTQueries {
 			scope= scope.getParent();
 		}
 		
-		return (IBinding[]) ArrayUtil.trim(IBinding.class, result);
+		return ArrayUtil.trim(IBinding.class, result);
 	}
 
 	private static IBinding externalBinding(IASTTranslationUnit tu, IASTName name) {
@@ -1199,14 +1206,15 @@ public class CVisitor extends ASTQueries {
 			node = node.getParent();
 		}
 		
-		if (node instanceof IASTParameterDeclaration)
-			declSpec = ((IASTParameterDeclaration) node).getDeclSpecifier();
-		else if (node instanceof IASTSimpleDeclaration)
+		if (node instanceof IASTSimpleDeclaration) {
 			declSpec = ((IASTSimpleDeclaration) node).getDeclSpecifier();
-		else if (node instanceof IASTFunctionDefinition)
+		} else if (node instanceof IASTParameterDeclaration) {
+			declSpec = ((IASTParameterDeclaration) node).getDeclSpecifier();
+		} else if (node instanceof IASTFunctionDefinition) {
 			declSpec = ((IASTFunctionDefinition) node).getDeclSpecifier();
-		else if (node instanceof IASTTypeId)
+		} else if (node instanceof IASTTypeId) {
 		    declSpec = ((IASTTypeId) node).getDeclSpecifier();
+		}
 	
 		boolean isParameter = (node instanceof IASTParameterDeclaration || node.getParent() instanceof ICASTKnRFunctionDeclarator); 
 		
@@ -1246,6 +1254,7 @@ public class CVisitor extends ASTQueries {
 	        return createType(baseType, (IASTFunctionDeclarator) declarator);
 		
 		IType type = baseType;
+		type = applyAttributes(type, declarator);
 		type = setupPointerChain(declarator.getPointerOperators(), type);
 		type = setupArrayChain(declarator, type);
 		
@@ -1256,6 +1265,49 @@ public class CVisitor extends ASTQueries {
 	    return type;
 	}
 	
+	private static IType applyAttributes(IType type, IASTDeclarator declarator) {
+		if (type instanceof IBasicType) {
+			IBasicType basicType = (IBasicType) type;
+			if (basicType.getKind() == IBasicType.Kind.eInt) {
+			    IASTAttribute[] attributes = declarator.getAttributes();
+				for (IASTAttribute attribute : attributes) {
+					char[] name = attribute.getName();
+					if (CharArrayUtils.equals(name, "__mode__") || CharArrayUtils.equals(name, "mode")) { //$NON-NLS-1$ //$NON-NLS-2$
+						char[] mode = AttributeUtil.getSimpleArgument(attribute);
+						if (CharArrayUtils.equals(mode, "__QI__") || CharArrayUtils.equals(mode, "QI")) { //$NON-NLS-1$ //$NON-NLS-2$
+							type = new CBasicType(IBasicType.Kind.eChar,
+									basicType.isUnsigned() ? IBasicType.IS_UNSIGNED : IBasicType.IS_SIGNED);
+						} else if (CharArrayUtils.equals(mode, "__HI__") || CharArrayUtils.equals(mode, "HI")) { //$NON-NLS-1$ //$NON-NLS-2$
+							type = new CBasicType(IBasicType.Kind.eInt,
+									IBasicType.IS_SHORT | getSignModifiers(basicType));
+						} else if (CharArrayUtils.equals(mode, "__SI__") || CharArrayUtils.equals(mode, "SI")) { //$NON-NLS-1$ //$NON-NLS-2$
+							type = new CBasicType(IBasicType.Kind.eInt, getSignModifiers(basicType));
+						} else if (CharArrayUtils.equals(mode, "__DI__") || CharArrayUtils.equals(mode, "DI")) { //$NON-NLS-1$ //$NON-NLS-2$
+							SizeofCalculator sizeofs = new SizeofCalculator(declarator.getTranslationUnit());
+							int modifier;
+							if (sizeofs.sizeof_long != null && sizeofs.sizeof_int != null &&
+									sizeofs.sizeof_long.size == 2 * sizeofs.sizeof_int.size) {
+								modifier = IBasicType.IS_LONG;
+							} else {
+								modifier = IBasicType.IS_LONG_LONG;
+							}
+							type = new CBasicType(IBasicType.Kind.eInt,
+									modifier | getSignModifiers(basicType));
+						} else if (CharArrayUtils.equals(mode, "__word__") || CharArrayUtils.equals(mode, "word")) { //$NON-NLS-1$ //$NON-NLS-2$
+							type = new CBasicType(IBasicType.Kind.eInt,
+									IBasicType.IS_LONG | getSignModifiers(basicType));
+						}
+					}
+				}
+			}
+		}
+		return type;
+	}
+
+	private static int getSignModifiers(IBasicType type) {
+		return type.getModifiers() & (IBasicType.IS_SIGNED | IBasicType.IS_UNSIGNED);
+	}
+
 	public static IType createType(IType returnType, IASTFunctionDeclarator declarator) {
 	    IType[] pTypes = getParmTypes(declarator);
 	    returnType = setupPointerChain(declarator.getPointerOperators(), returnType);
@@ -1467,6 +1519,8 @@ public class CVisitor extends ASTQueries {
         
         if (prop == IASTFieldReference.FIELD_NAME) {
             result = (IBinding[]) findBinding((IASTFieldReference) name.getParent(), isPrefix);
+        } else if (prop == ICASTFieldDesignator.FIELD_NAME) {
+            result = findBindingForContentAssist((ICASTFieldDesignator) name.getParent(), isPrefix);
         } else {
 	        IScope scope= getContainingScope(name);
 			try {
@@ -1478,10 +1532,38 @@ public class CVisitor extends ASTQueries {
 			} catch (DOMException e) {
 	        }
         }
-        return (IBinding[]) ArrayUtil.trim(IBinding.class, result);
+        return ArrayUtil.trim(IBinding.class, result);
     }
     
-    public static IBinding[] findBindings(IScope scope, String name) {
+	private static IBinding[] findBindingForContentAssist(ICASTFieldDesignator fd, boolean isPrefix) {
+		IASTNode blockItem = getContainingBlockItem(fd);
+		
+		IASTNode parent= blockItem;
+		while (parent != null && !(parent instanceof IASTSimpleDeclaration))
+			parent= parent.getParent();
+		
+		if (parent instanceof IASTSimpleDeclaration) {
+			IASTSimpleDeclaration simpleDecl = (IASTSimpleDeclaration) parent;
+			IBinding struct= null;
+			if (simpleDecl.getDeclSpecifier() instanceof IASTNamedTypeSpecifier) {
+				struct = ((IASTNamedTypeSpecifier) simpleDecl.getDeclSpecifier()).getName().resolveBinding();
+			} else if (simpleDecl.getDeclSpecifier() instanceof IASTElaboratedTypeSpecifier) {
+				struct = ((IASTElaboratedTypeSpecifier) simpleDecl.getDeclSpecifier()).getName().resolveBinding();
+			} else if (simpleDecl.getDeclSpecifier() instanceof IASTCompositeTypeSpecifier) {
+				struct = ((IASTCompositeTypeSpecifier) simpleDecl.getDeclSpecifier()).getName().resolveBinding();
+			}
+			if (struct instanceof IType) {
+				IType t= unwrapTypedefs((IType) struct);
+			
+				if (t instanceof ICompositeType) {
+					return findFieldsByPrefix((ICompositeType) t, fd.getName().toCharArray());
+				}
+			}
+		}
+		return null;
+	}
+	
+	public static IBinding[] findBindings(IScope scope, String name) {
         CASTName astName = new CASTName(name.toCharArray());
 	    
 	    // normal names
