@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2006, 2011 Wind River Systems, Inc. and others.
+ * Copyright (c) 2006, 2012 Wind River Systems, Inc. and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -17,13 +17,12 @@ package org.eclipse.cdt.internal.core.index;
 
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.BitSet;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
-import java.util.Iterator;
 import java.util.LinkedList;
 import java.util.List;
+import java.util.Set;
 import java.util.regex.Pattern;
 
 import org.eclipse.cdt.core.CCorePlugin;
@@ -42,6 +41,7 @@ import org.eclipse.cdt.core.index.IIndexInclude;
 import org.eclipse.cdt.core.index.IIndexMacro;
 import org.eclipse.cdt.core.index.IIndexName;
 import org.eclipse.cdt.core.index.IndexFilter;
+import org.eclipse.cdt.core.parser.ISignificantMacros;
 import org.eclipse.cdt.internal.core.dom.Linkage;
 import org.eclipse.cdt.internal.core.index.composite.CompositingNotImplementedError;
 import org.eclipse.cdt.internal.core.index.composite.ICompositesFactory;
@@ -60,19 +60,20 @@ public class CIndex implements IIndex {
 	private static final boolean SPECIALCASE_SINGLES = true;
 
 	private final IIndexFragment[] fFragments;
-	private final int fPrimaryFragmentCount;
 	private int fReadLock;
 	private ICompositesFactory cppCF, cCF, fCF;
 
-	public CIndex(IIndexFragment[] fragments, int primaryFragmentCount) {
-		fFragments= fragments;
-		fPrimaryFragmentCount= primaryFragmentCount;
-	}
-
+	/**
+	 * Creates an index consisting of one or more fragments.
+	 * 
+	 * @param fragments Fragments constituting the index. If there are extended fragments,
+	 * they are located in the array after the PDOM fragments for the same project. 
+	 */
 	public CIndex(IIndexFragment[] fragments) {
-		this(fragments, fragments.length);
+		fFragments= fragments;
 	}
 
+	@Override
 	public IIndexBinding findBinding(IName name) throws CoreException {
 		if (name instanceof IIndexFragmentName) {
 			return adaptBinding(((IIndexFragmentName) name).getBinding());
@@ -80,35 +81,38 @@ public class CIndex implements IIndex {
 			if (SPECIALCASE_SINGLES && fFragments.length == 1) {
 				return fFragments[0].findBinding((IASTName) name);
 			} else {
-				for (int i = 0; i < fPrimaryFragmentCount; i++) {
-					IIndexFragmentBinding binding= fFragments[i].findBinding((IASTName) name);
+				for (IIndexFragment fragment : fFragments) {
+					IIndexFragmentBinding binding= fragment.findBinding((IASTName) name);
 					if (binding != null) {
 						return getCompositesFactory(binding.getLinkage().getLinkageID()).getCompositeBinding(binding);
 					}
-				}
+				} 
 			}
 		}
 		return null;
 	}
 
+	@Override
 	public IIndexBinding[] findBindings(Pattern pattern, boolean isFullyQualified, IndexFilter filter,
 			IProgressMonitor monitor) throws CoreException {
 		return findBindings(new Pattern[] { pattern }, isFullyQualified, filter, monitor);
 	}
 
+	@Override
 	public IIndexBinding[] findBindings(Pattern[] patterns, boolean isFullyQualified, IndexFilter filter,
 			IProgressMonitor monitor) throws CoreException {
 		if (SPECIALCASE_SINGLES && fFragments.length == 1) {
-			 return fFragments[0].findBindings(patterns, isFullyQualified, filter, monitor);
+			return fFragments[0].findBindings(patterns, isFullyQualified, filter, monitor);
 		} else {
 			List<IIndexBinding[]> result = new ArrayList<IIndexBinding[]>();
 			ILinkage[] linkages = Linkage.getIndexerLinkages();
 			for (ILinkage linkage : linkages) {
 				if (filter.acceptLinkage(linkage)) {
-					IIndexFragmentBinding[][] fragmentBindings = new IIndexFragmentBinding[fPrimaryFragmentCount][];
-					for (int i = 0; i < fPrimaryFragmentCount; i++) {
+					IIndexFragmentBinding[][] fragmentBindings = new IIndexFragmentBinding[fFragments.length][];
+					for (int i = 0; i < fFragments.length; i++) {
 						try {
-							IBinding[] part = fFragments[i].findBindings(patterns, isFullyQualified, retargetFilter(linkage, filter), monitor);
+							IBinding[] part = fFragments[i].findBindings(patterns, isFullyQualified,
+									retargetFilter(linkage, filter), monitor);
 							fragmentBindings[i] = new IIndexFragmentBinding[part.length];
 							System.arraycopy(part, 0, fragmentBindings[i], 0, part.length);
 						} catch (CoreException e) {
@@ -124,6 +128,7 @@ public class CIndex implements IIndex {
 		}
 	}
 
+	@Override
 	public IIndexBinding[] findMacroContainers(Pattern pattern, IndexFilter filter, IProgressMonitor monitor)
 			throws CoreException {
 		if (SPECIALCASE_SINGLES && fFragments.length == 1) {
@@ -133,8 +138,8 @@ public class CIndex implements IIndex {
 			ILinkage[] linkages = Linkage.getIndexerLinkages();
 			for (ILinkage linkage : linkages) {
 				if (filter.acceptLinkage(linkage)) {
-					IIndexFragmentBinding[][] fragmentBindings = new IIndexFragmentBinding[fPrimaryFragmentCount][];
-					for (int i = 0; i < fPrimaryFragmentCount; i++) {
+					IIndexFragmentBinding[][] fragmentBindings = new IIndexFragmentBinding[fFragments.length][];
+					for (int i = 0; i < fFragments.length; i++) {
 						try {
 							IBinding[] part = fFragments[i].findMacroContainers(pattern,
 									retargetFilter(linkage, filter), monitor);
@@ -153,6 +158,7 @@ public class CIndex implements IIndex {
 		}
 	}
 
+	@Override
 	public IIndexName[] findNames(IBinding binding, int flags) throws CoreException {
 		LinkedList<IIndexFragmentName> result= new LinkedList<IIndexFragmentName>();
 		if (binding instanceof ICPPUsingDeclaration) {
@@ -170,47 +176,56 @@ public class CIndex implements IIndex {
 			binding= bindings[0];
 		}
 
-		int fragCount= 0;
-		for (int i = 0; i < fPrimaryFragmentCount; i++) {
-			final IIndexFragmentName[] names = fFragments[i].findNames(binding, flags);
-			if (names.length > 0) {
-				result.addAll(Arrays.asList(names));
-				fragCount++;
+		// Collect the names from all fragments. Since the same file may be represented by multiple
+		// variants in one or more index fragments, we need to filter out duplicate names.
+		// See bug 192352. 
+		// Read only fragments can be superseded by what the indexer writes into
+		// a writable fragment. Therefore names from a read-only fragement are
+		// ignored if there is a match in a writable fragment.
+		HashSet<NameKey> encounteredNames = new HashSet<NameKey>();
+		for (IIndexFragment fragment : fFragments) {
+			if (fragment instanceof IWritableIndexFragment) {
+				final IIndexFragmentName[] names = fragment.findNames(binding, flags);
+				for (IIndexFragmentName name : names) {
+					if (encounteredNames.add(new NameKey(name, true))) {
+						result.add(name);
+					}
+				}
 			}
 		}
-		// bug 192352, files can reside in multiple fragments, remove duplicates
-		if (fragCount > 1 || (flags & IIndex.SEARCH_ACROSS_LANGUAGE_BOUNDARIES) != 0) {
-			HashMap<String, IIndexFile> fileMap= new HashMap<String, IIndexFile>();
-			for (Iterator<IIndexFragmentName> iterator = result.iterator(); iterator.hasNext();) {
-				final IIndexFragmentName name = iterator.next();
-				final IIndexFile file= name.getFile();
-				final String fileKey= name.getFile().getLocation().getURI().toString();
-				final IIndexFile otherFile= fileMap.get(fileKey);
-				if (otherFile == null) {
-					fileMap.put(fileKey, file);
-				} else if (!otherFile.equals(file)) { // same file in another fragment
-					iterator.remove();
+		for (IIndexFragment fragment : fFragments) {
+			if (!(fragment instanceof IWritableIndexFragment)) {
+				final IIndexFragmentName[] names = fragment.findNames(binding, flags);
+				for (IIndexFragmentName name : names) {
+					if (encounteredNames.add(new NameKey(name, false))) {
+						result.add(name);
+					}
 				}
 			}
 		}
 		return result.toArray(new IIndexName[result.size()]);
 	}
 
+	@Override
 	public IIndexName[] findDeclarations(IBinding binding) throws CoreException {
 		return findNames(binding, FIND_DECLARATIONS_DEFINITIONS);
 	}
 
+	@Override
 	public IIndexName[] findDefinitions(IBinding binding) throws CoreException {
 		return findNames(binding, FIND_DEFINITIONS);
 	}
 
+	@Override
 	public IIndexName[] findReferences(IBinding binding) throws CoreException {
 		return findNames(binding, FIND_REFERENCES);
 	}
 
+	@Override
+	@Deprecated
 	public IIndexFile getFile(int linkageID, IIndexFileLocation location) throws CoreException {
-		for (int i = 0; i < fPrimaryFragmentCount; i++) {
-			IIndexFragmentFile candidate= fFragments[i].getFile(linkageID, location);
+		for (IIndexFragment fragment : fFragments) {
+			IIndexFragmentFile candidate= fragment.getFile(linkageID, location);
 			if (candidate != null && candidate.hasContent()) {
 				return candidate;
 			}
@@ -218,19 +233,33 @@ public class CIndex implements IIndex {
 		return null;
 	}
 
-	public IIndexFile[] getFiles(IIndexFileLocation location) throws CoreException {
+	@Override
+	public IIndexFile getFile(int linkageID, IIndexFileLocation location,
+			ISignificantMacros significantMacros) throws CoreException {
+		for (IIndexFragment fragment : fFragments) {
+			IIndexFragmentFile candidate= fragment.getFile(linkageID, location, significantMacros);
+			if (candidate != null && candidate.hasContent()) {
+				return candidate;
+			}
+		}
+		return null;
+	}
+
+	@Override
+	public IIndexFile[] getFiles(int linkageID, IIndexFileLocation location) throws CoreException {
 		if (location == null) {
 			return IIndexFile.EMPTY_FILE_ARRAY;
 		}
+		Set<ISignificantMacros> handled = new HashSet<ISignificantMacros>();
 		ArrayList<IIndexFragmentFile> result= new ArrayList<IIndexFragmentFile>();
-		BitSet linkages= new BitSet();
-		for (int i = 0; i < fPrimaryFragmentCount; i++) {
-			IIndexFragmentFile[] candidates= fFragments[i].getFiles(location);
+		for (IIndexFragment fragment : fFragments) {
+			IIndexFragmentFile[] candidates= fragment.getFiles(linkageID, location);
 			for (IIndexFragmentFile candidate : candidates) {
-				int linkage= candidate.getLinkageID();
-				if (!linkages.get(linkage) && candidate.hasContent()) {
-					result.add(candidate);
-					linkages.set(linkage);
+				if (candidate.hasContent()) {
+					ISignificantMacros macroKey = candidate.getSignificantMacros();
+					if (handled.add(macroKey)) {
+						result.add(candidate);
+					}
 				}
 			}
 		}
@@ -240,41 +269,70 @@ public class CIndex implements IIndex {
 		return result.toArray(new IIndexFile[result.size()]);
 	}
 
-	public IIndexFile resolveInclude(IIndexInclude include) throws CoreException {
-		if (!include.isResolved()) {
-			return null;
+	@Override
+	public IIndexFile[] getFiles(IIndexFileLocation location) throws CoreException {
+		if (location == null) {
+			return IIndexFile.EMPTY_FILE_ARRAY;
 		}
+		Set<FileContentKey> keys = new HashSet<FileContentKey>();
+		ArrayList<IIndexFragmentFile> result= new ArrayList<IIndexFragmentFile>();
+		for (IIndexFragment fragment : fFragments) {
+			IIndexFragmentFile[] candidates= fragment.getFiles(location);
+			for (IIndexFragmentFile candidate : candidates) {
+				if (candidate.hasContent()) {
+					if (keys.add(new FileContentKey(candidate.getLinkageID(), candidate.getLocation(), candidate.getSignificantMacros()))) {
+						result.add(candidate);
+					}
+				}
+			}
+		}
+		if (result.isEmpty()) {
+			return IIndexFile.EMPTY_FILE_ARRAY;
+		}
+		return result.toArray(new IIndexFile[result.size()]);
+	}
+
+	@Override
+	public IIndexFile resolveInclude(IIndexInclude include) throws CoreException {
 		IIndexFragmentInclude fragmentInclude = (IIndexFragmentInclude) include;
 		IIndexFragmentFile result= fragmentInclude.getIncludes();
-		if (result != null && result.hasContent()) {
+		if (result == null)
+			return null;
+		
+		if (result.hasContent()) {
 			return result;
 		}
 
-		return getFile(include.getIncludedBy().getLinkageID(), include.getIncludesLocation());
+		return getFile(result.getLinkageID(), result.getLocation(), result.getSignificantMacros());
 	}
 
+	@Override
 	public IIndexInclude[] findIncludedBy(IIndexFile file) throws CoreException {
 		return findIncludedBy(file, 0);
 	}
 
+	@Override
 	public IIndexInclude[] findIncludedBy(IIndexFile file, int depth) throws CoreException {
 		List<IIndexInclude> result= new ArrayList<IIndexInclude>();
-		findIncludedBy(Collections.singletonList(file), result, depth, new HashSet<IIndexFileLocation>());
+		findIncludedBy(file.getLinkageID(), Collections.singletonList(file), result, depth,
+				new HashSet<FileContentKey>());
 		return result.toArray(new IIndexInclude[result.size()]);
 	}
 
-	public void findIncludedBy(List<IIndexFile> in, List<IIndexInclude> out, int depth,
-			HashSet<IIndexFileLocation> handled) throws CoreException {
+	public void findIncludedBy(int linkageID, List<IIndexFile> in, List<IIndexInclude> out, int depth,
+			HashSet<FileContentKey> handled) throws CoreException {
 		List<IIndexFile> nextLevel= depth != 0 ? new LinkedList<IIndexFile>() : null;
 		for (IIndexFile iIndexFile : in) {
 			IIndexFragmentFile file = (IIndexFragmentFile) iIndexFile;
-			for (int j = 0; j < fPrimaryFragmentCount; j++) {
-				IIndexInclude[] includedBy= fFragments[j].findIncludedBy(file);
+			for (IIndexFragment fragment : fFragments) {
+				IIndexInclude[] includedBy= fragment.findIncludedBy(file);
 				for (IIndexInclude include : includedBy) {
-					if (handled.add(include.getIncludedByLocation())) {
+					final IIndexFile includer = include.getIncludedBy();
+					FileContentKey key= new FileContentKey(linkageID, includer.getLocation(), includer.getSignificantMacros());
+					if (handled.add(key)) {
 						out.add(include);
 						if (nextLevel != null) {
-							nextLevel.add(include.getIncludedBy());
+							nextLevel.add(includer);
 						}
 					}
 				}
@@ -286,14 +344,15 @@ public class CIndex implements IIndex {
 		if (depth > 0) {
 			depth--;
 		}
-		findIncludedBy(nextLevel, out, depth, handled);
+		findIncludedBy(linkageID, nextLevel, out, depth, handled);
 	}
 
-
+	@Override
 	public IIndexInclude[] findIncludes(IIndexFile file) throws CoreException {
 		return findIncludes(file, 0);
 	}
 
+	@Override
 	public IIndexInclude[] findIncludes(IIndexFile file, int depth) throws CoreException {
 		List<IIndexInclude> result= new ArrayList<IIndexInclude>();
 		findIncludes(Collections.singletonList(file), result, depth, new HashSet<Object>());
@@ -329,6 +388,7 @@ public class CIndex implements IIndex {
 		findIncludes(nextLevel, out, depth, handled);
 	}
 
+	@Override
 	public synchronized void acquireReadLock() throws InterruptedException {
 		if (++fReadLock == 1) {
 			int i= 0;
@@ -348,6 +408,7 @@ public class CIndex implements IIndex {
 		}
 	}
 
+	@Override
 	public synchronized void releaseReadLock() {
 		if (--fReadLock == 0) {
 			for (IIndexFragment fragment : fFragments) {
@@ -360,6 +421,7 @@ public class CIndex implements IIndex {
 		return fReadLock;
 	}
 
+	@Override
 	public boolean hasWaitingReaders() {
 		for (IIndexFragment fragment : fFragments) {
 			if (fragment.hasWaitingReaders()) {
@@ -369,6 +431,7 @@ public class CIndex implements IIndex {
 		return false;
 	}
 
+	@Override
 	public long getLastWriteAccess() {
 		long result= 0;
 		for (IIndexFragment fragment : fFragments) {
@@ -377,6 +440,7 @@ public class CIndex implements IIndex {
 		return result;
 	}
 
+	@Override
 	public IIndexBinding[] findBindings(char[][] names, IndexFilter filter, IProgressMonitor monitor)
 			throws CoreException {
 		if (SPECIALCASE_SINGLES && fFragments.length == 1) {
@@ -395,11 +459,11 @@ public class CIndex implements IIndex {
 			monitor.beginTask(Messages.CIndex_FindBindingsTask_label, fFragments.length * linkages.length);
 			for (ILinkage linkage : linkages) {
 				if (filter.acceptLinkage(linkage)) {
-					IIndexFragmentBinding[][] fragmentBindings = new IIndexFragmentBinding[fPrimaryFragmentCount][];
-					for (int i = 0; i < fPrimaryFragmentCount; i++) {
+					IIndexFragmentBinding[][] fragmentBindings = new IIndexFragmentBinding[fFragments.length][];
+					for (int i = 0; i < fFragments.length; i++) {
 						try {
-							IBinding[] part = fFragments[i].findBindings(names, retargetFilter(linkage, filter),
-									new SubProgressMonitor(monitor, 1));
+							IBinding[] part = fFragments[i].findBindings(names,
+									retargetFilter(linkage, filter), new SubProgressMonitor(monitor, 1));
 							fragmentBindings[i] = new IIndexFragmentBinding[part.length];
 							System.arraycopy(part, 0, fragmentBindings[i], 0, part.length);
 						} catch (CoreException e) {
@@ -413,17 +477,17 @@ public class CIndex implements IIndex {
 			}
 			monitor.done();
 			return flatten(result);
-
 		}
 	}
 
+	@Override
 	public IIndexBinding adaptBinding(IBinding binding) {
 		try {
 			if (SPECIALCASE_SINGLES && fFragments.length == 1) {
 				return fFragments[0].adaptBinding(binding);
 			} else {
-				for (int i = 0; i < fPrimaryFragmentCount; i++) {
-					IIndexFragmentBinding adaptedBinding= fFragments[i].adaptBinding(binding);
+				for (IIndexFragment fragment : fFragments) {
+					IIndexFragmentBinding adaptedBinding= fragment.adaptBinding(binding);
 					if (adaptedBinding != null) {
 						return getCompositesFactory(binding.getLinkage().getLinkageID()).getCompositeBinding(adaptedBinding);
 					}
@@ -435,6 +499,7 @@ public class CIndex implements IIndex {
 		return null;
 	}
 
+	@Override
 	public IIndexBinding[] findBindings(char[] name, IndexFilter filter, IProgressMonitor monitor)
 			throws CoreException {
 		return findBindings(name, true, filter, monitor);
@@ -459,10 +524,8 @@ public class CIndex implements IIndex {
 		return result;
 	}
 
-	public IIndexFragment[] getPrimaryFragments() {
-		IIndexFragment[] result= new IIndexFragment[fPrimaryFragmentCount];
-		System.arraycopy(fFragments, 0, result, 0, fPrimaryFragmentCount);
-		return result;
+	public IIndexFragment[] getFragments() {
+		return fFragments;
 	}
 
 	public IIndexFragmentBinding[] findEquivalentBindings(IBinding binding) throws CoreException {
@@ -480,19 +543,19 @@ public class CIndex implements IIndex {
 		switch (linkageID) {
 		case ILinkage.CPP_LINKAGE_ID:
 			if (cppCF == null) {
-				cppCF = new CPPCompositesFactory(new CIndex(fFragments, fFragments.length));
+				cppCF = new CPPCompositesFactory(this);
 			}
 			return cppCF;
 
 		case ILinkage.C_LINKAGE_ID:
 			if (cCF == null) {
-				cCF = new CCompositesFactory(new CIndex(fFragments, fFragments.length));
+				cCF = new CCompositesFactory(this);
 			}
 			return cCF;
 
 		case ILinkage.FORTRAN_LINKAGE_ID:
 			if (fCF == null) {
-				fCF = new CCompositesFactory(new CIndex(fFragments, fFragments.length));
+				fCF = new CCompositesFactory(this);
 			}
 			// This is a placeholder - it will throw CompositingNotImplementedError
 			// if non-empty (non-c) results are returned by a fragment
@@ -514,6 +577,7 @@ public class CIndex implements IIndex {
 		};
 	}
 
+	@Override
 	public IIndexBinding[] findBindingsForPrefix(char[] prefix, boolean filescope, IndexFilter filter,
 			IProgressMonitor monitor) throws CoreException {
 		if (SPECIALCASE_SINGLES && fFragments.length == 1) {
@@ -523,8 +587,8 @@ public class CIndex implements IIndex {
 			ILinkage[] linkages = Linkage.getIndexerLinkages();
 			for (ILinkage linkage : linkages) {
 				if (filter.acceptLinkage(linkage)) {
-					IIndexFragmentBinding[][] fragmentBindings = new IIndexFragmentBinding[fPrimaryFragmentCount][];
-					for (int i = 0; i < fPrimaryFragmentCount; i++) {
+					IIndexFragmentBinding[][] fragmentBindings = new IIndexFragmentBinding[fFragments.length][];
+					for (int i = 0; i < fFragments.length; i++) {
 						try {
 							IBinding[] part = fFragments[i].findBindingsForPrefix(prefix, filescope,
 									retargetFilter(linkage, filter), monitor);
@@ -543,6 +607,7 @@ public class CIndex implements IIndex {
 		}
 	}
 
+	@Override
 	public IIndexBinding[] findBindingsForContentAssist(char[] prefix, boolean filescope,
 			IndexFilter filter, IProgressMonitor monitor) throws CoreException {
 		if (SPECIALCASE_SINGLES && fFragments.length == 1) {
@@ -552,8 +617,8 @@ public class CIndex implements IIndex {
 			ILinkage[] linkages = Linkage.getIndexerLinkages();
 			for (ILinkage linkage : linkages) {
 				if (filter.acceptLinkage(linkage)) {
-					IIndexFragmentBinding[][] fragmentBindings = new IIndexFragmentBinding[fPrimaryFragmentCount][];
-					for (int i = 0; i < fPrimaryFragmentCount; i++) {
+					IIndexFragmentBinding[][] fragmentBindings = new IIndexFragmentBinding[fFragments.length][];
+					for (int i = 0; i < fFragments.length; i++) {
 						try {
 							IBinding[] part = fFragments[i].findBindingsForContentAssist(prefix,
 									filescope, retargetFilter(linkage, filter), monitor);
@@ -572,6 +637,7 @@ public class CIndex implements IIndex {
 		}
 	}
 
+	@Override
 	public IIndexBinding[] findBindings(char[] name, boolean filescope, IndexFilter filter, IProgressMonitor monitor)
 			throws CoreException {
 		if (SPECIALCASE_SINGLES && fFragments.length == 1) {
@@ -581,8 +647,8 @@ public class CIndex implements IIndex {
 			ILinkage[] linkages = Linkage.getIndexerLinkages();
 			for (ILinkage linkage : linkages) {
 				if (filter.acceptLinkage(linkage)) {
-					IIndexFragmentBinding[][] fragmentBindings = new IIndexFragmentBinding[fPrimaryFragmentCount][];
-					for (int i = 0; i < fPrimaryFragmentCount; i++) {
+					IIndexFragmentBinding[][] fragmentBindings = new IIndexFragmentBinding[fFragments.length][];
+					for (int i = 0; i < fFragments.length; i++) {
 						try {
 							IBinding[] part = fFragments[i].findBindings(name, filescope,
 									retargetFilter(linkage, filter), monitor);
@@ -601,10 +667,12 @@ public class CIndex implements IIndex {
 		}
 	}
 
+	@Override
 	public IIndexMacro[] findMacros(char[] name, IndexFilter filter, IProgressMonitor monitor) throws CoreException {
 		return findMacros(name, false, true, filter, monitor);
 	}
 
+	@Override
 	public IIndexMacro[] findMacrosForPrefix(char[] name, IndexFilter filter, IProgressMonitor monitor)
 			throws CoreException {
 		return findMacros(name, true, false, filter, monitor);
@@ -612,7 +680,7 @@ public class CIndex implements IIndex {
 
 	private IIndexMacro[] findMacros(char[] name, boolean isPrefix, boolean caseSensitive,
 			IndexFilter filter, IProgressMonitor monitor) throws CoreException {
-		// macros can be represented multiple times when a header is parsed in c- and c++ context,
+		// Macros can be represented multiple times when a header is parsed in c- and c++ context,
 		// so there is no special case for indexes with single fragments.
 		if (monitor == null) {
 			monitor= new NullProgressMonitor();
@@ -620,10 +688,10 @@ public class CIndex implements IIndex {
 		List<IIndexMacro> result = new ArrayList<IIndexMacro>();
 		HashSet<IIndexFileLocation> handledIFLs= new HashSet<IIndexFileLocation>();
 		monitor.beginTask(Messages.CIndex_FindBindingsTask_label, fFragments.length);
-		for (int i = 0; i < fPrimaryFragmentCount; i++) {
+		for (IIndexFragment fragment : fFragments) {
 			HashSet<IIndexFile> allowedFiles= new HashSet<IIndexFile>();
 			try {
-				IIndexMacro[] macros= fFragments[i].findMacros(name, isPrefix, caseSensitive, filter,
+				IIndexMacro[] macros= fragment.findMacros(name, isPrefix, caseSensitive, filter,
 						new SubProgressMonitor(monitor, 1));
 				for (IIndexMacro indexMacro : macros) {
 					IIndexFile file= indexMacro.getFile();
@@ -672,41 +740,122 @@ public class CIndex implements IIndex {
 		}
 	}
 
+	@Override
 	public IIndexFileSet createFileSet() {
 		return new IndexFileSet();
 	}
 
+	@Override
 	public IIndexFile[] getAllFiles() throws CoreException {
 		HashMap<IIndexFileLocation, IIndexFile> result= new HashMap<IIndexFileLocation, IIndexFile>();
 		for (IIndexFragment fragment : fFragments) {
 			for (IIndexFragmentFile file : fragment.getAllFiles()) {
-				result.put(file.getLocation(), file);
+				if (file.hasContent()) { 
+					result.put(file.getLocation(), file);
+				}
 			}
 		}
 		return result.values().toArray(new IIndexFile[result.size()]);
 	}
 
-	/* (non-Javadoc)
-	 * @see org.eclipse.cdt.core.index.IIndex#getInlineNamespaces()
-	 */
+	@Override
+	public IIndexFile[] getDefectiveFiles() throws CoreException {
+		HashMap<IIndexFileLocation, IIndexFile> result= new HashMap<IIndexFileLocation, IIndexFile>();
+		for (IIndexFragment fragment : fFragments) {
+			for (IIndexFragmentFile file : fragment.getDefectiveFiles()) {
+				if (file.hasContent()) { 
+					result.put(file.getLocation(), file);
+				}
+			}
+		}
+		return result.values().toArray(new IIndexFile[result.size()]);
+	}
+
+	@Override
+	public IIndexFile[] getFilesWithUnresolvedIncludes() throws CoreException {
+		HashMap<IIndexFileLocation, IIndexFile> result= new HashMap<IIndexFileLocation, IIndexFile>();
+		for (IIndexFragment fragment : fFragments) {
+			for (IIndexFragmentFile file : fragment.getFilesWithUnresolvedIncludes()) {
+				if (file.hasContent()) { 
+					result.put(file.getLocation(), file);
+				}
+			}
+		}
+		return result.values().toArray(new IIndexFile[result.size()]);
+	}
+
+	@Override
 	public IIndexScope[] getInlineNamespaces() throws CoreException {
 		if (SPECIALCASE_SINGLES && fFragments.length == 1) {
 			return fFragments[0].getInlineNamespaces();
 		}
 
-		IIndexFragmentBinding[][] preresult = new IIndexFragmentBinding[fPrimaryFragmentCount][];
-		for (int i = 0; i < fPrimaryFragmentCount; i++) {
+		IIndexFragmentBinding[][] preresult = new IIndexFragmentBinding[fFragments.length][];
+		for (int i = 0; i < fFragments.length; i++) {
 			IIndexScope[] raw = fFragments[i].getInlineNamespaces();
 			IIndexFragmentBinding[] arr = preresult[i] = new IIndexFragmentBinding[raw.length];
 			for (int j = 0; j < raw.length; j++) {
 				arr[j]= (IIndexFragmentBinding) raw[j].getScopeBinding();
 			}
 		}
-		IIndexBinding[] compBinding = getCompositesFactory(ILinkage.CPP_LINKAGE_ID).getCompositeBindings(preresult);
+		IIndexBinding[] compBinding =
+				getCompositesFactory(ILinkage.CPP_LINKAGE_ID).getCompositeBindings(preresult);
 		IIndexScope[] result = new IIndexScope[compBinding.length];
 		for (int i = 0; i < result.length; i++) {
 			result[i]= (IIndexScope) ((ICPPNamespace) compBinding[i]).getNamespaceScope();
 		}
 		return result;
 	}
+
+	@Override
+	public boolean isFullyInitialized() {
+		for (IIndexFragment fragment : fFragments) {
+			if (!fragment.isFullyInitialized())
+				return false;
+		}
+		return true;
+	}
+
+	/**
+	 * A key used to uniquely identify an IIndexFragmentName object. Uniqueness is guaranteed only
+	 * for names corresponding to the same binding.
+	 */
+	private static final class NameKey {
+		private final IIndexFileLocation location;
+		private final int linkageID;
+		private final int offset;
+		private final boolean inWritableFragment;
+
+		NameKey(IIndexFragmentName name, boolean writable) throws CoreException {
+			IIndexFile file = name.getFile();
+			location = file.getLocation();
+			linkageID = file.getLinkageID();
+			offset = name.getNodeOffset();
+			inWritableFragment= writable;
+		}
+
+		@Override
+		public int hashCode() {
+			return location.hashCode() * 31 + linkageID;
+		}
+
+		@Override
+		public boolean equals(Object obj) {
+			if (this == obj)
+				return true;
+			if (obj == null)
+				return false;
+			if (getClass() != obj.getClass())
+				return false;
+			NameKey other = (NameKey) obj;
+			// Offset is significant only if matches are from same kind of fragment
+			if (inWritableFragment == other.inWritableFragment && offset != other.offset)
+				return false;
+			if (linkageID != other.linkageID)
+				return false;
+			if (!location.equals(other.location))
+				return false;
+			return true;
+		}
+	}
 }
