@@ -1,18 +1,20 @@
 /*******************************************************************************
- * Copyright (c) 2005, 2011 QNX Software Systems and others.
+ * Copyright (c) 2005, 2012 QNX Software Systems and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors:
- *    Doug Schaefer (QNX) - Initial API and implementation
- *    Markus Schorn (Wind River Systems)
- *    Andrew Ferguson (Symbian)
- *    Sergey Prigogin (Google)
+ *     Doug Schaefer (QNX) - Initial API and implementation
+ *     Markus Schorn (Wind River Systems)
+ *     Andrew Ferguson (Symbian)
+ *     Sergey Prigogin (Google)
  *******************************************************************************/
 package org.eclipse.cdt.internal.core.pdom.dom.cpp;
 
+import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.LinkedList;
 import java.util.List;
 
@@ -125,10 +127,12 @@ class PDOMCPPLinkage extends PDOMLinkage implements IIndexCPPBindingConstants {
 		super(pdom, CPP_LINKAGE_NAME, CPP_LINKAGE_NAME.toCharArray());
 	}
 
+	@Override
 	public String getLinkageName() {
 		return CPP_LINKAGE_NAME;
 	}
 
+	@Override
 	public int getLinkageID() {
 		return CPP_LINKAGE_ID;
 	}
@@ -154,6 +158,7 @@ class PDOMCPPLinkage extends PDOMLinkage implements IIndexCPPBindingConstants {
 			postProcesses.add(this);
 		}
 		
+		@Override
 		public void run() {
 			for (int i = 0; i < fOriginal.length; i++) {
 				final IPDOMCPPTemplateParameter tp = fPersisted[i];
@@ -174,14 +179,13 @@ class PDOMCPPLinkage extends PDOMLinkage implements IIndexCPPBindingConstants {
 			postProcesses.add(this);
 		}
 		
+		@Override
 		public void run() {
 			try {
 				ICPPTemplateArgument[] args = binding.getTemplateArguments();
 				partial.setArguments(args);
 			} catch (CoreException e) {
 				CCorePlugin.log(e);
-			} catch (DOMException e) {
-				CCorePlugin.log(e);
 			} finally {
 				partial = null;
 				binding = null;
@@ -207,6 +211,7 @@ class PDOMCPPLinkage extends PDOMLinkage implements IIndexCPPBindingConstants {
 			postProcesses.add(this);
 		}
 		
+		@Override
 		public void run() {
 			for (int i = 0; i < fOriginalTemplateParameters.length; i++) {
 				final IPDOMCPPTemplateParameter tp = fTemplateParameters[i];
@@ -239,7 +244,7 @@ class PDOMCPPLinkage extends PDOMLinkage implements IIndexCPPBindingConstants {
 	}
 
 	@Override
-	public PDOMBinding addUnknownValue(IBinding binding) throws CoreException {
+	public PDOMBinding addPotentiallyUnknownBinding(IBinding binding) throws CoreException {
 		return addBinding(binding, null);
 	}
 
@@ -331,7 +336,7 @@ class PDOMCPPLinkage extends PDOMLinkage implements IIndexCPPBindingConstants {
 			return null;
 		
 		if (binding instanceof ICPPSpecialization) {
-			IBinding specialized = ((ICPPSpecialization)binding).getSpecializedBinding();
+			IBinding specialized = ((ICPPSpecialization) binding).getSpecializedBinding();
 			PDOMBinding pdomSpecialized= addBinding(specialized, null);
 			if (pdomSpecialized == null)
 				return null;
@@ -366,11 +371,11 @@ class PDOMCPPLinkage extends PDOMLinkage implements IIndexCPPBindingConstants {
 			}
 		} else if (binding instanceof ICPPConstructor) {
 			if (parent instanceof PDOMCPPClassType || parent instanceof PDOMCPPClassSpecialization) {
-				pdomBinding = new PDOMCPPConstructor(this, parent, (ICPPConstructor)binding);
+				pdomBinding = new PDOMCPPConstructor(this, parent, (ICPPConstructor) binding);
 			}
 		} else if (binding instanceof ICPPMethod) {
 			if (parent instanceof PDOMCPPClassType || parent instanceof PDOMCPPClassSpecialization) {
-				pdomBinding = new PDOMCPPMethod(this, parent, (ICPPMethod)binding);
+				pdomBinding = new PDOMCPPMethod(this, parent, (ICPPMethod) binding);
 			}
 		} else if (binding instanceof ICPPFunction) {
 			pdomBinding = new PDOMCPPFunction(this, parent, (ICPPFunction) binding, true);
@@ -392,7 +397,7 @@ class PDOMCPPLinkage extends PDOMLinkage implements IIndexCPPBindingConstants {
 				}
 			}
 		} else if (binding instanceof ITypedef) {
-			pdomBinding = new PDOMCPPTypedef(this, parent, (ITypedef)binding);
+			pdomBinding = new PDOMCPPTypedef(this, parent, (ITypedef) binding);
 		}
 
 		if (pdomBinding != null) {
@@ -461,6 +466,8 @@ class PDOMCPPLinkage extends PDOMLinkage implements IIndexCPPBindingConstants {
 			result= new PDOMCPPClassSpecialization(this, parent, (ICPPClassType) special, orig);
 		} else if (special instanceof ITypedef) {
 			result= new PDOMCPPTypedefSpecialization(this, parent, (ITypedef) special, orig);
+		} else if (special instanceof ICPPUsingDeclaration) {
+			result= new PDOMCPPUsingDeclarationSpecialization(this, parent, (ICPPUsingDeclaration) special, orig);
 		}
 		
 		return result;
@@ -471,6 +478,13 @@ class PDOMCPPLinkage extends PDOMLinkage implements IIndexCPPBindingConstants {
 			final long fileLocalRec= type.getLocalToFileRec();
 			IScope scope = binding.getCompositeScope();
 			if (scope instanceof ICPPClassScope) {
+				List<ICPPMethod> old= new ArrayList<ICPPMethod>();
+				if (type instanceof ICPPClassType) {
+					IScope oldScope = ((ICPPClassType)type).getCompositeScope();
+					if (oldScope instanceof ICPPClassScope) {
+						old.addAll(Arrays.asList(((ICPPClassScope) oldScope).getImplicitMethods()));
+					}
+				}
 				ICPPMethod[] implicit= ((ICPPClassScope) scope).getImplicitMethods();
 				for (ICPPMethod method : implicit) {
 					if (!(method instanceof IProblemBinding)) {
@@ -479,9 +493,14 @@ class PDOMCPPLinkage extends PDOMLinkage implements IIndexCPPBindingConstants {
 							pdomBinding = createBinding(type, method, fileLocalRec);
 						} else if (!getPDOM().hasLastingDefinition(pdomBinding)) {
 							pdomBinding.update(this, method);
+							old.remove(pdomBinding);
 						}
 					}
 				}
+				for (ICPPMethod method : old) {
+					if (method instanceof PDOMBinding)
+						((PDOMBinding) method).update(this, null);
+				}
 			}
 		} catch (DOMException e) {
 			CCorePlugin.log(e);
@@ -588,11 +607,12 @@ class PDOMCPPLinkage extends PDOMLinkage implements IIndexCPPBindingConstants {
 	}
 
 	@Override
-	public final PDOMBinding adaptBinding(final IBinding inputBinding) throws CoreException {
-		return adaptBinding(null, inputBinding, FILE_LOCAL_REC_DUMMY);
+	public final PDOMBinding adaptBinding(final IBinding inputBinding, boolean includeLocal) throws CoreException {
+		return adaptBinding(null, inputBinding, includeLocal ? FILE_LOCAL_REC_DUMMY : null);
 	}
 
-	private final PDOMBinding adaptBinding(final PDOMNode parent, IBinding inputBinding, long[] fileLocalRecHolder) throws CoreException {
+	private final PDOMBinding adaptBinding(final PDOMNode parent, IBinding inputBinding, long[] fileLocalRecHolder)
+			throws CoreException {
 		if (cannotAdapt(inputBinding)) {
 			return null;
 		}
@@ -620,7 +640,8 @@ class PDOMCPPLinkage extends PDOMLinkage implements IIndexCPPBindingConstants {
 		final Long key = record + CACHE_BASES;
 		PDOMCPPNamespace[] result= (PDOMCPPNamespace[]) getPDOM().getCachedResult(key);
 		if (result == null) {
-			List<PDOMCPPNamespace> nslist = PDOMCPPNamespace.collectInlineNamespaces(getDB(), getLinkage(), record+FIRST_NAMESPACE_CHILD_OFFSET);
+			List<PDOMCPPNamespace> nslist = PDOMCPPNamespace.collectInlineNamespaces(getDB(),
+					getLinkage(), record + FIRST_NAMESPACE_CHILD_OFFSET);
 			if (nslist == null) {
 				result= new PDOMCPPNamespace[0];
 			} else {
@@ -632,27 +653,18 @@ class PDOMCPPLinkage extends PDOMLinkage implements IIndexCPPBindingConstants {
 	}
 
 	/**
-	 * Find the equivalent binding, or binding place holder within this PDOM
+	 * Finds the equivalent binding, or binding placeholder within this PDOM.
 	 */
-	private final PDOMBinding doAdaptBinding(PDOMNode parent, IBinding binding, long[] fileLocalRecHolder) throws CoreException {
+	private final PDOMBinding doAdaptBinding(PDOMNode parent, IBinding binding, long[] fileLocalRecHolder)
+			throws CoreException {
 		if (parent == null) {
 			parent= adaptOrAddParent(false, binding);
 		}
-		PDOMNode inheritFileLocal= parent;
-		if (binding instanceof IEnumerator) {
-			try {
-				IType enumeration= ((IEnumerator) binding).getType();
-				if (enumeration instanceof IEnumeration) {
-					inheritFileLocal= adaptBinding((IEnumeration) enumeration);
-				}
-			} catch (DOMException e) {
-				CCorePlugin.log(e);
-			}
-		}
-
 		if (parent == this) {
 			PDOMBinding glob= CPPFindBinding.findBinding(getIndex(), this, binding, 0);
-			final long loc= getLocalToFileRec(inheritFileLocal, binding, glob);
+			if (fileLocalRecHolder == null)
+				return glob;
+			final long loc= getLocalToFileRec(parent, binding, glob);
 			if (loc == 0) 
 				return glob;
 			fileLocalRecHolder[0]= loc;
@@ -661,7 +673,9 @@ class PDOMCPPLinkage extends PDOMLinkage implements IIndexCPPBindingConstants {
 		if (parent instanceof PDOMCPPNamespace) {
 			final BTree btree = ((PDOMCPPNamespace) parent).getIndex();
 			PDOMBinding glob= CPPFindBinding.findBinding(btree, this, binding, 0);
-			final long loc= getLocalToFileRec(inheritFileLocal, binding, glob);
+			if (fileLocalRecHolder == null)
+				return glob;
+			final long loc= getLocalToFileRec(parent, binding, glob);
 			if (loc == 0) 
 				return glob;
 			fileLocalRecHolder[0]= loc;
@@ -673,7 +687,7 @@ class PDOMCPPLinkage extends PDOMLinkage implements IIndexCPPBindingConstants {
 		}
 		if (parent instanceof IPDOMMemberOwner) {
 			PDOMBinding glob= CPPFindBinding.findBinding(parent, this, binding, 0);
-			final long loc= getLocalToFileRec(inheritFileLocal, binding, glob);
+			final long loc= getLocalToFileRec(parent, binding, glob);
 			if (loc == 0) 
 				return glob;
 			fileLocalRecHolder[0]= loc;
@@ -817,6 +831,8 @@ class PDOMCPPLinkage extends PDOMLinkage implements IIndexCPPBindingConstants {
 			return new PDOMCPPClassTemplateSpecialization(this, record);
 		case CPP_TYPEDEF_SPECIALIZATION:
 			return new PDOMCPPTypedefSpecialization(this, record);
+		case CPP_USING_DECLARATION_SPECIALIZATION:
+			return new PDOMCPPUsingDeclarationSpecialization(this, record);
 		}
 		assert false : "nodeid= " + nodeType; //$NON-NLS-1$
 		return null;
@@ -884,7 +900,7 @@ class PDOMCPPLinkage extends PDOMLinkage implements IIndexCPPBindingConstants {
 				long rec= file.getLastUsingDirectiveRec();
 				PDOMCPPUsingDirective ud= new PDOMCPPUsingDirective(this, rec, containerNS,
 						pdomName.getBinding(), pdomName.getFileLocation().getNodeOffset());
-				file.setFirstUsingDirectiveRec(ud.getRecord());
+				file.setLastUsingDirective(ud.getRecord());
 			}
 		} else if (parentNode instanceof ICPPASTElaboratedTypeSpecifier) {
 			ICPPASTElaboratedTypeSpecifier elaboratedSpecifier = (ICPPASTElaboratedTypeSpecifier)parentNode;
@@ -922,9 +938,6 @@ class PDOMCPPLinkage extends PDOMLinkage implements IIndexCPPBindingConstants {
 		}
 	}
 
-	/* (non-Javadoc)
-	 * @see org.eclipse.cdt.internal.core.pdom.dom.PDOMLinkage#getUsingDirectives()
-	 */
 	@Override
 	public ICPPUsingDirective[] getUsingDirectives(PDOMFile file) throws CoreException {
 		long rec= file.getLastUsingDirectiveRec();
@@ -977,23 +990,23 @@ class PDOMCPPLinkage extends PDOMLinkage implements IIndexCPPBindingConstants {
 			final WritablePDOM wpdom= (WritablePDOM) pdom;
 			PDOMFile file= null;
 			if (binding instanceof ICPPUsingDeclaration) {
-				String path= ASTInternal.getDeclaredInOneFileOnly(binding);
-				if (path != null) {
-					file= wpdom.getFileForASTPath(getLinkageID(), path);
+				IASTNode node= ASTInternal.getDeclaredInOneFileOnly(binding);
+				if (node != null) {
+					file= wpdom.getFileForASTNode(getLinkageID(), node);
 				}
 			} else if (binding instanceof ICPPNamespaceAlias) {
-				String path= ASTInternal.getDeclaredInSourceFileOnly(binding, false, glob);
-				if (path != null) {
-					file= wpdom.getFileForASTPath(getLinkageID(), path);
+				IASTNode node= ASTInternal.getDeclaredInSourceFileOnly(getPDOM(), binding, false, glob);
+				if (node != null) {
+					file= wpdom.getFileForASTNode(getLinkageID(), node);
 				}
 			}
 			if (file == null && !(binding instanceof IIndexBinding)) {
 				IBinding owner= binding.getOwner();
 				if (owner instanceof ICPPNamespace) {
 					if (owner.getNameCharArray().length == 0) {
-						String path= ASTInternal.getDeclaredInSourceFileOnly(owner, false, glob);
-						if (path != null) {
-							file= wpdom.getFileForASTPath(getLinkageID(), path);
+						IASTNode node= ASTInternal.getDeclaredInSourceFileOnly(getPDOM(), owner, false, glob);
+						if (node != null) {
+							file= wpdom.getFileForASTNode(getLinkageID(), node);
 						}
 					}
 				}
@@ -1012,7 +1025,6 @@ class PDOMCPPLinkage extends PDOMLinkage implements IIndexCPPBindingConstants {
 	public PDOMBinding addTypeBinding(IBinding type) throws CoreException {
 		return addBinding(type, null);
 	}
-
 	
 	@Override
 	public IType unmarshalType(ITypeMarshalBuffer buffer) throws CoreException {
