@@ -270,7 +270,8 @@ public class CPPSelectionTestsNoIndexer extends BaseUITestCase {
             	final ITextSelection textSel = (ITextSelection)sel;
             	ITranslationUnit tu= CUIPlugin.getDefault().getWorkingCopyManager().getWorkingCopy(editor.getEditorInput());
         		IStatus ok= ASTProvider.getASTProvider().runOnAST(tu, ASTProvider.WAIT_IF_OPEN, monitor, new ASTRunnable() {
-        			public IStatus runOnAST(ILanguage language, IASTTranslationUnit ast) throws CoreException {
+        			@Override
+					public IStatus runOnAST(ILanguage language, IASTTranslationUnit ast) throws CoreException {
         				result[0]= ast.getNodeSelector(null).findName(textSel.getOffset(), textSel.getLength());        	
         				return Status.OK_STATUS;
         			}
@@ -992,7 +993,7 @@ public class CPPSelectionTestsNoIndexer extends BaseUITestCase {
     //  typedef int (functionPointerArray[2])(int);
     //  functionPointerArray fctVariablArray;
     public void testBug195822() throws Exception {
-    	StringBuffer[] contents= getContentsForTest(2);
+    	StringBuilder[] contents= getContentsForTest(2);
     	String code= contents[0].toString();
     	String appendCode= contents[1].toString();
 
