@@ -52,6 +52,9 @@ public final class PreferencesMessages extends NLS {
 	public static String CEditorPreferencePage_ContentAssistPage_proposalFilterSelect;
 	public static String CEditorPreferencePage_ContentAssistPage_completionProposalBackgroundColor;
 	public static String CEditorPreferencePage_ContentAssistPage_completionProposalForegroundColor;
+	public static String CEditorPreferencePage_ContentAssistPage_completionProposalTimeout;
+	public static String CEditorPreferencePage_ContentAssistPage_completionProposalTimeoutErrMsg;
+	public static String CEditorPreferencePage_ContentAssistPage_completionProposalTimeoutToolTip;
 	public static String CEditorPreferencePage_ContentAssistPage_parameterBackgroundColor;
 	public static String CEditorPreferencePage_ContentAssistPage_parameterForegroundColor;
 	public static String CEditorPreferencePage_ContentAssistPage_sortingSection_title;
@@ -148,35 +151,11 @@ public final class PreferencesMessages extends NLS {
 	public static String BuildLogPreferencePage_LogLocation;
 	public static String GlobalBuildLogPreferencePage_EnableLogging;
 	public static String GlobalBuildLogPreferencePage_LogLocation;
-	public static String CEditorPreferencePage_folding_title;
+	public static String FoldingPreferencePage_folding_title;
 	public static String FoldingConfigurationBlock_enable;
 	public static String FoldingConfigurationBlock_combo_caption;
 	public static String FoldingConfigurationBlock_info_no_preferences;
 	public static String FoldingConfigurationBlock_error_not_exist;
-	public static String PathEntryVariablePreference_explanation;
-	public static String PathEntryVariableDialog_shellTitle_newVariable;
-	public static String PathEntryVariableDialog_shellTitle_existingVariable;
-	public static String PathEntryVariableDialog_dialogTitle_newVariable;
-	public static String PathEntryVariableDialog_dialogTitle_existingVariable;
-	public static String PathEntryVariableDialog_message_newVariable;
-	public static String PathEntryVariableDialog_message_existingVariable;
-	public static String PathEntryVariableDialog_variableName;
-	public static String PathEntryVariableDialog_variableValue;
-	public static String PathEntryVariableDialog_variableNameEmptyMessage;
-	public static String PathEntryVariableDialog_variableValueEmptyMessage;
-	public static String PathEntryVariableDialog_variableValueInvalidMessage;
-	public static String PathEntryVariableDialog_file;
-	public static String PathEntryVariableDialog_folder;
-	public static String PathEntryVariableDialog_selectFileTitle;
-	public static String PathEntryVariableDialog_selectFolderTitle;
-	public static String PathEntryVariableDialog_selectFolderMessage;
-	public static String PathEntryVariableDialog_variableAlreadyExistsMessage;
-	public static String PathEntryVariableDialog_pathIsRelativeMessage;
-	public static String PathEntryVariableDialog_pathDoesNotExistMessage;
-	public static String PathEntryVariablesBlock_variablesLabel;
-	public static String PathEntryVariablesBlock_addVariableButton;
-	public static String PathEntryVariablesBlock_editVariableButton;
-	public static String PathEntryVariablesBlock_removeVariableButton;
 	public static String ProposalFilterPreferencesUtil_defaultFilterName;
 
 	public static String CEditorPreferencePage_typing_tabTitle;
@@ -215,6 +194,17 @@ public final class PreferencesMessages extends NLS {
 	public static String CodeFormatterPreferencePage_title;
 	public static String CodeFormatterPreferencePage_description;
 
+	public static String CodeStylePreferencePage_title;
+	public static String CodeStyleBlock_class_member_order;
+	public static String CodeStyleBlock_public_private;
+	public static String CodeStyleBlock_private_public;
+	public static String CodeStyleBlock_function_parameter_order;
+	public static String CodeStyleBlock_input_output;
+	public static String CodeStyleBlock_output_input;
+	public static String CodeStyleBlock_function_output_parameter_style;
+	public static String CodeStyleBlock_pass_by_reference;
+	public static String CodeStyleBlock_pass_by_pointer;
+
 	public static String TodoTaskPreferencePage_title;
 	public static String TodoTaskPreferencePage_description;
 	public static String TodoTaskConfigurationBlock_markers_tasks_high_priority;
@@ -383,6 +373,8 @@ public final class PreferencesMessages extends NLS {
 	public static String NameStyleBlock_variable_node_description;
 	public static String NameStyleBlock_field_node;
 	public static String NameStyleBlock_field_node_description;
+	public static String NameStyleBlock_method_node;
+	public static String NameStyleBlock_method_node_description;
 	public static String NameStyleBlock_getter_node;
 	public static String NameStyleBlock_getter_node_description;
 	public static String NameStyleBlock_setter_node;
