@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2008 Institute for Software, HSR Hochschule fuer Technik  
+ * Copyright (c) 2008, 2012 Institute for Software, HSR Hochschule fuer Technik  
  * Rapperswil, University of applied sciences and others
  * All rights reserved. This program and the accompanying materials 
  * are made available under the terms of the Eclipse Public License v1.0 
@@ -15,7 +15,7 @@ import org.eclipse.osgi.util.NLS;
 
 public final class Messages extends NLS {
 	public static String InputPage_ConstName;
-	public static String InputPage_EnterContName;
+	public static String InputPage_EnterConstName;
 	public static String InputPage_NameAlreadyDefined;
 	public static String ExtractConstantRefactoring_ExtractConst;
 	public static String ExtractConstantRefactoring_LiteralMustBeSelected;
