@@ -1,5 +1,5 @@
 /******************************************************************************* 
- * Copyright (c) 2005, 2011 Wind River Systems, Inc. and others.
+ * Copyright (c) 2005, 2012 Wind River Systems, Inc. and others.
  * All rights reserved. This program and the accompanying materials 
  * are made available under the terms of the Eclipse Public License v1.0 
  * which accompanies this distribution, and is available at 
@@ -97,7 +97,9 @@ import org.eclipse.cdt.core.dom.ast.cpp.ICPPNamespace;
 import org.eclipse.cdt.core.dom.ast.cpp.ICPPNamespaceScope;
 import org.eclipse.cdt.core.dom.ast.cpp.ICPPReferenceType;
 import org.eclipse.cdt.core.dom.ast.cpp.ICPPTemplateTypeParameter;
+import org.eclipse.cdt.core.dom.ast.cpp.ICPPUsingDeclaration;
 import org.eclipse.cdt.core.index.IIndex;
+import org.eclipse.cdt.core.index.IIndexBinding;
 import org.eclipse.cdt.core.index.IIndexName;
 import org.eclipse.cdt.core.model.CoreModel;
 import org.eclipse.cdt.core.model.ICElement;
@@ -183,13 +185,26 @@ public class ASTManager implements IDisposable {
      * Returns TRUE, FALSE or UNKNOWN.
      * @throws DOMException 
      */
-    public static int isSameBinding(IBinding b1, IBinding b2) throws DOMException {
+    public static int isSameBinding(IIndex index, IBinding b1, IBinding b2) throws DOMException {
         if (b1 == null || b2 == null) {
             return UNKNOWN;
         }
         if (b1.equals(b2)) {
             return TRUE;
         }
+        if (b1 instanceof IIndexBinding || b2 instanceof IIndexBinding) {
+        	if (index != null) {
+        		IIndexBinding b11 = index.adaptBinding(b1);
+        		if (b11 != null)
+        			b1= b11;
+        		IIndexBinding b21 = index.adaptBinding(b2);
+        		if (b21 != null)
+        			b2= b21;
+                if (b1.equals(b2))
+                	return TRUE;
+        	}
+        }
+        
         String n1= b1.getName();
         String n2= b2.getName();
         if (n1 == null || n2 == null) {
@@ -791,6 +806,7 @@ public class ASTManager implements IDisposable {
 	/**
 	 * @see IDisposable#dispose()
 	 */
+	@Override
 	public void dispose() {
         Assert.isTrue(!fDisposed, "ASTManager.dispose() called more than once"); //$NON-NLS-1$
 		fDisposed = true;
@@ -1191,18 +1207,29 @@ public class ASTManager implements IDisposable {
         	if (problemInQualifier) {
         		cmp= UNKNOWN;
         	} else {
-        		for (IBinding renameBinding : fValidBindings) {
-        			try {
-        				int cmp0= isSameBinding(binding, renameBinding);
-        				if (cmp0 != FALSE) {
-        					cmp= cmp0;
-        				}
-        				if (cmp0 == TRUE) {
-        					break;
-        				}
-        			} catch (DOMException e) {
-        				handleDOMException(name.getTranslationUnit(), e, status);
-        				cmp= UNKNOWN;
+    			final IASTTranslationUnit tu = name.getTranslationUnit();
+    			final IIndex index= tu != null ? tu.getIndex() : null;
+    			IBinding[] bindings = binding instanceof ICPPUsingDeclaration ?
+        				((ICPPUsingDeclaration) binding).getDelegates() : new IBinding[] { binding };
+   				// When a 'using' declaration has multiple delegate bindings and only some of them
+        		// are being renamed, to preserve correctness of the code we would have to split
+        		// the 'using' declaration into two separate ones. We currently don't do that and
+        		// rename the 'using' declaration if at least one of its delegate bindings is being
+        		// renamed.
+        		outer: for (IBinding b : bindings) {
+        			for (IBinding renameBinding : fValidBindings) {
+	        			try {
+	        				int cmp0= isSameBinding(index, b, renameBinding);
+	        				if (cmp0 != FALSE) {
+	        					cmp= cmp0;
+	        				}
+	        				if (cmp0 == TRUE) {
+	        					break outer;
+	        				}
+	        			} catch (DOMException e) {
+	        				handleDOMException(name.getTranslationUnit(), e, status);
+	        				cmp= UNKNOWN;
+	        			}
         			}
         		}
         	}
@@ -1486,7 +1513,7 @@ public class ASTManager implements IDisposable {
                 for (int i = 0; !isAboveOrEqual && i<newBindingsAboverOrEqual.length; i++) {
                     IBinding aboveBinding = newBindingsAboverOrEqual[i];
                     try {
-                        if (isSameBinding(aboveBinding, conflictingBinding) == TRUE) {
+                        if (isSameBinding(tu.getIndex(), aboveBinding, conflictingBinding) == TRUE) {
                             isAboveOrEqual= true;
                         }
                     } catch (DOMException e) {
