@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2008, 2011 Institute for Software, HSR Hochschule fuer Technik  
+ * Copyright (c) 2008, 2012 Institute for Software, HSR Hochschule fuer Technik  
  * Rapperswil, University of applied sciences and others
  * All rights reserved. This program and the accompanying materials 
  * are made available under the terms of the Eclipse Public License v1.0 
@@ -21,10 +21,8 @@ public final class Messages extends NLS {
 	public static String IdentifierHelper_emptyIdentifier;
 	public static String IdentifierHelper_illegalCharacter;
 	public static String IdentifierHelper_unidentifiedMistake;
-	public static String VisibilityEnum_public;
-	public static String VisibilityEnum_protected;
-	public static String VisibilityEnum_private;
-	public static String Checks_validateEdit;
+	public static String Checks_validate_edit;
+	public static String Checks_choose_name;
 
 	static {
 		NLS.initializeMessages(Messages.class.getName(), Messages.class);
