@@ -39,7 +39,6 @@ import org.eclipse.cdt.internal.corext.util.CodeFormatterUtil;
  * </p>
  */
 public final class CIndenter {
-	
 	/**
 	 * The CDT Core preferences.
 	 */
@@ -528,7 +527,7 @@ public final class CIndenter {
 		StringBuilder reference= getLeadingWhitespace(offset);
 		IRegion line= fDocument.getLineInformationOfOffset(offset);
 		String string= fDocument.get(line.getOffset(), offset - line.getOffset());
-		if (string.trim().length() == 0)
+		if (string.trim().isEmpty())
 			return reference;
 		// Add additional indent
 		return createReusingIndent(reference, fPrefs.prefContinuationIndent, 0);
@@ -887,7 +886,8 @@ public final class CIndenter {
 	 */
 	public int findReferencePosition(int offset, boolean danglingElse, boolean matchBrace, boolean matchParen,
 			boolean matchCase, boolean matchAccessSpecifier) {
-		fIndent= 0; // the indentation modification
+		fIndent= 0; // The indentation modification
+		fExtraSpaces= 0;
 		fAlign= CHeuristicScanner.NOT_FOUND;
 		fPosition= offset;
 
@@ -1587,6 +1587,7 @@ public final class CIndenter {
 	private int skipToPreviousListItemOrListStart() {
 		int startLine= fLine;
 		int startPosition= fPosition;
+		int linesSkippedInsideScopes = 0;
 		boolean continuationLineCandidate =
 				fToken == Symbols.TokenEQUAL || fToken == Symbols.TokenSHIFTLEFT ||
 				fToken == Symbols.TokenRPAREN;
@@ -1595,7 +1596,7 @@ public final class CIndenter {
 			nextToken();
 
 			// If any line item comes with its own indentation, adapt to it
-			if (fLine < startLine) {
+			if (fLine < startLine - linesSkippedInsideScopes) {
 				try {
 					int lineOffset= fDocument.getLineOffset(startLine);
 					int bound= Math.min(fDocument.getLength(), startPosition + 1);
@@ -1616,6 +1617,7 @@ public final class CIndenter {
 				return startPosition;
 			}
 
+			int line = fLine;
 			switch (fToken) {
 			// scopes: skip them
 			case Symbols.TokenRPAREN:
@@ -1624,6 +1626,7 @@ public final class CIndenter {
 			case Symbols.TokenRBRACKET:
 			case Symbols.TokenRBRACE:
 				skipScope();
+				linesSkippedInsideScopes = line - fLine;
 				break;
 
 			// scope introduction: special treat who special is
