@@ -8,6 +8,7 @@
  * Contributors:
  *     Wind River Systems - initial API and implementation
  *     Ericsson			  - Additional handling of events  	
+ *     Mikhail Khodjaiants (Mentor Graphics) - Refactor common code in GDBControl* classes (bug 372795)
  *******************************************************************************/
 package org.eclipse.cdt.dsf.mi.service.command;
 
@@ -23,10 +24,8 @@ import org.eclipse.cdt.dsf.debug.service.IRunControl.IContainerDMContext;
 import org.eclipse.cdt.dsf.debug.service.IRunControl.IExecutionDMContext;
 import org.eclipse.cdt.dsf.debug.service.command.ICommand;
 import org.eclipse.cdt.dsf.debug.service.command.ICommandControlService.ICommandControlDMContext;
-import org.eclipse.cdt.dsf.debug.service.command.ICommandListener;
 import org.eclipse.cdt.dsf.debug.service.command.ICommandResult;
 import org.eclipse.cdt.dsf.debug.service.command.ICommandToken;
-import org.eclipse.cdt.dsf.debug.service.command.IEventListener;
 import org.eclipse.cdt.dsf.gdb.internal.GdbPlugin;
 import org.eclipse.cdt.dsf.gdb.service.IGDBBackend;
 import org.eclipse.cdt.dsf.mi.service.IMIProcesses;
@@ -72,7 +71,7 @@ import org.eclipse.cdt.dsf.service.DsfServicesTracker;
  * received by other services and clients.
  */
 public class MIRunControlEventProcessor 
-    implements IEventListener, ICommandListener
+    implements IEventProcessor
 {
 	private static final String STOPPED_REASON = "stopped"; //$NON-NLS-1$
 	   
@@ -107,12 +106,14 @@ public class MIRunControlEventProcessor
     /**
      * This processor must be disposed before the control service is un-registered. 
      */
+    @Override
     public void dispose() {
         fCommandControl.removeEventListener(this);        
         fCommandControl.removeCommandListener(this);
         fServicesTracker.dispose();
     }
     
+	@Override
     public void eventReceived(Object output) {
     	for (MIOOBRecord oobr : ((MIOutput)output).getMIOOBRecords()) {
 			List<MIEvent<?>> events = new LinkedList<MIEvent<?>>();
@@ -294,18 +295,22 @@ public class MIRunControlEventProcessor
     	}
     }
     
+	@Override
     public void commandQueued(ICommandToken token) {
         // Do nothing.
     }
     
+	@Override
     public void commandSent(ICommandToken token) {
         // Do nothing.
     }
     
+	@Override
     public void commandRemoved(ICommandToken token) {
         // Do nothing.
     }
     
+	@Override
     public void commandDone(ICommandToken token, ICommandResult result) {
         ICommand<?> cmd = token.getCommand();
     	MIInfo cmdResult = (MIInfo) result ;
