@@ -8,18 +8,17 @@
  * Contributors:
  *     Ericsson		- Initial Implementation
  *******************************************************************************/
-package org.eclipse.cdt.tests.dsf.gdb.tests.tests_7_2;
+package org.eclipse.cdt.tests.dsf.gdb.tests.tests_6_8;
 
 import org.eclipse.cdt.tests.dsf.gdb.framework.BackgroundRunner;
 import org.eclipse.cdt.tests.dsf.gdb.tests.ITestConstants;
-import org.eclipse.cdt.tests.dsf.gdb.tests.PostMortemCoreTest;
-import org.junit.BeforeClass;
+import org.eclipse.cdt.tests.dsf.gdb.tests.tests_6_7.PostMortemCoreTest_6_7;
 import org.junit.runner.RunWith;
 
 @RunWith(BackgroundRunner.class)
-public class PostMortemCoreTest_7_2 extends PostMortemCoreTest {
-	@BeforeClass
-    public static void beforeClassMethod_7_2() {
-		setGdbProgramNamesLaunchAttributes(ITestConstants.SUFFIX_GDB_7_2);		
+public class PostMortemCoreTest_6_8 extends PostMortemCoreTest_6_7 {
+    @Override
+	protected void setGdbVersion() {
+		setGdbProgramNamesLaunchAttributes(ITestConstants.SUFFIX_GDB_6_8);		
 	}
 }
