@@ -1,26 +1,25 @@
 /*******************************************************************************
- * Copyright (c) 2011 Ericsson and others.
+ * Copyright (c) 2012 Ericsson and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors:
- *     Ericsson	AB		  - Initial implementation of Test cases
+ *     Marc Khouzam (Ericsson) - Initial implementation of Test cases
  *******************************************************************************/
-package org.eclipse.cdt.tests.dsf.gdb.tests.tests_7_0;
+package org.eclipse.cdt.tests.dsf.gdb.tests.tests_7_4;
 
 
 import org.eclipse.cdt.tests.dsf.gdb.framework.BackgroundRunner;
 import org.eclipse.cdt.tests.dsf.gdb.tests.ITestConstants;
-import org.eclipse.cdt.tests.dsf.gdb.tests.tests_6_8.MIRunControlTargetAvailableTest_6_8;
-import org.junit.BeforeClass;
+import org.eclipse.cdt.tests.dsf.gdb.tests.tests_7_3.MIRunControlTargetAvailableTest_7_3;
 import org.junit.runner.RunWith;
 
 @RunWith(BackgroundRunner.class)
-public class MIRunControlTargetAvailableTest_7_0 extends MIRunControlTargetAvailableTest_6_8 {
-	@BeforeClass
-    public static void beforeClassMethod_7_0() {
-		setGdbProgramNamesLaunchAttributes(ITestConstants.SUFFIX_GDB_7_0);
+public class MIRunControlTargetAvailableTest_7_4 extends MIRunControlTargetAvailableTest_7_3 {
+	@Override
+	protected void setGdbVersion() {
+		setGdbProgramNamesLaunchAttributes(ITestConstants.SUFFIX_GDB_7_4);
 	}
 }
