@@ -1,27 +1,32 @@
 /*******************************************************************************
- * Copyright (c) 2011 Ericsson and others.
+ * Copyright (c) 2012 Ericsson and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors:
- *     Ericsson	AB		  - Initial implementation of Test cases
+ *     Marc Khouzam (Ericsson) - Initial implementation of Test cases
  *******************************************************************************/
-package org.eclipse.cdt.tests.dsf.gdb.tests.tests_7_2;
+package org.eclipse.cdt.tests.dsf.gdb.tests.tests_7_5;
 
 
 import org.eclipse.cdt.dsf.gdb.IGDBLaunchConfigurationConstants;
 import org.eclipse.cdt.tests.dsf.gdb.framework.BackgroundRunner;
 import org.eclipse.cdt.tests.dsf.gdb.tests.ITestConstants;
-import org.junit.BeforeClass;
 import org.junit.runner.RunWith;
 
 @RunWith(BackgroundRunner.class)
-public class OperationsWhileTargetIsRunningNonStopTest_7_2 extends OperationsWhileTargetIsRunningTest_7_2 {
-	@BeforeClass
-    public static void beforeClassMethod_7_2() {
-		setGdbProgramNamesLaunchAttributes(ITestConstants.SUFFIX_GDB_7_2);
+public class OperationsWhileTargetIsRunningNonStopTest_7_5 extends OperationsWhileTargetIsRunningTest_7_5 {
+    @Override
+	protected void setGdbVersion() {
+		setGdbProgramNamesLaunchAttributes(ITestConstants.SUFFIX_GDB_7_5);
+	}
+	
+	@Override
+	protected void setLaunchAttributes() {
+		super.setLaunchAttributes();
+		
     	setLaunchAttribute(IGDBLaunchConfigurationConstants.ATTR_DEBUGGER_NON_STOP, true);
 	}
 }
